@@ -27,6 +27,7 @@
 #include "ipx.h"
 #include "pktdrvr.h"
 #include "iodev.h"
+#include "debug.h"
 
 #include "keyb_clients.h"
 #include "keyb_server.h"
@@ -93,8 +94,10 @@ static void cleanup_child(void)
   restore_eflags_fs_gs();
   if (portserver_pid &&
       waitpid(portserver_pid, &status, WNOHANG) > 0 &&
-      WIFSIGNALED(status))
+      WIFSIGNALED(status)) {
+    gdb_debug();
     leavedos(1);
+  }
 }
 
 static void leavedos_signal(int sig)
@@ -189,7 +192,7 @@ void
 signal_init(void)
 {
   struct sigaction sa;
-  sigset_t trashset;
+  sigset_t trashset, set;
 
   save_eflags_fs_gs();
 
@@ -282,6 +285,11 @@ signal_init(void)
   NEWSETSIG(SIGSEGV, dosemu_fault);
   SETSIG(SIGCHLD, cleanup_child);
 
+  /* unblock SIGIO, SIGALRM, SIG_ACQUIRE, SIG_RELEASE */
+  sigemptyset(&set);
+  ADDSET_SIGNALS_THAT_QUEUE(&set);
+  sigprocmask(SIG_UNBLOCK, &set, NULL);
+
   SIG_init();			/* silly int generator support */
 }
 
