@@ -1,6 +1,6 @@
 /* 
  * All modifications in this file to the original code are
- * (C) Copyright 1992, ..., 2001 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2004 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -20,9 +20,11 @@
  *		- better readable output
  */
 
+#include "config.h"
 #include <stdio.h>
 #include <stdarg.h>
 #include <string.h>
+#include "dis8086.h"
 
 #ifdef __GNUC__
 #define INLINE inline
@@ -188,16 +190,25 @@ static const char *loop_codes[] =
   "loopne", "loope ", "loop  ", "jcxz  "
 };
 
-#if defined(i386) || defined(vax)
+#if defined(i386) || defined(vax) /* || defined(alpha) */
 /* Little-endian machines which allow unaligned addresses.  */
 #define IMMED16(code) *(unsigned short *)(code)
 #define IMMED32(code) *(unsigned int *)(code)
 #define IMMED32I(code) *(int *)(code)
 #else
 /* Big-endian or strict alignment.  */
-#define IMMED16(code) code[0] | (code[1] << 8)
-#define IMMED32(code) code[0] | (code[1] << 8) | (code[2] << 16) | (code[3] << 24)
-#define IMMED32I(code) code[0] | (code[1] << 8) | (code[2] << 16) | (code[3] << 24)
+#ifndef __GNUC__
+#error needs gcc
+#endif
+#define IMMED16(code)	({const unsigned char *p=(code); unsigned short v;\
+				v=p[0]|((unsigned short)(p[1])<<8); v; })
+#define IMMED32(code)	({const unsigned char *p=(code); unsigned int v;\
+				v=p[0]|((unsigned int)(p[1])<<8)|\
+				((unsigned int)(p[2])<<16)|\
+				((unsigned int)(p[3])<<24); v; })
+#define IMMED32I(code)	({const unsigned char *p=(code); int v;\
+				v=p[0]|((int)(p[1])<<8)|\
+				((int)(p[2])<<16)|((int)(p[3])<<24); v; })
 #endif
 
 #undef index
@@ -246,42 +257,42 @@ static INLINE unsigned int   resolva (unsigned int addr)
 
 static unsigned char *modrm_disp32(int d)
 {
-   static char buf[10];
+   static char buf[16];
 
    if (d == 0)
      buf[0] = 0;
    else if (d > 0)
-     sprintf(buf, "+%08X", d);
+     sprintf(buf, "+%#08x", d);
    else
-     sprintf(buf, "-%08X", (-d));
+     sprintf(buf, "-%#08x", (-d));
 
    return buf;
 }
 
 static unsigned char *modrm_disp16(int d)
 {
-   static char buf[5];
+   static char buf[10];
 
    if (d == 0)
      buf[0] = 0;
    else if (d > 0)
-     sprintf(buf, "+%04X", d);
+     sprintf(buf, "+%#04x", d);
    else
-     sprintf(buf, "-%04X", (-d));
+     sprintf(buf, "-%#04x", (-d));
 
    return buf;
 }
 
 static unsigned char *modrm_disp8(int d)
 {
-   static char buf[4];
+   static char buf[8];
 
    if (d == 0)
      buf[0] = 0;
    else if (d > 0)
-     sprintf(buf, "+%02X", d);
+     sprintf(buf, "+%#02x", d);
    else
-     sprintf(buf, "-%02X", (-d));
+     sprintf(buf, "-%#02x", (-d));
 
    return buf;
 }
@@ -311,7 +322,7 @@ unsigned char hex32[9];
     {
         if (rm == 5)
         {
-            d86_printf("%s[%08X]", seg, immed32(code));
+            d86_printf("%s[%08x]", seg, immed32(code));
             return code+4;
         }
         if (rm != 4)
@@ -322,7 +333,7 @@ unsigned char hex32[9];
 
         if (base == 5)
         {
-            sprintf(hex32, "%08X", immed32(code));
+            sprintf(hex32, "%08x", immed32(code));
             basestr = hex32;
             code += 4;
         }
@@ -378,7 +389,7 @@ unsigned char rm  = *code & 7;
 
   if (rm == 6)
   {
-      d86_printf("%s[%04X]", seg, immed16(code));
+      d86_printf("%s[%04x]", seg, immed16(code));
       return code+2;
   }
   else
@@ -402,7 +413,7 @@ char *nw = 0;
       if (code[0] & 1)
           d86_printf("st(%d)",reg);
 
-      else if (reg == 1) ; /*((code[1] & 0x30) == 0x10) ;*/
+//      else if (reg == 1) ; /*((code[1] & 0x30) == 0x10) ;*/
 
       else if (code[0] & 4)
           d86_printf("st(%d),st",reg);
@@ -482,7 +493,7 @@ static INLINE const unsigned char *esc_op(const unsigned char opcode,
 				int addr32)
 {
 
-  d86_printf("esc     %02X,", ((opcode & 7) << 3) | ((*code >> 3) & 7));
+  d86_printf("esc     %02x,", ((opcode & 7) << 3) | ((*code >> 3) & 7));
   return mod_rm(code, seg, regs, addr32);
 }
 
@@ -577,7 +588,7 @@ int  dis_8086(unsigned int org,
 	case 0x2c:
 	case 0x34:
 	case 0x3c:
-	  d86_printf("%s     al,%02X", arith[opcode >> 3],
+	  d86_printf("%s     al,%#02x", arith[opcode >> 3],
 		  (unsigned char)*code++);
 	  break;
 
@@ -591,12 +602,12 @@ int  dis_8086(unsigned int org,
 	case 0x3d:
 	  if (data32)
 	    {
-	      d86_printf("%s     eax,%08X", arith[opcode >> 3], immed32(code));
+	      d86_printf("%s     eax,%08x", arith[opcode >> 3], immed32(code));
 	      code += 4;
 	    }
 	  else
 	    {
-	      d86_printf("%s     ax,%04X", arith[opcode >> 3], immed16(code));
+	      d86_printf("%s     ax,%04x", arith[opcode >> 3], immed16(code));
 	      code += 2;
 	    }
 	  break;
@@ -691,11 +702,11 @@ int  dis_8086(unsigned int org,
 	    case 0x8e:
 	    case 0x8f:
 	      if (addr32) {
-		d86_printf("j%s     %08X", conditions[opcode & 15],
+		d86_printf("j%s     %08x", conditions[opcode & 15],
 			resolva((code-code0)+org + 4 + IMMED32I(code)));
 		code += 4;
 	      } else {
-		d86_printf("j%s     %04X", conditions[opcode & 15],
+		d86_printf("j%s     %04x", conditions[opcode & 15],
 			(code-code0)+org + 2 + (short)(code[0] + (code[1] << 8)));
 		code += 2;
 	      }
@@ -744,7 +755,7 @@ int  dis_8086(unsigned int org,
 	    case 0xa4:
 	      d86_printf("shld    ");
 	      code = mod_reg_rm_r(code, seg, wreg, wreg, addr32);
-	      d86_printf(",%02X", *code++);
+	      d86_printf(",%#02x", *code++);
 	      break;
 
 	    case 0xa5:
@@ -764,7 +775,7 @@ int  dis_8086(unsigned int org,
 	    case 0xac:
 	      d86_printf("shrd    ");
 	      code = mod_reg_rm_r(code, seg, wreg, wreg, addr32);
-	      d86_printf(",%02X", *code++);
+	      d86_printf(",%#02x", *code++);
 	      break;
 
 	    case 0xad:
@@ -813,7 +824,7 @@ int  dis_8086(unsigned int org,
 	    case 0xb7:
 	      d86_printf("movzx   %s,", wreg[(*code >> 3) & 7]);
 	      if ((*code & 0xc0) != 0xc0)
-	        d86_printf("word ptr ");
+	        d86_printf("%cword ptr ",(data32? 'd':' '));
 	      code = mod_rm(code, seg, reg16, addr32);
 	      break;
 	      	
@@ -821,7 +832,7 @@ int  dis_8086(unsigned int org,
 	      d86_printf(bt_codes[(*code >> 3) & 3]);
 	      d86_printf("     ");
 	      code = mod_rm(code, seg, wreg, addr32);
-	      d86_printf(",%02X", *code++);
+	      d86_printf(",%#02x", *code++);
 	      break;
 
 	    case 0xbc:
@@ -844,7 +855,7 @@ int  dis_8086(unsigned int org,
 	    case 0xbf:
 	      d86_printf("movsx   %s,", wreg[(*code >> 3) & 7]);
 	      if ((*code & 0xc0) != 0xc0)
-	        d86_printf("word ptr ");
+	        d86_printf("%cword ptr ",(data32? 'd':' '));
 	      code = mod_rm(code, seg, reg16, addr32);
 	      break;
 
@@ -870,7 +881,7 @@ int  dis_8086(unsigned int org,
 	      break;
 
 	    default:	/* Maybe better to just say "db 0F"? - ARA */
-	      d86_printf("ESC 0F %02X", opcode);
+	      d86_printf("ESC 0F %02x", opcode);
 	      break;
 	    }
 	  break;
@@ -1000,24 +1011,24 @@ int  dis_8086(unsigned int org,
 	  continue;
 
 	case 0x66:
-	  data32 = !def_size;
+	  data32 = ((def_size & 2)==0);
 	  prefix = 1; i--;
 	  continue;
 
 	case 0x67:
-	  addr32 = !def_size;
+	  addr32 = ((def_size & 1)==0);
 	  prefix = 1; i--;
 	  continue;
 
 	case 0x68:
 	  if (data32)
 	    {
-	      d86_printf("push    %08X", immed32(code));
+	      d86_printf("push    %08x", immed32(code));
 	      code += 4;
 	    }
 	  else
 	    {
-	      d86_printf("push    %04X", immed16(code));
+	      d86_printf("push    %04x", immed16(code));
 	      code += 2;
 	    }
 	  break;
@@ -1027,27 +1038,27 @@ int  dis_8086(unsigned int org,
 	  code = mod_reg_rm(code, seg, wreg, wreg, addr32);
 	  if (data32)
 	    {
-	      d86_printf(",%08X", IMMED32(code));
+	      d86_printf(",%08x", IMMED32(code));
 	      code += 4;
 	    }
 	  else
 	    {
-	      d86_printf(",%04X", IMMED16(code));
+	      d86_printf(",%04x", IMMED16(code));
 	      code += 2;
 	    }
 	  break;
 
 	case 0x6a:
 	  if (data32)
-	      d86_printf("push    %08X", (signed char)*code++);
+	      d86_printf("push    %08x", (signed char)*code++);
 	  else
-	      d86_printf("push    %04X", (signed char)*code++);
+	      d86_printf("push    %04x", (signed char)*code++);
 	  break;
 
 	case 0x6b:
 	  d86_printf("imul    ");
 	  code = mod_reg_rm(code, seg, wreg, wreg, addr32);
-	  d86_printf(",%02X", (unsigned char)*code++);
+	  d86_printf(",%#02x", (unsigned char)*code++);
 	  break;
 
 	case 0x6c:
@@ -1082,9 +1093,9 @@ int  dis_8086(unsigned int org,
 	case 0x7d:
 	case 0x7e:
 	case 0x7f:
-	  if (addr32) d86_printf("j%s     %08X", conditions[opcode & 15],
+	  if (addr32) d86_printf("j%s     %08x", conditions[opcode & 15],
 		  SEG32REL(resolva((code-code0)+org + (signed char)*code + 1)));
-	  else d86_printf("j%s     %04X", conditions[opcode & 15],
+	  else d86_printf("j%s     %04x", conditions[opcode & 15],
 	          (unsigned short)SEG16REL(resolva((code-code0)+org + (signed char)*code + 1)));
 	  code++;
 	  break;
@@ -1094,7 +1105,7 @@ int  dis_8086(unsigned int org,
 	  d86_printf(arith[(*code >> 3) & 7]);
 	  d86_printf("     ");
 	  code = mod_rm(code, seg, reg8, addr32);
-	  d86_printf(",%02X", (unsigned char)*code++);
+	  d86_printf(",%#02x", (unsigned char)*code++);
 	  break;
 
 	case 0x81:
@@ -1103,12 +1114,12 @@ int  dis_8086(unsigned int org,
 	  code = mod_rm(code, seg, wreg, addr32);
 	  if (data32)
 	    {
-	      d86_printf(",%08X", immed32(code));
+	      d86_printf(",%08x", immed32(code));
 	      code += 4;
 	    }
 	  else
 	    {
-	      d86_printf(",%04X", immed16(code));
+	      d86_printf(",%04x", immed16(code));
 	      code += 2;
 	    }
 	  break;
@@ -1118,9 +1129,9 @@ int  dis_8086(unsigned int org,
 	  d86_printf("     ");
 	  code = mod_rm(code, seg, wreg, addr32);
 	  if (data32)
-	      d86_printf(",%08X", (signed char)*code++);
+	      d86_printf(",%08x", (signed char)*code++);
 	  else
-	      d86_printf(",%04X", (unsigned short)((signed char)*code++));
+	      d86_printf(",%04x", (unsigned short)((signed char)*code++));
 	  break;
 
 	case 0x84:
@@ -1182,7 +1193,7 @@ int  dis_8086(unsigned int org,
 	case 0x8f:
 	  d86_printf("pop     ");
 	  if ((*code & 0xc0) != 0xc0)
-	    d86_printf("%sword ptr ", data32 ? "d" : "");
+	    d86_printf("%cword ptr ",(data32? 'd':' '));
 	  code = mod_rm(code, seg, wreg, addr32);
 	  break;
 
@@ -1211,13 +1222,13 @@ int  dis_8086(unsigned int org,
 	case 0x9a:
 	  if (data32)
 	    {
-	      d86_printf("call    %04X:%08X", IMMED16(code+4),
+	      d86_printf("call    %04x:%08x", IMMED16(code+4),
 	      	  IMMED32(code));
 	      code += 6;
 	    }
 	  else
 	    {
-	      d86_printf("call    %04X:%04X", IMMED16(code+2),
+	      d86_printf("call    %04x:%04x", IMMED16(code+2),
 		  IMMED16(code));
 	      resolva( makeaddr( IMMED16(code+2), IMMED16(code)));
 	      code += 4;
@@ -1229,11 +1240,11 @@ int  dis_8086(unsigned int org,
 	  break;
 
 	case 0x9c:
-	  d86_printf("pushf");
+	  d86_printf("pushf%s", data32 ? "d" : "");
 	  break;
 
 	case 0x9d:
-	  d86_printf("popf");
+	  d86_printf("popf%s", data32 ? "d" : "");
 	  break;
 
 	case 0x9e:
@@ -1246,40 +1257,40 @@ int  dis_8086(unsigned int org,
 
 	case 0xa0:
 	  if (addr32) {
-	    d86_printf("mov     al,%s[%08X]", seg, immed32(code));
+	    d86_printf("mov     al,%s[%08x]", seg, immed32(code));
 	    code += 4;
 	  } else {
-	    d86_printf("mov     al,%s[%04X]", seg, immed16(code));
+	    d86_printf("mov     al,%s[%04x]", seg, immed16(code));
 	    code += 2;
 	  }
 	  break;
 
 	case 0xa1:
 	  if (addr32) {
-	    d86_printf("mov     %s,%s[%08X]", wreg[0], seg, immed32(code));
+	    d86_printf("mov     %s,%s[%08x]", wreg[0], seg, immed32(code));
 	    code += 4;
 	  } else {
-	    d86_printf("mov     %s,%s[%04X]", wreg[0], seg, immed16(code));
+	    d86_printf("mov     %s,%s[%04x]", wreg[0], seg, immed16(code));
 	    code += 2;
 	  }
 	  break;
 
 	case 0xa2:
 	  if (addr32) {
-	    d86_printf("mov     %s[%08X],al", seg, immed32(code));
+	    d86_printf("mov     %s[%08x],al", seg, immed32(code));
 	    code += 4;
 	  } else {
-	    d86_printf("mov     %s[%04X],al", seg, immed16(code));
+	    d86_printf("mov     %s[%04x],al", seg, immed16(code));
 	    code += 2;
 	  }
 	  break;
 
 	case 0xa3:
 	  if (addr32) {
-	    d86_printf("mov     %s[%08X],%s", seg, immed32(code), wreg[0]);
+	    d86_printf("mov     %s[%08x],%s", seg, immed32(code), wreg[0]);
 	    code += 4;
 	  } else {
-	    d86_printf("mov     %s[%04X],%s", seg, immed16(code), wreg[0]);
+	    d86_printf("mov     %s[%04x],%s", seg, immed16(code), wreg[0]);
 	    code += 2;
 	  }
 	  break;
@@ -1317,18 +1328,18 @@ int  dis_8086(unsigned int org,
 	  break;
 
 	case 0xa8:
-	  d86_printf("test    al,%02X", *code++);
+	  d86_printf("test    al,%#02x", *code++);
 	  break;
 
 	case 0xa9:
 	  if (data32)
 	    {
-	      d86_printf("test    eax,%08X", immed32(code));
+	      d86_printf("test    eax,%08x", immed32(code));
 	      code += 4;
 	    }
 	  else
 	    {
-	      d86_printf("test    ax,%04X", immed16(code));
+	      d86_printf("test    ax,%04x", immed16(code));
 	      code += 2;
 	    }
 	  break;
@@ -1372,7 +1383,7 @@ int  dis_8086(unsigned int org,
 	case 0xb5:
 	case 0xb6:
 	case 0xb7:
-	  d86_printf("mov     %s,%02X", reg8[opcode & 7], *code++);
+	  d86_printf("mov     %s,%#02x", reg8[opcode & 7], *code++);
 	  break;
 
 	case 0xb8:
@@ -1385,12 +1396,12 @@ int  dis_8086(unsigned int org,
 	case 0xbf:
 	  if (data32)
 	    {
-	      d86_printf("mov     %s,%08X", wreg[opcode & 7], immed32(code));
+	      d86_printf("mov     %s,%08x", wreg[opcode & 7], immed32(code));
 	      code += 4;
 	    }
 	  else
 	    {
-	      d86_printf("mov     %s,%04X", wreg[opcode & 7], immed16(code));
+	      d86_printf("mov     %s,%04x", wreg[opcode & 7], immed16(code));
 	      code += 2;
 	    }
 	  break;
@@ -1401,20 +1412,20 @@ int  dis_8086(unsigned int org,
 	  if ((*code & 0xc0) != 0xc0)
 	      d86_printf("byte ptr ");
 	  code = mod_rm(code, seg, reg8, addr32);
-	  d86_printf(",%02X", 31 & *code++);
+	  d86_printf(",%d", 31 & *code++);
 	  break;
 
 	case 0xc1:
 	  d86_printf(shift[(*code >> 3) & 7]);
 	  d86_printf("     ");
 	  if ((*code & 0xc0) != 0xc0)
-	      d86_printf("%sword ptr ", data32 ? "d" : "");
+	      d86_printf("%cword ptr ",(data32? 'd':' '));
 	  code = mod_rm(code, seg, wreg, addr32);
-	  d86_printf(",%02X", 31 & *code++);
+	  d86_printf(",%d", 31 & *code++);
 	  break;
 
 	case 0xc2:
-	  d86_printf("ret     %04X", IMMED16(code));
+	  d86_printf("ret     %04x", IMMED16(code));
 	  code += 2;
 	  break;
 
@@ -1435,7 +1446,7 @@ int  dis_8086(unsigned int org,
 	case 0xc6:
 	  d86_printf("mov     ");
 	  code = mod_rm(code, seg, reg8, addr32);
-	  d86_printf(",%02X", (unsigned char)*code++);
+	  d86_printf(",%#02x", (unsigned char)*code++);
 	  break;
 
 	case 0xc7:
@@ -1443,20 +1454,20 @@ int  dis_8086(unsigned int org,
 	  code = mod_rm(code, seg, wreg, addr32);
 	  if (data32)
 	    {
-	      d86_printf(",%08X", immed32(code));
+	      d86_printf(",%08x", immed32(code));
 	      code += 4;
 	    }
 	  else
 	    {
-	      d86_printf(",%04X", immed16(code));
+	      d86_printf(",%04x", immed16(code));
 	      code += 2;
 	    }
 	  break;
 
 	case 0xc8:
-	  d86_printf("enter   %04X",IMMED16(code));
+	  d86_printf("enter   %04x",IMMED16(code));
 	  code+=2;
-	  d86_printf(",%02X", *code++);
+	  d86_printf(",%#02x", *code++);
 	
 	  break;
 
@@ -1465,7 +1476,7 @@ int  dis_8086(unsigned int org,
 	  break;
 
 	case 0xca:
-	  d86_printf("retf    %04X", IMMED16(code));
+	  d86_printf("retf    %04x", IMMED16(code));
 	  code += 2;
 	  break;
 
@@ -1478,7 +1489,7 @@ int  dis_8086(unsigned int org,
 	  break;
 
 	case 0xcd:
-	  d86_printf("int     %02X", *code++);
+	  d86_printf("int     %#02x", *code++);
 	  break;
 
 	case 0xce:
@@ -1486,7 +1497,7 @@ int  dis_8086(unsigned int org,
 	  break;
 
 	case 0xcf:
-	  d86_printf("iret");
+	  d86_printf("iret%s", data32 ? "d" : "");
 	  break;
 
 	case 0xd0:
@@ -1502,7 +1513,7 @@ int  dis_8086(unsigned int org,
 	  d86_printf(shift[(*code >> 3) & 7]);
 	  d86_printf("     ");
 	  if ((*code & 0xc0) != 0xc0)
-	      d86_printf("%sword ptr ", data32 ? "d" : "");
+	      d86_printf("%cword ptr ",(data32? 'd':' '));
 	  code = mod_rm(code, seg, wreg, addr32);
 	  d86_printf(",1");
 	  break;
@@ -1520,17 +1531,17 @@ int  dis_8086(unsigned int org,
 	  d86_printf(shift[(*code >> 3) & 7]);
 	  d86_printf("     ");
 	  if ((*code & 0xc0) != 0xc0)
-	      d86_printf("%sword ptr ", data32 ? "d" : "");
+	      d86_printf("%cword ptr ",(data32? 'd':' '));
 	  code = mod_rm(code, seg, wreg, addr32);
 	  d86_printf(",cl");
 	  break;
 
 	case 0xd4:
-	  d86_printf("aam     %02X", *code++);
+	  d86_printf("aam     %02d", *code++);
 	  break;
 
 	case 0xd5:
-	  d86_printf("aad     %02X", *code++);
+	  d86_printf("aad     %02d", *code++);
 	  break;
 	
 	case 0xd6:
@@ -1654,39 +1665,39 @@ int  dis_8086(unsigned int org,
 	case 0xe2:
 	case 0xe3:
 	  if (addr32)
-              d86_printf("%s  %08X",
+              d86_printf("%s  %08x",
                          loop_codes[opcode & 3],
                          SEG32REL((code-code0)+org + (signed char)*code + 1));
 	  else
-              d86_printf("%s  %04X",
+              d86_printf("%s  %04x",
                          loop_codes[opcode & 3],
                          SEG16REL((code-code0)+org + (signed char)*code + 1));
 	  code++;
 	  break;
 
 	case 0xe4:
-	  d86_printf("in      al,%02X", *code++);
+	  d86_printf("in      al,%#02x", *code++);
 	  break;
 
 	case 0xe5:
-	  d86_printf("in      ax,%02X", *code++);
+	  d86_printf("in      ax,%#02x", *code++);
 	  break;
 
 	case 0xe6:
-	  d86_printf("out     %02X,al", *code++);
+	  d86_printf("out     %#02x,al", *code++);
 	  break;
 
 	case 0xe7:
-	  d86_printf("out     %02X,ax", *code++);
+	  d86_printf("out     %#02x,ax", *code++);
 	  break;
 
 	case 0xe8:
 	  if (addr32) {
-	    d86_printf("call    %08X",
+	    d86_printf("call    %08x",
 		    SEG32REL(resolva((code-code0)+org + 4 + IMMED32I(code))));
 	    code += 4;
 	  } else {
-	    d86_printf("call    %04X",
+	    d86_printf("call    %04x",
 		    SEG16REL(resolva((code-code0)+org + 2 + (code[0] | (code[1] << 8)))) & 0xffff);
 
 	    code += 2;
@@ -1695,11 +1706,11 @@ int  dis_8086(unsigned int org,
 
 	case 0xe9:
 	  if (data32) {
-	    d86_printf("jmp     %08X",
+	    d86_printf("jmp     %08x",
 		    SEG32REL(resolva((code-code0)+org + 4 + IMMED32I(code))));
 	    code += 4;
 	  } else {
-	    d86_printf("jmp     %04X",
+	    d86_printf("jmp     %04x",
 		    SEG16REL((unsigned short) ((code-code0)+org + 2 + (code[0] | (code[1] << 8)))) & 0xffff);
 	    code += 2;
 	  }
@@ -1707,11 +1718,11 @@ int  dis_8086(unsigned int org,
 
 	case 0xea:
 	  if (data32) {
-	    d86_printf("jmp     %04X:%08X", immed16(code+4),
+	    d86_printf("jmp     %04x:%08x", immed16(code+4),
 		    immed32i(code));
 	    code += 6;
 	  } else {
-	    d86_printf("jmp     %04X:%04X", IMMED16(code+2),
+	    d86_printf("jmp     %04x:%04x", IMMED16(code+2),
 		    IMMED16(code));
 	    resolva( makeaddr( IMMED16(code+2), IMMED16(code)));
 	    code += 4;
@@ -1719,8 +1730,8 @@ int  dis_8086(unsigned int org,
 	  break;
 
 	case 0xeb:
-	  if (data32) d86_printf("jmp     %08X", SEG32REL(resolva((code-code0)+org + (signed char)*code + 1)));
-	  else d86_printf("jmp     %04X", SEG16REL(resolva((code-code0)+org + (signed char)*code + 1)));
+	  if (data32) d86_printf("jmp     %08x", SEG32REL(resolva((code-code0)+org + (signed char)*code + 1)));
+	  else d86_printf("jmp     %04x", SEG16REL(resolva((code-code0)+org + (signed char)*code + 1)));
 	  code++;
 	  break;
 
@@ -1771,7 +1782,7 @@ int  dis_8086(unsigned int org,
 	    d86_printf("byte ptr ");
 	  code = mod_rm(code, seg, reg8, addr32);
 	  if ((opcode & 0x38) == 0)
-	    d86_printf(",%02X", *code++);
+	    d86_printf(",%#02x", *code++);
 	  break;
 
 	case 0xf7:
@@ -1779,18 +1790,18 @@ int  dis_8086(unsigned int org,
 	  d86_printf(f7_code[(*code >> 3) & 7]);
 	  d86_printf("    ");
 	  if (((*code & 0xc0) != 0xc0) && (*code & 0x38))
-	      d86_printf("%sword ptr ", data32 ? "d" : "");
+	      d86_printf("%cword ptr ",(data32? 'd':' '));
 	  code = mod_rm(code, seg, wreg, addr32);
 	  if ((opcode & 0x38) == 0)
 	    {
 	      if (data32)
 		{
-		  d86_printf(",%08X", immed32(code));
+		  d86_printf(",%08x", immed32(code));
 		  code += 4;
 		}
 	      else
 		{
-		  d86_printf(",%04X", immed16(code));
+		  d86_printf(",%04x", immed16(code));
 		  code += 2;
 		}
 	    }
@@ -1823,7 +1834,7 @@ int  dis_8086(unsigned int org,
 	case 0xfe:
 	  if (*code & 0x30)
 	    {
-	      d86_printf("db      %02X", opcode);
+	      d86_printf("db      %#02x", opcode);
 	      break;
 	    }
 	  d86_printf( ((*code & 0x08) ? "dec     " : "inc     ") );
@@ -1838,12 +1849,12 @@ int  dis_8086(unsigned int org,
 	  d86_printf(ff_code[(*code >> 3) & 7]);
 	  if (((*code & 0xc0) != 0xc0) &&
 	      (((*code & 0x30) == 0) || ((*code & 0x30) == 0x30)))
-	      d86_printf("%sword ptr ", data32 ? "d" : "");
+	      d86_printf("%cword ptr ",(data32? 'd':' '));
 	  code = mod_rm(code, seg, wreg, addr32);
 	  break;
 
 	default:
-	  d86_printf("db      %02X", opcode);
+	  d86_printf("db      %#02x", opcode);
 	  break;
 	}
       seg = "";
