@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 1999 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2004 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -17,6 +17,7 @@
  *   08Jan98 Hans Lermen <lermen@elserv.ffm.fgan.de>
  */
 
+#include "config.h"
 #include <stdio.h>
 #include <stdarg.h>
 #include <stdlib.h>
@@ -27,36 +28,28 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 
-#if 0
-#include <sys/socket.h>
-#include <netinet/in.h>
-#endif
 #include "bitops.h"
-
-#include "config.h"
 #include "emu.h"
 #include "cpu.h"
+#include "bios.h"
 #include "shared.h"
 #include "dpmi.h"
+#include "timers.h"
 
 #define MHP_PRIVATE
 #include "mhpdbg.h"
-#include "priv.h"
-
-extern void handle_signals(void);
 
 #if 0
 /* NOTE: the below is already defined with #include "emu.h"
- *       Must NOT redefine it, else REQUIRES_VM86PLUS won't work !!!
+ *       Must NOT redefine it, else vm86plus won't work !!!
  */
 extern struct vm86_struct vm86s;
 #endif
 
+static void vmhp_printf(const char *fmt, va_list args);
 static void mhp_poll(void);
 static void mhp_puts(char*);
 void mhp_putc(char);
-extern int mhp_getcsip_value();
-extern int mhp_modify_eip(int delta);
 
 static char mhp_banner[] = {
   "\nDOSEMU Debugger V0.6 connected\n"
@@ -64,10 +57,6 @@ static char mhp_banner[] = {
 };
 struct mhpdbgc mhpdbgc ={0};
 
-extern int traceloop;
-extern char loopbuf[];
-static void vmhp_printf(const char *fmt, va_list args);
-
 /********/
 /* CODE */
 /********/
@@ -111,20 +100,19 @@ void mhp_send(void)
    }
 }
 
-static  char pipename_in[128], pipename_out[128];
+static  char *pipename_in, *pipename_out;
 
 void mhp_close(void)
 {
-   PRIV_SAVE_AREA
    if (mhpdbg.fdin == -1) return;
    if (mhpdbg.active) {
      mhp_putc(1); /* tell debugger terminal to also quit */
      mhp_send();
    }
-   enter_priv_on();
    unlink(pipename_in);
+   free(pipename_in);
    unlink(pipename_out);   
-   leave_priv_setting();
+   free(pipename_out);
    mhpdbg.fdin = mhpdbg.fdout = -1;
    mhpdbg.active = 0;
    vm86s.vm86plus.vm86dbg_active = 0;
@@ -141,7 +129,6 @@ int vmhp_log_intercept(int flg, const char *fmt, va_list args)
       mhp_send();
     }
     if (dosdebug_flags & DBGF_LOG_TO_BREAK){
-      extern void mhp_regex(const char *fmt, va_list args);
       mhp_regex(fmt, args);
     }
   }
@@ -150,7 +137,6 @@ int vmhp_log_intercept(int flg, const char *fmt, va_list args)
 
 static void mhp_init(void)
 {
-  PRIV_SAVE_AREA
   int retval;
 
   mhpdbg.fdin = mhpdbg.fdout = -1;
@@ -161,28 +147,18 @@ static void mhp_init(void)
   vm86s.vm86plus.vm86dbg_TFpendig = 0;
   memset(&vm86s.vm86plus.vm86dbg_intxxtab, 0, sizeof(vm86s.vm86plus.vm86dbg_intxxtab));
 
-  sprintf(pipename_in, "%sdbgin.%d", TMPFILE, getpid());
-  enter_priv_on();
+  asprintf(&pipename_in, "%s/dosemu.dbgin.%d", RUNDIR, getpid());
   retval = mkfifo(pipename_in, S_IFIFO | 0600);
-  leave_priv_setting();
   if (!retval) {
-    sprintf(pipename_out, "%sdbgout.%d", TMPFILE, getpid());
-    enter_priv_on();
-    chown(pipename_in, get_orig_uid(), get_orig_gid());
+    asprintf(&pipename_out, "%s/dosemu.dbgout.%d", RUNDIR, getpid());
     retval = mkfifo(pipename_out, S_IFIFO | 0600);
-    leave_priv_setting();
     if (!retval) {
-      enter_priv_on();
-      chown(pipename_out, get_orig_uid(), get_orig_gid());
       mhpdbg.fdin = open(pipename_in, O_RDONLY | O_NONBLOCK);
-      leave_priv_setting();
       if (mhpdbg.fdin != -1) {
         /* NOTE: need to open read/write else O_NONBLOCK would fail to open */
-        enter_priv_on();
         mhpdbg.fdout = open(pipename_out, O_RDWR | O_NONBLOCK);
-        leave_priv_setting();
         if (mhpdbg.fdout != -1) {
-          add_to_io_select(mhpdbg.fdin,0);
+          add_to_io_select(mhpdbg.fdin, 0, mhp_input);
         }
         else {
           close(mhpdbg.fdin);
@@ -195,23 +171,20 @@ static void mhp_init(void)
     fprintf(stderr, "Can't create debugger pipes, dosdebug not available\n");
   }
   if (mhpdbg.fdin == -1) {
-    enter_priv_on();
     unlink(pipename_in);
+    free(pipename_in);
     unlink(pipename_out);
-    leave_priv_setting();
+    free(pipename_out);
   }
   else {
-    uid_t owner=get_orig_uid();
-    fchown(mhpdbg.fdin,owner,-1);
-    fchown(mhpdbg.fdout,owner,-1);
     if (dosdebug_flags) {
       /* don't fiddle with select, just poll until the terminal
        * comes up to send the first input
        */
        mhpdbg.nbytes = -1;
-       do mhp_input(); while (mhpdbg.nbytes <= 0);
        mhpdbgc.stopped = 1;
        wait_for_debug_terminal = 1;
+       mhp_input();
     }
   }
 }
@@ -221,40 +194,22 @@ void mhp_input()
   if (mhpdbg.fdin == -1) return;
   mhpdbg.nbytes = read(mhpdbg.fdin, mhpdbg.recvbuf, SRSIZE);
   if (mhpdbg.nbytes == -1) return;
+  if (mhpdbg.nbytes == 0 && !wait_for_debug_terminal) {
+    if (mhpdbgc.stopped) {
+      mhp_cmd("g");
+      mhp_send();
+    }
+    mhpdbg.active = 0;
+    return;
+  }
   if (!mhpdbg.active) {
     mhpdbg.active = 1; /* 1 = new session */
   }
 }
 
-static void mhp_poll(void)
+static void mhp_poll_loop(void)
 {
-
-   if (!mhpdbg.active) {
-     mhpdbg.nbytes = 0;
-     return;
-   }
-
-   if (mhpdbg.active == 1) {
-      /* new session has started */
-      mhpdbg.active++;
-      vm86s.vm86plus.vm86dbg_active = 1;
-      
-      mhp_printf ("%s", mhp_banner);
-      mhp_cmd("rmapfile");
-      if (wait_for_debug_terminal) wait_for_debug_terminal =0;
-      else mhp_cmd("r0");
-      mhp_send();
-   }
-
-   if (mhpdbgc.want_to_stop) {
-      mhpdbgc.stopped = 1;
-      mhpdbgc.want_to_stop = 0;
-   }
-   if (mhpdbgc.stopped) {
-      mhp_cmd("r0");
-      mhp_send();
-   }
-
+   char *ptr, *ptr1;
    for (;;) {
       handle_signals();
       /* NOTE: if there is input on mhpdbg.fdin, as result of handle_signals
@@ -267,7 +222,10 @@ static void mhp_poll(void)
            mhpdbg.nbytes=strlen(loopbuf);
          }
          else {
-          if (mhpdbgc.stopped) continue;
+          if (mhpdbgc.stopped) {
+            usleep(JIFFIE_TIME/10);
+            continue;
+          }
           else break;
         }
       }
@@ -275,6 +233,10 @@ static void mhp_poll(void)
         if (traceloop) { traceloop=loopbuf[0]=0; }
       }
       if ((mhpdbg.recvbuf[0] == 'q') && (mhpdbg.recvbuf[1] <= ' ')) {
+	 if (mhpdbgc.stopped) {
+	   mhp_cmd("g");
+	   mhp_send();
+	 }
 	 mhpdbg.active = 0;
 	 vm86s.vm86plus.vm86dbg_active = 0;
 	 mhpdbg.sendptr = 0;
@@ -282,58 +244,97 @@ static void mhp_poll(void)
          return;
       }
       mhpdbg.recvbuf[mhpdbg.nbytes] = 0x00;
-      mhp_cmd(mhpdbg.recvbuf);
-      mhp_send();
+      ptr = mhpdbg.recvbuf;
+      while (ptr && *ptr) {
+	ptr1 = strsep(&ptr, "\r\n");
+	if (!ptr1)
+	  ptr1 = ptr;
+	if (!ptr1)
+	  break;
+        mhp_cmd(ptr1);
+        mhp_send();
+      }
       mhpdbg.nbytes = 0;
    }
 }
 
-void mhp_exit_intercept(int errcode)
+static void mhp_poll(void)
 {
 
-   if (!errcode || !mhpdbg.active || (mhpdbg.fdin == -1) ) return;
+   if (!mhpdbg.active && !wait_for_debug_terminal) {
+     mhpdbg.nbytes = 0;
+     return;
+   }
 
-   mhp_printf("\n****\nleavedos(%d) called, at termination point of DOSEMU\n****\n\n", errcode);
-   mhp_send();
+   if (mhpdbg.active == 1) {
+      /* new session has started */
+      mhpdbg.active++;
+      vm86s.vm86plus.vm86dbg_active = 1;
+      
+      mhp_printf ("%s", mhp_banner);
+      mhp_cmd("rmapfile");
+      mhp_send();
+      if (wait_for_debug_terminal) wait_for_debug_terminal =0;
+   }
+
+   if (mhpdbgc.want_to_stop) {
+      mhpdbgc.stopped = 1;
+      mhpdbgc.want_to_stop = 0;
+   }
+   if (mhpdbgc.stopped) {
+      if (dosdebug_flags & DBGF_LOG_TEMPORARY) {
+         dosdebug_flags &= ~DBGF_LOG_TEMPORARY;
+	 mhp_cmd("log off");
+      }
+      mhp_cmd("r0");
+      mhp_send();
+   }
+   mhp_poll_loop();
+}
+
+void mhp_intercept_log(char *flags, int temporary)
+{
+   char buf[255];
+   sprintf(buf, "log %s", flags);
+   mhp_cmd(buf);
+   mhp_cmd("log on");
+   if (temporary)
+      dosdebug_flags |= DBGF_LOG_TEMPORARY;
+}
+
+void mhp_intercept(char *msg, char *logflags)
+{
+   if (!mhpdbg.active || (mhpdbg.fdin == -1)) return;
    mhpdbgc.stopped = 1;
    mhpdbgc.want_to_stop = 0;
+   traceloop = 0;
+   mhp_printf(msg);
    mhp_cmd("r0");
    mhp_send();
-   dosdebug_flags |= DBGF_IN_LEAVEDOS;
-   for (;;) {
-      mhp_input();
-      if (mhpdbg.nbytes <= 0) {
-         if (traceloop && mhpdbgc.stopped) {
-           strcpy(mhpdbg.recvbuf,loopbuf);
-           mhpdbg.nbytes=strlen(loopbuf);
-         }
-         else {
-          if (mhpdbgc.stopped) continue;
-          else break;
-        }
-      }
-      else {
-        if (traceloop) { traceloop=loopbuf[0]=0; }
-      }
-      if ((mhpdbg.recvbuf[0] == 'q') && (mhpdbg.recvbuf[1] <= ' ')) {
-	 mhpdbg.active = 0;
-	 vm86s.vm86plus.vm86dbg_active = 0;
-	 mhpdbg.sendptr = 0;
-         mhpdbg.nbytes = 0;
-         return;
-      }
-      mhpdbg.recvbuf[mhpdbg.nbytes] = 0x00;
-      mhp_cmd(mhpdbg.recvbuf);
-      mhp_send();
-      mhpdbg.nbytes = 0;
-      if (!(dosdebug_flags & DBGF_IN_LEAVEDOS)) return;
+   if (!(dosdebug_flags & DBGF_IN_LEAVEDOS)) {
+     set_VIP();
+     if (in_dpmi)
+       dpmi_eflags |= VIP;
+     if (logflags)
+       mhp_intercept_log(logflags, 1);
+     return;
    }
+   mhp_poll_loop();
+}
+
+void mhp_exit_intercept(int errcode)
+{
+   char buf[255];
+   if (!errcode || !mhpdbg.active || (mhpdbg.fdin == -1) ) return;
+
+   sprintf(buf, "\n****\nleavedos(%d) called, at termination point of DOSEMU\n****\n\n", errcode);
+   dosdebug_flags |= DBGF_IN_LEAVEDOS;
+   mhp_intercept(buf, NULL);
 }
 
 unsigned int mhp_debug(unsigned int code, unsigned int parm1, unsigned int parm2)
 {
   int rtncd = 0;
-  extern void bios_f000(), DBGload(), DBGload_CSIP(), DBGload_parblock();
 #if 0
   return rtncd;
 #endif
@@ -371,6 +372,8 @@ unsigned int mhp_debug(unsigned int code, unsigned int parm1, unsigned int parm2
 	    else {
 	      if ((DBG_ARG(mhpdbgc.currcode) != 0x21) || !mhpdbgc.bpload ) {
 	        mhpdbgc.stopped = 1;
+		vm86s.vm86plus.vm86dbg_active = 1;
+		vm86s.vm86plus.vm86dbg_TFpendig = 1;
 	        mhp_poll();
 	      }
 	    }
