@@ -1,6 +1,6 @@
 /* 
  * All modifications in this file to the original code are
- * (C) Copyright 1992, ..., 1999 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2004 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -45,7 +45,6 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
-#include <linux/unistd.h>
 #include <ctype.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -63,33 +62,15 @@
 #include "cpu.h"
 #include "dpmi.h"
 #include "utilities.h"
+#include "dosemu_config.h"
+#include "xms.h"
+#include "dis8086.h"
 
 #define MHP_PRIVATE
 #include "mhpdbg.h"
 
 #define makeaddr(x,y) ((((unsigned long)x) << 4) + (unsigned long)y)
 
-#if 0 /* modify_ldt already defined in dpmi.c */
-_syscall3(int, modify_ldt, int, func, void *, ptr, unsigned long, bytecount)
-#endif
-extern int modify_ldt(int func, void *ptr, unsigned long bytecount);
-
-/* externs */
-extern struct mhpdbgc mhpdbgc;
-extern int a20;
-extern void mhp_send(void), mhp_close(void), mhp_putc(char c1);
-
-#if 0
-/* NOTE: the below is already defined with #include "emu.h"
- *       Must NOT redefine it, else REQUIRES_VM86PLUS won't work !!!
- */
-extern struct vm86_struct vm86s;
-#endif
-
-extern int  dis_8086(unsigned int, const unsigned char *,
-                     unsigned char *, int, unsigned int *, unsigned int *,
-                     unsigned int, int);
-
 /* prototypes */
 static void* mhp_getadr(unsigned char *, unsigned int *, unsigned int *, unsigned int *);
 static void mhp_regs  (int, char *[]);
@@ -131,7 +112,7 @@ static unsigned int linmode = 0;
 static unsigned int codeorg = 0;
 
 static unsigned int dpmimode=1, saved_dpmimode=1;
-#define IN_DPMI  (in_dpmi && dpmimode)
+#define IN_DPMI  (in_dpmi && !in_dpmi_dos_int && dpmimode)
 
 static char lastd[32];
 static char lastu[32];
@@ -389,8 +370,7 @@ static void mhp_rusermap(int argc, char *argv[])
 
 static void mhp_rmapfile(int argc, char *argv[])
 {
-  PRIV_SAVE_AREA
-  FILE * ifp;
+  FILE * ifp = NULL;
   unsigned char bytebuf[IBUFS];
   unsigned long a1;
   char *map_fname = DOSEMU_MAP_PATH;
@@ -398,9 +378,16 @@ static void mhp_rmapfile(int argc, char *argv[])
   if (argc >= 2) {
     map_fname = argv[1];
   }
-  enter_priv_off();
-  ifp = fopen(map_fname, "r");
-  leave_priv_setting();
+  if (map_fname == NULL && dosemu_proc_self_exe != NULL) {
+    /* try to get symbols on the fly */
+    map_fname = malloc(strlen(dosemu_proc_self_exe) + 60);
+    strcpy(map_fname, "nm ");
+    strcat(map_fname, dosemu_proc_self_exe);
+    strcat(map_fname, " | grep -v '\\(compiled\\)\\|\\(\\.o$\\)\\|\\( a \\)' | sort");
+    ifp = popen(map_fname, "r");
+  } else if (map_fname != NULL) {
+    ifp = fopen(map_fname, "r");
+  }
   if (!ifp) {
      mhp_printf("unable to open map file %s\n", map_fname);
      return;
@@ -428,7 +415,12 @@ static void mhp_rmapfile(int argc, char *argv[])
      symbol_table[last_symbol].addr = a1;
      last_symbol++;
   }
-  fclose(ifp);
+  if (map_fname != argv[1] && map_fname != DOSEMU_MAP_PATH) {
+    pclose(ifp);
+    free(map_fname);
+  }
+  else
+    fclose(ifp);
   addrmax = symbol_table[last_symbol-1].addr;
   mhp_printf("%d symbol(s) processed\n", last_symbol);
   mhp_printf("highest address %08x(%s)\n", addrmax, getname(addrmax));
@@ -543,6 +535,7 @@ static void mhp_go(int argc, char * argv[])
       WRITE_FLAGS(READ_FLAGS() & ~TF);
       vm86s.vm86plus.vm86dbg_TFpendig=0;
       mhpdbgc.stopped = 0;
+      pic_sti();
    }
 }
 
@@ -569,10 +562,10 @@ static void mhp_trace(int argc, char * argv[])
       mhp_printf("must be in stopped state\n");
    } else {
       mhpdbgc.stopped = 0;
-      if (IN_DPMI) {
-        if (!dpmi_mhp_setTF(1)) return;
+      if (in_dpmi) {
+        dpmi_mhp_setTF(1);
       }
-      else WRITE_FLAGS(READ_FLAGS() | TF);
+      WRITE_FLAGS(READ_FLAGS() | TF);
       mhpdbgc.trapcmd = 1;
    }
 }
@@ -594,12 +587,14 @@ static void mhp_trace_force(int argc, char * argv[])
       mhp_printf("must be in stopped state\n");
    } else {
       mhpdbgc.stopped = 0;
-      if (IN_DPMI) {
-        if (!dpmi_mhp_setTF(1)) return;
+      if (in_dpmi) {
+        dpmi_mhp_setTF(1);
       }
-      else WRITE_FLAGS(READ_FLAGS() | TF);
+      WRITE_FLAGS(READ_FLAGS() | TF);
       vm86s.vm86plus.vm86dbg_TFpendig=1;
       mhpdbgc.trapcmd = 1;
+      /* disable PIC: we want to trace the program, not the HW int handlers */
+      pic_cli();
    }
 }
 
@@ -676,7 +671,6 @@ static void mhp_dis(int argc, char * argv[])
 
 static void mhp_dump_to_file(int argc, char * argv[])
 {
-   PRIV_SAVE_AREA
    unsigned int nbytes;
    unsigned long seekval;
    unsigned char * buf = 0;
@@ -699,21 +693,14 @@ static void mhp_dump_to_file(int argc, char * argv[])
       return;
    }
 
-   if (config.secure && ((seekval+nbytes) >0x10ffff)) {
-      mhp_printf("secure is ON, can\'t access above 1MEG\n");
-      return;
-   }
-
-   enter_priv_off();
    fd = open(argv[3], O_WRONLY | O_CREAT | O_TRUNC, 00775);
-   leave_priv_setting();
 
    if (fd < 0) {
-      mhp_printf("cannot open/create file %s\n%s\n", argv[3], sys_errlist[errno]);
+      mhp_printf("cannot open/create file %s\n%s\n", argv[3], strerror(errno));
       return;
    }
    if (write(fd, buf, nbytes) != nbytes) {
-      mhp_printf("write error: %s\n", sys_errlist[errno]);
+      mhp_printf("write error: %s\n", strerror(errno));
    }
    close(fd);
 }
@@ -734,7 +721,6 @@ static void mhp_mode(int argc, char * argv[])
 
 static void mhp_disasm(int argc, char * argv[])
 {
-   extern int a20; /* a20 is defined in dosemu/xms.c */
    int rc;
    unsigned int nbytes;
    unsigned long org;
@@ -782,14 +768,14 @@ static void mhp_disasm(int argc, char * argv[])
 #endif
 
    if (IN_DPMI) {
-     def_size = dpmi_mhp_get_selector_size(seg);
+     def_size = (dpmi_mhp_get_selector_size(seg)? 3:0);
      segmented =1;
    }
    else {
-     if (seekval < (a20 ? 0x10fff0 : 0x100000)) def_size = linmode;
+     if (seekval < (a20 ? 0x10fff0 : 0x100000)) def_size = (linmode? 3:0);
      else if (lastu[0] == '#')
         def_size = 0;
-     else def_size = 1;
+     else def_size = 3;
    }
    rc=0;
    buf = (unsigned char *) seekval;
@@ -933,6 +919,7 @@ static void mhp_enter(int argc, char * argv[])
 
 static void* mhp_getadr(unsigned char * a1, unsigned int * s1, unsigned int *o1, unsigned int *lim)
 {
+   static char buffer[0x10000];
    unsigned char * srchp;
    unsigned int seg1;
    unsigned int off1;
@@ -941,7 +928,6 @@ static void* mhp_getadr(unsigned char * a1, unsigned int * s1, unsigned int *o1,
    int use_ldt = IN_DPMI;
    unsigned long * lp;
    unsigned long base_addr, limit;
-   char buffer[0x10000];
 
    *lim = 0xFFFFFFFF;
 
@@ -1039,7 +1025,7 @@ static void* mhp_getadr(unsigned char * a1, unsigned int * s1, unsigned int *o1,
    }
 
    *lim = limit - off1;
-   return (void *) base_addr + off1;
+   return (void *) (base_addr + off1);
 }
 
 int mhp_setbp(unsigned long seekval)
@@ -1301,7 +1287,11 @@ static void mhp_regs(int argc, char * argv[])
 
  if (!traceloop)
   mhp_printf( "system state: %s%s%s%s\n",
+#ifdef X86_EMULATOR
+       config.cpuemu > 1 ? "emulated," : "",
+#else
        "",
+#endif
        mhpdbgc.stopped ? "stopped" : "running",
        IN_DPMI ? " in DPMI" : (in_dpmi?" in real mode while in DPMI":""),
        IN_DPMI ?(dpmi_mhp_getcsdefault()?"-32bit":"-16bit") : "");
@@ -1426,7 +1416,6 @@ int mhp_getcsip_value()
 
 void mhp_modify_eip(int delta)
 {
-  extern int dpmi_mhp_modify_eip(int delta);
   if (IN_DPMI) dpmi_mhp_modify_eip(delta);
   else LWORD(eip) +=delta;
 }
@@ -1439,9 +1428,8 @@ void mhp_cmd(const char * cmd)
 
 static void mhp_print_ldt(int argc, char * argv[])
 {
-  char buffer[0x10000];
-  extern unsigned long *ldt_buffer;
-  unsigned long *lp, *lp_=ldt_buffer;
+  static char buffer[0x10000];
+  unsigned long *lp, *lp_=(unsigned long *)ldt_buffer;
   unsigned long base_addr, limit;
   int type, type2, i;
   unsigned int seg;
@@ -1595,13 +1583,13 @@ static void print_log_breakpoints(void)
    num_logbp = 0;
    for (rx=0; rx <num_regex; rx++) {
      if (rxbuf[rx]) {
-       mhp_printf("log break point %d: %s\n", rx, rxpatterns[rx]);
+       mhp_printf("log breakpoint %d: %s\n", rx, rxpatterns[rx]);
        mhp_send();
        num_logbp++;
      }
    }
    if (!num_logbp) {
-     mhp_printf("no log break points aktiv\n");
+     mhp_printf("no log breakpoint active\n");
    }
 }
 
