@@ -45,6 +45,9 @@
  * that it supports fonts in gfx modes.
  * -- sw
  *
+ * 2002/11/30: Started user font support. Needs some more work!
+ * -- eric@coli.uni-sb.de - Eric
+ *
  * DANG_END_CHANGELOG
  */
 
@@ -115,6 +118,7 @@ static void tty_char_out(unsigned char ch, int s, int attr);
 static void int10_old(void);
 #if X_GRAPHICS
 static void int10_new(void);
+static void vga_ROM_to_RAM(unsigned height, int bank);
 #endif
 
 static inline void set_cursor_shape(ushort shape) {
@@ -162,13 +166,16 @@ Scroll(us *sadr, int x0, int y0, int x1, int y1, int l, int att)
 {
   int dx = x1 - x0 + 1;
   int dy = y1 - y0 + 1;
-  int x, y;
+  int x, y, co, li;
   us blank = ' ' | (att << 8);
   us tbuf[MAX_COLUMNS];
 
   if (config.cardtype == CARD_NONE)
      return;
 
+  li= READ_BYTE(BIOS_ROWS_ON_SCREEN_MINUS_1) + 1;
+  co= READ_WORD(BIOS_SCREEN_COLUMNS);
+
   if ( (config.cardtype == CARD_MDA) && ((att & 7) != 0) && ((att & 7) != 7) )
     {
       blank = ' ' | ((att | 7) << 8);
@@ -241,6 +248,14 @@ Scroll(us *sadr, int x0, int y0, int x1, int y1, int l, int att)
   #define bios_scroll(x0,y0,x1,y1,n,attr) Scroll(screen_adr,x0,y0,x1,y1,n,attr)
 #endif
 
+#if X_GRAPHICS
+static int using_text_mode(void)
+{
+  unsigned char mode = READ_BYTE(BIOS_VIDEO_MODE);
+  return mode <= 3 || mode == 7 || (mode >= 0x50 && mode <= 0x5a);
+}
+#endif
+
 /* Output a character to the screen. */ 
 void char_out(unsigned char ch, int page)
 {
@@ -254,12 +269,15 @@ void char_out(unsigned char ch, int page)
 void tty_char_out(unsigned char ch, int s, int attr)
 {
   int newline_att = 7;
-  int xpos, ypos;
+  int xpos, ypos, co, li;
   int gfx_mode = 0;
   unsigned char *dst;
 
 #if USE_DUALMON
   int virt_text_base = BIOS_SCREEN_BASE;
+#if X_GRAPHICS
+  if (config.X) virt_text_base = (int)vga.mem.base;
+#endif
 #endif
 
 /* i10_deb("tty_char_out: char 0x%02x, page %d, attr 0x%02x\n", ch, s, attr); */
@@ -269,12 +287,15 @@ void tty_char_out(unsigned char ch, int s, int attr)
      return;
   }
 
+  li= READ_BYTE(BIOS_ROWS_ON_SCREEN_MINUS_1) + 1;
+  co= READ_WORD(BIOS_SCREEN_COLUMNS);
+
   xpos = get_bios_cursor_x_position(s);
   ypos = get_bios_cursor_y_position(s);
 
   /* check for gfx mode */
 #if X_GRAPHICS
-  if(config.X && vga.mode_class != TEXT) gfx_mode = 1;
+  if(config.X && !using_text_mode()) gfx_mode = 1;
 #endif
 
   switch (ch) {
@@ -348,14 +369,20 @@ void
 clear_screen(int s, int att)
 {
   u_short *schar, blank = ' ' | (att << 8);
-  int lx;
+  int lx, co, li;
 #if USE_DUALMON
   int virt_text_base = BIOS_SCREEN_BASE;
+#if X_GRAPHICS
+  if (config.X) virt_text_base = (int)vga.mem.base;
+#endif
 #endif
 
   if (config.cardtype == CARD_NONE)
      return;
 
+  li= READ_BYTE(BIOS_ROWS_ON_SCREEN_MINUS_1) + 1;
+  co= READ_WORD(BIOS_SCREEN_COLUMNS);
+
   v_printf("INT10: cleared screen: page %d, attr 0x%02x, screen_adr %p\n", s, att, SCREEN_ADR(s));
   if (s > max_page) return;
   
@@ -416,12 +443,19 @@ static boolean X_set_video_mode(int mode) {
   }
 
   if(adjust_font_size) {
+    i10_msg("set_video_mode: font size %d lines\n", vga_font_height);
+
+    CRTC_set_index(9);
+    CRTC_write_value((CRTC_read_value() & ~0x1f) + vga_font_height -1);
+    vmi->char_height = vga_font_height; /* should already be done, */
+    /* as side effect of the CRTC write access in crtcemu... */
+
     text_scanlines = vmi->height;
     li = text_scanlines / vga_font_height;
     if(li > MAX_LINES) li = MAX_LINES;
     WRITE_BYTE(BIOS_ROWS_ON_SCREEN_MINUS_1, li - 1);
     WRITE_WORD(BIOS_FONT_HEIGHT, vga_font_height);
-    if(vmi->mode_class == TEXT) X_set_textsize(co, li);
+    if(using_text_mode()) X_set_textsize(co, li);
     return 1;
   }
 
@@ -452,6 +486,10 @@ static boolean X_set_video_mode(int mode) {
   if(li > MAX_LINES) li = MAX_LINES;
   vga_font_height = vmi->char_height;
   text_scanlines = vmi->height;
+  if (using_text_mode()) {
+    vga_font_height = text_scanlines / 25;
+    vmi->char_height = vga.char_height = vga_font_height;
+  }
 
   WRITE_BYTE(BIOS_ROWS_ON_SCREEN_MINUS_1, li - 1);
   WRITE_WORD(BIOS_SCREEN_COLUMNS, co);
@@ -487,14 +525,15 @@ static boolean X_set_video_mode(int mode) {
    */
   video_mode = mode;
 
-  if(clear_mem && vmi->mode_class == TEXT) clear_screen(0, 7);
+  if(clear_mem && using_text_mode()) clear_screen(0, 7);
 
   WRITE_BYTE(BIOS_VIDEO_INFO_0, clear_mem ? 0x60 : 0xe0);
   memset((void *) 0x450, 0, 0x10);	/* equiv. to set_bios_cursor_(x/y)_position(0..7, 0) */
 
   video_page = 0;
   WRITE_WORD(BIOS_VIDEO_MEMORY_ADDRESS, 0);
-  screen_adr = (void *) (vga.buffer_seg << 4);
+  virt_text_base = (int)vga.mem.base;
+  screen_adr = (void *)vga.mem.base;
   screen_mask = 0;
   cursor_col = get_bios_cursor_x_position(0);
   cursor_row = get_bios_cursor_y_position(0);
@@ -503,6 +542,10 @@ static boolean X_set_video_mode(int mode) {
   WRITE_WORD(BIOS_CURSOR_SHAPE, 0x0607);
 
   WRITE_WORD(BIOS_VIDEO_PORT, vga.config.mono_port ? 0x3b4 : 0x3d4);
+  if (mode == 0x6)
+    WRITE_BYTE(BIOS_VDU_COLOR_REGISTER, 0x3f);
+  else if (mode <= 0x7)
+    WRITE_BYTE(BIOS_VDU_COLOR_REGISTER, 0x30);
 
   switch(vga.char_height) {
     case 14:
@@ -515,6 +558,13 @@ static boolean X_set_video_mode(int mode) {
       u = vgaemu_bios.font_8;
   }
 
+  if (using_text_mode()) {
+    v_printf("INT10: X_set_video_mode: 8x%d ROM font -> bank 0\n",
+             vga.char_height);
+    vga_ROM_to_RAM(vga.char_height, 0); /* 0 is default bank */
+    i10_msg("activated font bank 0\n");
+  }
+
   SETIVEC(0x43, 0xc000, u);
 
   return 1;
@@ -692,6 +742,55 @@ static void return_state(Bit8u *statebuf) {
 	memset(statebuf + 0x33, 0, 13);
 }
 
+/* helpers for font processing - eric@coli.uni-sb.de  11/2002 */
+/* only for TEXT mode: Otherwise, int 0x43 is used...         */
+
+#ifdef X_GRAPHICS
+static void vga_RAM_to_RAM(unsigned height, unsigned char chr, unsigned count,
+                           unsigned seg, unsigned ofs, int bank)
+{
+  char *dst;
+  unsigned i;
+  unsigned long src;
+  unsigned bankofs;
+  if (!count) {
+    v_printf("Tried to load 0 characters of font data???\n");
+    return;
+  }
+  src = seg;
+  src <<= 4;
+  src += ofs;
+  bankofs = ((bank & 3) << 1) + ((bank & 4) >> 2);
+  bankofs <<= 13; /* unit is 8k */
+  i10_msg("load 8x%d font (char %d..%d) 0x%04x:0x%04x -> bank %d\n",
+          height, chr, chr+count-1, seg, ofs, bank);
+  dst = vga.mem.base + 0x20000 + bankofs;
+  /* copy count characters of height bytes each to vga_font_mem */
+  for(i = chr; i < chr + count; i++)
+    MEMCPY_2UNIX(dst + i * 32, src + i * height, height);
+}
+
+static void vga_ROM_to_RAM(unsigned height, int bank)
+{
+  unsigned seg, ofs;
+  seg = 0xc000;
+  switch (height) { /* ALTERNATE ROM fonts not yet usable! */
+  case 8:
+    ofs = vgaemu_bios.font_8;
+    break;
+  case 14:
+    ofs = vgaemu_bios.font_14;
+    break;
+  case 16:
+    ofs = vgaemu_bios.font_16;
+    break;
+  default:
+    v_printf("Error! Tried to load 8x%d ROM font!?\n",height);
+    ofs = vgaemu_bios.font_16;
+  }
+  vga_RAM_to_RAM(height,0,256,seg,ofs,bank);
+}
+#endif /* X_GRAPHICS */ 
 
 /******************************************************************/
 
@@ -715,6 +814,7 @@ void int10_old(void) /* with dualmon */
   unsigned int page;
   u_char c;
   us *sm;
+  int co, li;
 
 #if USE_DUALMON
   int virt_text_base = BIOS_SCREEN_BASE;
@@ -725,11 +825,12 @@ void int10_old(void) /* with dualmon */
     last_equip = BIOS_CONFIG_SCREEN_MODE;
     if (IS_SCREENMODE_MDA) Video->is_mapped = 1;
     else Video->is_mapped = Video_default->is_mapped;
-    li= READ_BYTE(BIOS_ROWS_ON_SCREEN_MINUS_1) + 1;
-    co= READ_WORD(BIOS_SCREEN_COLUMNS);
   }
 #endif
 
+  li= READ_BYTE(BIOS_ROWS_ON_SCREEN_MINUS_1) + 1;
+  co= READ_WORD(BIOS_SCREEN_COLUMNS);
+
   if (debug_level('v') >= 3)
     {
       if (debug_level('v') >= 4)
@@ -1283,11 +1384,14 @@ void int10_old(void) /* with dualmon */
 void int10_new(void) /* with X but without dualmon */
 {
   /* some code here is copied from Alan Cox ***************/
-  int x, y;
+  int x, y, co, li;
   unsigned m, page;
   u_char c;
   us *sm;
 
+  li= READ_BYTE(BIOS_ROWS_ON_SCREEN_MINUS_1) + 1;
+  co= READ_WORD(BIOS_SCREEN_COLUMNS);
+
   if(HI(ax) > 0x0f)	// EVIL hack! Remove it later!!!
   if (debug_level('v') >= 3)
     {
@@ -1398,7 +1502,7 @@ void int10_new(void) /* with X but without dualmon */
         "scroll up: %u lines, area %u.%u-%u.%u, attr 0x%02x\n",
         LO(ax), LO(cx), HI(cx), LO(dx), HI(dx), HI(bx)
       );
-      if(vga.mode_class == TEXT) {
+      if(using_text_mode()) {
         bios_scroll(LO(cx), HI(cx), LO(dx), HI(dx), LO(ax), HI(bx));
       }
       else {
@@ -1412,7 +1516,7 @@ void int10_new(void) /* with X but without dualmon */
         "scroll dn: %u lines, area %u.%u-%u.%u, attr 0x%02x\n",
         LO(ax), LO(cx), HI(cx), LO(dx), HI(dx), HI(bx)
       );
-      if(vga.mode_class == TEXT) {
+      if(using_text_mode()) {
         bios_scroll(LO(cx), HI(cx), LO(dx), HI(dx), -LO(ax), HI(bx));
       }
       else {
@@ -1427,7 +1531,7 @@ void int10_new(void) /* with X but without dualmon */
         i10_msg("read char: invalid page %d\n", page);
         break;
       }
-      if(vga.mode_class == TEXT) {
+      if(using_text_mode()) {
         sm = SCREEN_ADR(page);
         LWORD(eax) = sm[co * get_bios_cursor_y_position(page)
                    + get_bios_cursor_x_position(page)];
@@ -1445,7 +1549,7 @@ void int10_new(void) /* with X but without dualmon */
 
     case 0x09:		/* write char & attr */
     case 0x0a:		/* write char */
-      if(HI(ax) == 0x0a && vga.mode_class == TEXT) {
+      if(HI(ax) == 0x0a && using_text_mode()) {
         i10_deb(
           "write char: page %u, char 0x%02x '%c'\n",
           HI(bx), LO(ax), LO(ax) > ' ' && LO(ax) < 0x7f ? LO(ax) : ' '
@@ -1457,7 +1561,7 @@ void int10_new(void) /* with X but without dualmon */
           HI(bx), LO(ax), LO(ax) > ' ' && LO(ax) < 0x7f ? LO(ax) : ' ', LO(bx)
         );
       }
-      if(vga.mode_class == TEXT) {
+      if(using_text_mode()) {
         u_short *sadr;
         u_short c_attr;
         int n;
@@ -1497,27 +1601,42 @@ void int10_new(void) /* with X but without dualmon */
       break;
 
 
-    case 0x0b:		/* set bg/border color */
-      i10_msg("set bg/border color: NOT IMPLEMENTED\n");
+    case 0x0b:         /* set palette/bg/border color */
+      {
+       unsigned char currentpalette = READ_BYTE(BIOS_VDU_COLOR_REGISTER);
+       i10_msg("set palette or bg/border, bx=%x\n",LWORD(ebx));
+       if (HI(bx) == 0) {
+         currentpalette &= ~0x1f;
+         currentpalette |= LO(bx) & 0x1f;
+       } else if (HI(bx) == 1) {
+         if (LO(bx))
+           currentpalette |= 0x20;
+         else
+           currentpalette &= ~0x20;
+       } else {
+         break;
+       }
+       Misc_set_color_select(currentpalette);
+       WRITE_BYTE(BIOS_VDU_COLOR_REGISTER, currentpalette);
+      }
       break;
 
-
     case 0x0c:		/* write pixel */
-      if(vga.mode_class != TEXT)
+      if(!using_text_mode())
         vgaemu_put_pixel(LWORD(ecx), LWORD(edx), HI(bx), LO(ax));
       break;
 
 
     case 0x0d:		/* read pixel */
       LO(ax) = 0;
-      if(vga.mode_class != TEXT) {
+      if(!using_text_mode()) {
         LO(ax) = vgaemu_get_pixel(LWORD(ecx), LWORD(edx), HI(bx));
         i10_msg("read pixel: 0x%02x\n", LO(ax));
       }
       break;
 
     case 0x0e:		/* print char */
-      if(vga.mode_class == TEXT) {
+      if(using_text_mode()) {
         i10_deb(
           "tty put char: page %u, char 0x%02x '%c'\n",
           HI(bx), LO(ax), LO(ax) > ' ' && LO(ax) < 0x7f ? LO(ax) : ' '
@@ -1690,35 +1809,75 @@ void int10_new(void) /* with X but without dualmon */
         i10_deb("char gen: func 0x%02x, bx 0x%04x\n", LO(ax), LWORD(ebx));
 
         switch(LO(ax)) {
+          case 0x03:
+            /* For EGA: Select fonts xx and yy for attrib bit 3 1/0 */
+            /* For VGA: Same, but using Xxx and Yyy. BL is 00XYxxyy */
+            /* ***          see also vgaemu_put_char()         ***  */
+            /* the X/Y bits mean 8k offset, xx/yy use 16k units     */
+            /* see: sequencer, map select, data[3] ... sequemu.c    */
+            Seq_set_index(3);
+            Seq_write_value(LO(bx));
+            i10_msg("sequencer char map select: 0x%02x\n", LO(bx));
+            break;
+
           case 0x01:		/* load 8x14 charset */
           case 0x11:
             vga_font_height = 14;
+	    vga_ROM_to_RAM(14, LO(bx));
             goto more_lines;
 
           case 0x02:		/* load 8x8 charset */
           case 0x12:
             vga_font_height = 8;
+	    vga_ROM_to_RAM(8, LO(bx));
             goto more_lines;
 
           case 0x04:		/* load 8x16 charset */
           case 0x14:
             vga_font_height = 16;
+	    vga_ROM_to_RAM(16, LO(bx));
             goto more_lines;
 
           /* load a custom font */
-          /* for now just ust it's size to set things */
           case 0x00:
           case 0x10:
             vga_font_height = HI(bx);
-            i10_deb("loaded font completely ignored except size!\n");
-            /* the rest is ignored for now */
+            /* *************************************************************
+             * func 00 would not change as much as func 0x10, which would  *
+             * reprogram registers 9 = bh-1 (mode 7 only, max scan line),  *
+             * a = bh-2 (cursor start), b = 0 (cursor end),                *
+             * 12 = (rows+1) / (bh-1) (vertical display end),              *
+             * 14 = bh-1 (underline loc). Recalcluates CRT buffer length.  *
+             * Max character rows is also recalculated, as well as by/char *
+             * ... and page 0 must be active                               *
+             * ES:BP -> table CX = count of chars DX = from which char on  *
+             * BL = which block to load into map2 BH = bytes / char        *
+             ************************************************************* */
+            vga_RAM_to_RAM(HI(bx), LO(dx), LWORD(ecx),
+                REG(es), LWORD(ebp), LO(bx));
+            i10_msg("some user font data loaded\n");
             goto more_lines;
 
           more_lines:
+            /* Also activating the target bank - some programs */
+            /* seem to assume this. Sigh...                    */
+            
+            /* FIXME: only useful when using VGAEMU, add a #define   */
+            Seq_set_index(3);     /* sequencer: character map select */
+            /* available: x, y, c...!?    transforming bitfields...  */
+            x = (LO(bx) & 3) | ((LO(bx) & 4) << 2);
+            x |= ((LO(bx) & 3) << 2) | ((LO(bx) & 4) << 3);
+            Seq_write_value(x);         /* set bank N for both fonts */
+            i10_msg("activated font bank %d (0x%02x)\n",
+                (LO(bx) & 7), x);
+            /* | (1 << 16) is "only update font stuff" */
             if(!set_video_mode(video_mode | (1 << 16))) {
               li = old_li;	/* not that it changed */
+              v_printf("Problem changing font height %d->%d\n",
+                  old_font_height, vga_font_height);
               vga_font_height = old_font_height;
             }
+            WRITE_WORD(BIOS_FONT_HEIGHT, vga_font_height);
             break;
 
           case 0x20:		/* set 8x8 gfx chars */
@@ -1730,6 +1889,8 @@ void int10_new(void) /* with X but without dualmon */
           case 0x22:		/* load 14x8 gfx chars */
           case 0x23:		/* load 8x8 gfx chars */
           case 0x24:		/* load 16x8 gfx chars */
+          /* BL=0/1/2/3 for DL/14/25/43 rows                   */
+          /* CX is byte / char, ES:BP is pointer for case 0x21 */
             seg = 0xc000;
             switch(LO(ax)) {
               case 0x21:
@@ -1836,9 +1997,11 @@ void int10_new(void) /* with X but without dualmon */
 
         case 0x30:	/* select vertical resolution */
           {
-            static int scanlines[3] = {200, 350, 400};
-
-            if((unsigned) LO(ax) < 3) {
+            static int scanlines[4] = {200, 350, 400, 480};
+            /* 480 (undocumented) (int 0x10 AH=12 BL=30)          */
+            /* "will take effect on the next mode set" is         */
+            /* not yet implemented                                */
+            if((unsigned) LO(ax) < 4) {
               text_scanlines = scanlines[LO(ax)];
               LO(ax) = 0x12;  
               i10_deb("select vert res: %d lines", text_scanlines);
@@ -1865,10 +2028,20 @@ void int10_new(void) /* with X but without dualmon */
 #endif
 
         case 0x36:	/* video screen on/off */
-          if(LO(ax) == 0)
-            i10_deb("turn video screen on (ignored)\n");
-          else
-            i10_deb("turn video screen off (ignored)\n");
+          /* VGAEMU oriented (do port 3c0 flag as well?)            */
+          /* (probably not: port 0x3c0|=0x20 -> all overscan color) */
+          /* FIXME: only useful when using VGAEMU, add a #define    */
+          /* Will influence vga.config.video_off -> X_update_screen */
+          Seq_set_index(1);             /* sequencer: clocking mode */
+          if(LO(ax) == 0) {
+            i10_deb("turn video screen on (partially ignored)\n");
+             Seq_write_value(vga.seq.data[1] & ~0x20);
+             /* bit 0x20 is screen refresh off */
+          } else {
+             i10_deb("turn video screen off (partially ignored)\n");
+             Seq_write_value(vga.seq.data[1] | 0x20);
+             /* bit 0x20 is screen refresh off */
+          }
 #if 0
           LO(ax) = 0x12;  
 #endif
