@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 2001 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2004 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -131,35 +131,33 @@
 #include "config.h"
 #include "port.h"
 #include "hlt.h"
-#ifndef C_RUN_IRQS
-#define C_RUN_IRQS
 #include "bitops.h"
-#undef C_RUN_IRQS
-#else
-#include "bitops.h"
-#endif
-#include "timers.h"
 #include "pic.h"
-#include "iodev.h"
 #include "memory.h"
 /* #include <sys/vm86.h> */
 #include <sys/time.h>
 #include "cpu.h"
 #include "emu.h"
+#include "timers.h"
+#include "iodev.h"
 #include "dpmi.h"
 #include "serial.h"
 #include "int.h"
 #include "ipx.h"
+#ifdef X86_EMULATOR
+#include "cpu-emu.h"
+#endif
 
 #undef us
 #define us unsigned
-void timer_tick(void);
-void pic_activate();
-extern void timer_int_engine(void);
+static void pic_activate(void);
 
 static unsigned long pic1_isr;         /* second isr for pic1 irqs */
 static unsigned long pic_irq2_ivec = 0;
 
+static Bit16u cb_cs = 0;
+static Bit16u cb_ip = 0;
+
 /*
  * pic_pirr contains a bit set for each interrupt which has attempted to
  * re-trigger.
@@ -172,33 +170,15 @@ static unsigned long pic_irq2_ivec = 0;
  * occurs.
  */
 
-#if 0
-static unsigned long pic1_mask;        /* bits set for pic1 levels */
-#endif
 static unsigned long pic1_mask = 0x07f8; /* bits set for pic1 levels */
-/* 
-even asgcc -Wall says;
-pic.c: At top level:
-pic.c:81: warning: `pic1_mask' defined but not used
-though it _is_ used in the assembly part
-*/
 static unsigned long   pic_smm = 0;      /* 32=>special mask mode, 0 otherwise */
 
 static unsigned long   pic_pirr;         /* pending requests: ->irr when icount==0 */
 static unsigned long   pic_wirr;             /* watchdog timer for pic_pirr */
 static unsigned long   pic_wcount = 0;       /* watchdog for pic_icount  */
 unsigned long	pic_icount_od = 1;           /* overdrive for pic_icount_od */
-#if 0
-unsigned long   pic0_imr = 0xf800;  /* IRQs 3-7 on pic0 start out disabled */
-unsigned long   pic1_imr = 0x07f8;  /* IRQs 8-15 on pic1 start out disabled */
-unsigned long   pic_imr = 0xfff8;   /* interrupt mask register, enable irqs 0,1 */
-unsigned long   pice_imr = -1;      /* interrupt mask register, dos emulator */ 
-unsigned long   pic_stack[32];      /* stack of interrupt levels */
-unsigned long   pic_sp = 0;         /* pointer to stack */
-unsigned long   pic_vm86_count=0;   /* counter for trips around vm86 loop */
-unsigned long   pic_dpmi_count=0;   /* counter for trips around dpmi loop */
-         hitimer_t pic_sys_time=NEVER; /* system time for scheduling interrupts */
-#endif
+unsigned long	pic_irqs_active = 0;
+
 static   hitimer_t pic_ltime[33] =     /* timeof last pic request honored */
                 {NEVER, NEVER, NEVER, NEVER, NEVER, NEVER, NEVER, NEVER,
                  NEVER, NEVER, NEVER, NEVER, NEVER, NEVER, NEVER, NEVER,
@@ -213,15 +193,16 @@ static   hitimer_t pic_ltime[33] =     /* timeof last pic request honored */
                  NEVER};
   
 #define PNULL	(void *) 0
-static struct lvldef pic_iinfo[32] =
-                     {{PNULL,0x02}, {PNULL,0x08}, {PNULL,0x09}, {PNULL,0x70},
-                      {PNULL,0x71}, {PNULL,0x72}, {PNULL,0x73}, {PNULL,0x74},
-                      {PNULL,0x75}, {PNULL,0x76}, {PNULL,0x77}, {PNULL,0x0b},
-                      {PNULL,0x0c}, {PNULL,0x0d}, {PNULL,0x0e}, {PNULL,0x0f},
-                      {PNULL,0x00}, {PNULL,0x00}, {PNULL,0x00}, {PNULL,0x00},
-                      {PNULL,0x00}, {PNULL,0x00}, {PNULL,0x00}, {PNULL,0x00},
-                      {PNULL,0x00}, {PNULL,0x00}, {PNULL,0x00}, {PNULL,0x00},
-                      {PNULL,0x00}, {PNULL,0x00}, {PNULL,0x00}, {PNULL,0x00}};   
+static struct lvldef pic_iinfo[32] = {
+{PNULL,PNULL,0x02}, {PNULL,PNULL,0x08}, {PNULL,PNULL,0x09}, {PNULL,PNULL,0x70},
+{PNULL,PNULL,0x71}, {PNULL,PNULL,0x72}, {PNULL,PNULL,0x73}, {PNULL,PNULL,0x74},
+{PNULL,PNULL,0x75}, {PNULL,PNULL,0x76}, {PNULL,PNULL,0x77}, {PNULL,PNULL,0x0b},
+{PNULL,PNULL,0x0c}, {PNULL,PNULL,0x0d}, {PNULL,PNULL,0x0e}, {PNULL,PNULL,0x0f},
+{PNULL,PNULL,0x00}, {PNULL,PNULL,0x00}, {PNULL,PNULL,0x00}, {PNULL,PNULL,0x00},
+{PNULL,PNULL,0x00}, {PNULL,PNULL,0x00}, {PNULL,PNULL,0x00}, {PNULL,PNULL,0x00},
+{PNULL,PNULL,0x00}, {PNULL,PNULL,0x00}, {PNULL,PNULL,0x00}, {PNULL,PNULL,0x00},
+{PNULL,PNULL,0x00}, {PNULL,PNULL,0x00}, {PNULL,PNULL,0x00}, {PNULL,PNULL,0x00}
+};
 
 /*
  * run_irq()       checks for and runs any interrupts requested in pic_irr
@@ -296,7 +277,7 @@ static unsigned char pic1_cmd;
 #ifdef NO_DEBUGPRINT_AT_ALL
 #define pic_print(code,s1,v1,s2)
 #else
-#define pic_print(code,s1,v1,s2)	if (d.request>code){p_pic_print(s1,v1,s2);}
+#define pic_print(code,s1,v1,s2)	if (debug_level('r')>code){p_pic_print(s1,v1,s2);}
 
 static void p_pic_print(char *s1, int v1, char *s2)
 {
@@ -317,10 +298,10 @@ char ci,cc;
   if(header_count>15) header_count=0;
   
   if(s2)
-  log_printf(1, "PIC: %c%2ld %c%2ld %08lx %08lx %08lx %s%02d%s\n",
+  log_printf(1, "PIC: %c%2d %c%2d %08lx %08lx %08lx %s%02d%s\n",
      cc, pic_icount, ci, pic_ilevel, pic_isr, pic_imr, pic_irr, s1, v1, s2);
   else
-  log_printf(1, "PIC: %c%2ld %c%2ld %08lx %08lx %08lx %s\n",
+  log_printf(1, "PIC: %c%2d %c%2d %08lx %08lx %08lx %s\n",
      cc, pic_icount, ci, pic_ilevel, pic_isr, pic_imr, pic_irr, s1);
 
 }
@@ -340,7 +321,7 @@ char ci,cc;
 
  * DANG_END_REMARK pic_print
  */
-inline void pic_push(int val)
+static inline void pic_push(int val)
 {
     if(pic_sp<32){
        pic_stack[pic_sp++]=val;
@@ -349,7 +330,7 @@ inline void pic_push(int val)
     }
 }
 
-inline int pic_pop()
+static inline int pic_pop(void)
 {
     if(pic_sp) {
        return pic_stack[--pic_sp];
@@ -360,8 +341,7 @@ inline int pic_pop()
 }
 
 
-void set_pic0_base(int_num)
-unsigned char int_num;
+static void set_pic0_base(unsigned char int_num)
 {
   unsigned char int_n;
   int_n        = int_num & 0xf8;         /* it's not worth doing a loop */
@@ -377,8 +357,7 @@ unsigned char int_num;
 }
 
 
-void set_pic1_base(int_num)
-unsigned char int_num;
+static void set_pic1_base(unsigned char int_num)
 {
   unsigned char int_n;
   int_n        = int_num & 0xf8;         /* it's not worth doing a loop */
@@ -418,14 +397,21 @@ static char                icw_max_state;          /* number of icws expected
 int ilevel;			  /* level to reset on outb 0x20  */
 
 port -= 0x20;
-ilevel=pic_ilevel;
-if(pic_sp) {
-   if(ilevel!=pic_stack[pic_sp-1]) {
-      pic_print(2,"ilevel=",ilevel," differed from pic_stack");
-   }
-   ilevel=pic_stack[pic_sp-1];
+ilevel = 32;
+if (pic_isr)
+  ilevel=find_bit(pic_isr);
+if(ilevel != pic_ilevel)
+  error("PIC0: ilevel=%x != pic_ilevel=%x, pic_isr=%lx\n",
+    ilevel, pic_ilevel, pic_isr);
+if (ilevel != 32 && !test_bit(ilevel, &pic_irqall)) {
+  /* this is a fake IRQ, don't allow to reset its ISR bit */
+  pic_print(1, "Protecting ISR bit for lvl ", ilevel, " from spurious EOI");
+  ilevel = 32;
 }
 
+if (in_dpmi)
+  dpmi_eflags |= VIP;	/* we have to leave the signal context */
+
 if(!port){                          /* icw1, ocw2, ocw3 */
   if(value&0x10){                   /* icw1 */
     icw_max_state = (value & 1) + 1;
@@ -440,15 +426,9 @@ if(!port){                          /* icw1, ocw2, ocw3 */
     pic0_cmd=3;
     }
   else if((value&0xb8) == 0x20) {    /* ocw2 */
-     if(!pic_sp) {
-	pic_print(2,"EOI with nothing on pic_stack! ilevel=",ilevel,"");
-     } else if(pic_stack[pic_sp-1]!=ilevel) {
-	pic_print(2,"Stack value differed from ilevel! stack=",pic_stack[pic_sp-1],"");
-     }
     /* irqs on pic1 require an outb20 to each pic. we settle for any 2 */
      if(!clear_bit(ilevel,&pic1_isr)) {
        clear_bit(ilevel,&pic_isr);  /* the famous outb20 */
-       pic_ilevel=pic_pop();  /* only pop stack when pic_isr gets cleared */
        pic_print(1,"EOI resetting bit ",ilevel, " on pic0");
        }
      else
@@ -478,14 +458,21 @@ static char /* icw_state, */     /* !=0 => port 1 does icw 2,3,(4) */
 int ilevel;			  /* level to reset on outb 0x20  */
 
 port -= 0xa0;
-ilevel=pic_ilevel;
-if(pic_sp) {
-   if(ilevel!=pic_stack[pic_sp-1]) {
-      pic_print(2,"ilevel=",ilevel," differed from pic_stack");
-   }
-   ilevel=pic_stack[pic_sp-1];
+ilevel = 32;
+if (pic_isr)
+  ilevel=find_bit(pic_isr);
+if(ilevel != pic_ilevel)
+  error("PIC1: ilevel=%x != pic_ilevel=%x, pic_isr=%lx\n",
+    ilevel, pic_ilevel, pic_isr);
+if (ilevel != 32 && !test_bit(ilevel, &pic_irqall)) {
+  /* this is a fake IRQ, don't allow to reset its ISR bit */
+  pic_print(1, "Protecting ISR bit for lvl ", ilevel, " from spurious EOI");
+  ilevel = 32;
 }
 
+if (in_dpmi)
+  dpmi_eflags |= VIP;	/* we have to leave the signal context */
+
 if(!port){                            /* icw1, ocw2, ocw3 */
   if(value&0x10){                     /* icw1 */
     icw_max_state = (value & 1) + 1;
@@ -499,15 +486,9 @@ if(!port){                            /* icw1, ocw2, ocw3 */
     pic1_cmd=3;
     }
   else if((value&0xb8) == 0x20) {    /* ocw2 */
-     if(!pic_sp) {
-	pic_print(2,"EOI with nothing on pic_stack! ilevel=",ilevel,"");
-     } else if(pic_stack[pic_sp-1]!=ilevel) {
-	pic_print(2,"Stack value differed from ilevel! stack=",pic_stack[pic_sp-1],"");
-     }
     /* irqs on pic1 require an outb20 to each pic. we settle for any 2 */
      if(!clear_bit(ilevel,&pic1_isr)) {
        clear_bit(ilevel,&pic_isr);  /* the famous outb20 */
-       pic_ilevel=pic_pop();  /* only pop stack when pic_isr gets cleared */
        pic_print(1,"EOI resetting bit ",ilevel, " on pic0");
        }
      else
@@ -567,8 +548,7 @@ Bit8u read_pic1(ioport_t port)
  *
  * DANG_END_FUNCTION
  */
-void pic_unmaski(level)
-int level;
+void pic_unmaski(int level)
 {
  pic_print(2,"Unmasking lvl= ",level,"");
  if(pic_iinfo[level].func != (void*)0) clear_bit(level,&pice_imr);
@@ -576,8 +556,7 @@ int level;
 }
 
 
-void pic_maski(level)
-int level;
+void pic_maski(int level)
 {
   pic_print(2,"Masking lvl= ",level,"");
   set_bit(level,&pice_imr);
@@ -587,32 +566,50 @@ int level;
 /* DANG_BEGIN_FUNCTION pic_seti
  *
  * pic_seti is used to initialize an interrupt for dosemu.  It requires
- * three parameters.  The first parameter is the interrupt level, which
- * man select the NMI, any of the IRQs, or any of the 16 extra levels
+ * four parameters.  The first parameter is the interrupt level, which
+ * may select the NMI, any of the IRQs, or any of the 16 extra levels
  * (16 - 31).  The second parameter is the dosemu function to be called
  * when the interrupt is activated.  This function should call do_irq()
- * if the DOS interruptis really to be activated.  If there is no special
+ * if the DOS interrupt is really to be activated.  If there is no special
  * dosemu code to call, the second parameter can specify do_irq(), but
- * see that description for some special considerations.
+ * see that description for some special considerations. 
+ * The third parameter is a number of an interrupt to activate if there is
+ * no default interrupt for this ilevel.
+ * The fourth parameter is the dosemu function to be called from do_irq().
+ * Required by some internal dosemu drivers that needs some additional code
+ * before calling an actual handler. This function MUST provide a EOI at
+ * the end of a callback.
  *
  * DANG_END_FUNCTION
  */
-void pic_seti(level,func,ivec)
-unsigned int level,ivec;
-void (*func);
+void pic_seti(unsigned int level, void (*func), unsigned int ivec,
+  void (*callback))
 {
   if(level>=32) return;
+  if(!func) {
+    error("pic_seti() called with no function for IRQ level %i\n", level);
+    leavedos(1);
+  }
+  if(pic_iinfo[level].func) {
+    if(pic_iinfo[level].func != func) {
+      error("Attempt to register more than one handler for IRQ level %i (%p %p)\n",
+        level, pic_iinfo[level].func, func);
+      config.exitearly = 1;
+    } else {
+      error("Handler for IRQ level %i was registered more than once! (%p)\n",
+        level, func);
+    }
+  }
   pic_iinfo[level].func = func;
+  if(callback) {
+    pic_iinfo[level].callback = callback;
+    set_bit(level, &pic_irqall);
+  }
   if(level>15) pic_iinfo[level].ivec = ivec;
   if(func == (void*)0) pic_maski(level);
 }
 
 
-/* this is the guts if the irq processor. it should be called from an if:
- * if(pic_irr) run_irqs();     
- * This will maximize speed.
- */
- 
 /* DANG_BEGIN_FUNCTION run_irqs
  *
  * run_irqs, which is initiated via the macro pic_run, is the "brains" of
@@ -625,126 +622,86 @@ void (*func);
  * assembly macros, because such macros can only return a single result.
  * If I find a way to do it in c, I will, but don't hold your breath.
  *
+ * I found a way to write it in C --EB 15 Jan 97
+ *
  * DANG_END_FUNCTION
  */
-#ifdef C_RUN_IRQS
-/* see assembly language version below */
-void run_irqs()
+/* DANG_BEGIN_COMMENT
+ *   This is my C version of the assembly version of run_irqs.
+ *   I believe it is a correct translation of the assembly into C.
+ *
+ *   I got frustrated with the old incomplete version.  Especially
+ *   because of indent problems and I don't believe in changing code I
+ *   don't understand.
+ *
+ *   This code is correct in the single process case.
+ *   Except possilby it's handling of special mask mode.
+ *   If pic_smm is set to 32 it appears to me that no interrupt is
+ *   ever run.
+ *
+ *   But this code is certainly incorrect in the multiple process
+ *   case.  Because it spins without rereading pic_irr, to recalculate
+ *   pic_ilevel.  And I think there are other less serious problems as
+ *   well.  However it doesn't matter because we aren't doing multiple
+ *   processes, or multiple threads. :)
+ *
+ *   Note Comments taken from the original C version
+ *
+ *   --EB 5 Jan 97
+ *  DANG_END_COMMENT
+ */
+void run_irqs(void)
 /* find the highest priority unmasked requested irq and run it */
 {
- int old_ilevel;
- int int_request;
-#warning using C run_irqs
+       int old_ilevel;
+       int int_request;
 
-/* check for and find any requested irqs.  Having found one, we atomic-ly
- * clear it and verify it was there when we cleared it.  If it wasn't, we
- * look for the next request.  There are two in_service bits for pic1 irqs.
- * This is needed, because irq 8-15 must do 2 outb20s, which, if the dos
- * irq code actually runs, will reset the bits.  We also reset them here,
- * since dos code won't necessarily run.  
- */
-   
-#if 0
- if(!(pic_irr&~(pic_isr|pic_imr))) return;     /* exit if nothing to do */
-#else
- if(!(pic_irr&~(pic_isr))) return;     /* exit if nothing to do */
-#endif
- old_ilevel=pic_ilevel;                          /* save old pic_ilevl   */
- 
-#if 0
- while(int_request= pic_irr&~(pic_isr|pic_imr)) /* while something to do*/
-#else
- while(int_request= pic_irr&~(pic_isr)) /* while something to do*/
-#endif
-  {
-      
-  pic_ilevel=find_bit(&int_request);              /* find out what it is  */
-      
-  if(pic_ilevel<(old_ilevel+pic_smm))            /* priority check       */
-    if(clear_bit(pic_ilevel,&pic_irr))           /* dbl check & clear req*/
-      {     
-      /* we got one!  it is identified by pic_ilevel */
+       /* don't allow HW interrupts in force trace mode */
+       if (vm86s.vm86plus.vm86dbg_TFpendig) return;
 
-     set_bit(pic_ilevel,&pic_isr);               /* set in-service bit   */
-     pic1_isr = pic_isr & pic1_mask;            /* pic1 too            */
-     pic_iinfo[pic_ilevel].func();               /* run the function     */
-     clear_bit(pic_ilevel,&pic_isr);             /* clear in_service bit */
-     clear_bit(pic_ilevel,&pic1_isr);            /* pic1 too            */
-     }
-   }
-   /* whether we did or didn't :-( get one, we must still reset pic_ilevel */
-   pic_ilevel=old_ilevel;
+       /* check for and find any requested irqs.  Having found one, we atomic-ly
+        * clear it and verify it was there when we cleared it.  If it wasn't, we
+        * look for the next request.  There are two in_service bits for pic1 irqs.
+        * This is needed, because irq 8-15 must do 2 outb20s, which, if the dos
+        * irq code actually runs, will reset the bits.  We also reset them here,
+        * since dos code won't necessarily run.
+        */
+
+       old_ilevel=pic_ilevel;                          /* save old pic_ilevl   */
+
+       while((int_request = pic_irr & ~(pic_isr | pic_imr)) != 0) { /* while something to do*/
+               int local_pic_ilevel;
+
+	       if (!isset_IF() && !in_dpmi)
+	    	       goto exit;                      /* exit if ints are disabled */
+
+               local_pic_ilevel = find_bit(int_request);    /* find out what it is  */
+               /* In case int_request has no bits set */
+               if (local_pic_ilevel == -1)
+                       goto exit;
+               if (local_pic_ilevel >= old_ilevel + pic_smm)  /* priority check */
+                       goto exit;
+
+               if (pic_irqs_active && local_pic_ilevel >= find_bit(pic_irqs_active))
+                       goto exit;
+
+               clear_bit(local_pic_ilevel, &pic_irr);
+               pic_ilevel = local_pic_ilevel;
+               set_bit(local_pic_ilevel, &pic_isr);     /* set in-service bit */
+               set_bit(local_pic_ilevel, &pic1_isr);    /* pic1 too */
+               pic1_isr &= pic_isr & pic1_mask;         /* isolate pic1 irqs */
+               pic_iinfo[local_pic_ilevel].func();      /* run the function */
+               clear_bit(local_pic_ilevel, &pic_isr);   /* clear in_service bit */
+               clear_bit(local_pic_ilevel, &pic1_isr);  /* pic1 too */
+	       pic_ilevel=old_ilevel;
+       }
+ exit:
+       /* whether we did or didn't :-( get one, we must still reset pic_ilevel */
+       pic_ilevel=old_ilevel;
+       if ((pic_ilevel==32 && pic_isr!=0) || (pic_ilevel!=32 && pic_ilevel!=find_bit(pic_isr)))
+         error("PIC: pic_ilevel=0x%x pic_isr=0x%x\n", pic_ilevel, pic_isr);
 }
 
-#else
-
-/* I got inspired.  Here's an assembly language version, somewhat faster.
- * This is also much shorter - fewer checks needed, because there's no
- * question of compiler behavior, and less code because we can use both 
- * results and flags from an instruction.
- * RE-ENTRANT CODE - uses 16 bytes of stack
- */
-void run_irqs()
-{
-
-#if 1 /* BUG FIXER (if 1), lets WIN31 run */
-if ((!(REG(eflags) & VIF)) && (!in_dpmi) ) {
-#if 0
-  g_printf("*");
-#endif
-  return;
-}
-#else
-if (!(REG(eflags) & VIF)) return;
-#endif
-
-/* Without this one dope crashes (DPMI_rm_procedure_* running increases 
- * up to a stack fault)
- *
- * This is a gross disgusting cludge, that I am going to make even grosser
- * and more disgusting in an attempt to get it to work properly.  Someday
- * I am going to find and fix the root problems that are making this 
- * necessary.  -KenC
- */
-if (in_dpmi && in_dpmi_timer_int) return;
-
-#warning using assembly run_irqs,expect pic1_mask not used
-  __asm__ __volatile__
-  ("movl "CISH_INLINE(pic_ilevel)",%%ecx\n\t" /* get old ilevel              */
-   "pushl %%ecx\n\t"                          /* save old ilevel             */
-   "addl "CISH_INLINE(pic_smm)",%%ecx\n\t"    /* check spec. mask mode       */
-                                              /***  while(.....){            */
-   "1:movl "CISH_INLINE(pic_isr)",%%eax\n\t" /* 1: int_request = (pic_isr   */
-   "orl "CISH_INLINE(pic_imr)",%%eax\n\t"     /* | pic_imr)                  */
-   "notl %%eax\n\t"                           /* int_request = ~int_request  */
-   "andl "CISH_INLINE(pic_irr)",%%eax\n\t"    /* int_request &= pic_irr      */
-   "jz 3f\n\t"                                /* if(!int_request) goto 3     */
-   "2:bsfl %%eax,%%ebx\n\t"                  /* 2: ebx=find_bit(int_request)*/
-   "jz 3f\n\t"                                /* if(!int_request) goto 3     */
-   "cmpl %%ebx,%%ecx\n\t"                     /* if(ebx > pic_ilevel)...     */
-   "jle 3f\n\t"                               /* ... goto 3                  */
-   "btrl %%ebx,"CISH_INLINE(pic_irr)"\n\t"    /* clear_bit(ebx,&pic_irr)     */
-   "jnc 2b\n\t"                               /* if bit wasn't set, go to 2  */
-   "movl "CISH_INLINE(pic_isr)",%%ecx\n\t"    /* get current pic_isr         */
-   "btsl %%ebx,%%ecx\n\t"                     /* set bit in pic_isr          */
-   "movl %%ebx,"CISH_INLINE(pic_ilevel)"\n\t" /* set new ilevel              */
-   "movl %%ecx,"CISH_INLINE(pic_isr)"\n\t"    /* save new isr                */
-   "andl "CISH_INLINE(pic1_mask)",%%ecx\n\t"  /* isolate pic1 irqs           */
-   "movl %%ecx,"CISH_INLINE(pic1_isr)"\n\t"   /* save in pic1_isr            */
-   "call *"CISH_INLINE(pic_iinfo)"(,%%ebx,8)\n\t" /* call interrupt handler  */
-   "movl "CISH_INLINE(pic_ilevel)",%%eax\n\t" /* get new ilevel              */
-   "btrl %%eax,"CISH_INLINE(pic_isr)"\n\t"    /* reset isr bit - just in case*/
-   "btrl %%eax,"CISH_INLINE(pic1_isr)"\n\t"   /* reset isr bit - just in case*/
-   "jmp 1b\n\t"                               /* go back for next irq        */
-                                              /**** end of while   }      ****/
-   "3:popl "CISH_INLINE(pic_ilevel)"\n\t"    /* 3: restore old ilevel & exit*/
-   :                                          /* no output                   */
-   :                                          /* no input                    */
-   :"eax","ebx","ecx");                       /* registers eax, ebx, ecx used*/
-
- } 
-#endif
-               
    
 /* DANG_BEGIN_FUNCTION do_irq
  *
@@ -773,34 +730,31 @@ if (in_dpmi && in_dpmi_timer_int) return;
  *
  * DANG_END_FUNCTION
  */
-int do_irq()
+int do_irq(void)
 {
  int intr;
+ unsigned long ilevel;
  unsigned char * ssp;
  unsigned long sp;
+#ifdef X86_EMULATOR
+ unsigned char *tmp_ssp;
+ int tmp;
+#endif
 
     if(pic_ilevel==32) return 0;
-    intr=pic_iinfo[pic_ilevel].ivec;
+    ilevel=pic_ilevel;
+    intr=pic_iinfo[ilevel].ivec;
 
-    if(pic_ilevel==PIC_IRQ9)      /* unvectored irq9 just calls int 0x0a.. */
+    if(ilevel==PIC_IRQ9)      /* unvectored irq9 just calls int 0x0a.. */
       if(!IS_REDIRECTED(intr)) {intr=0x0a;pic1_isr&= 0xffef;} /* & one EOI */
-#ifndef USE_NEW_INT
-    if(IS_REDIRECTED(intr)||pic_ilevel<=PIC_IRQ1||in_dpmi)
-#endif /* not USE_NEW_INT */
-    {
-#if 0 /* BUG CATCHER (if 1) */
-/* outputting more then one character here will change dynamic behave such
- * that we measure the wrong thing.
- */
-g_printf("+%d",(int)pic_ilevel);
-#endif
-     if(pic_ilevel < 16) pic_push(pic_ilevel);
-     if (in_dpmi) {
-      run_pm_int(intr);
-     } else {
-#ifndef USE_NEW_INT
-      pic_cli();
-#endif /* not USE_NEW_INT */
+
+     if (test_bit(ilevel, &pic_irqall)) {
+       pic_push(ilevel);
+       set_bit(ilevel, &pic_irqs_active);
+       pic_icount++;
+     }
+
+     if (!in_dpmi || in_dpmi_dos_int) {
       ssp = (unsigned char *)(LWORD(ss)<<4);
       sp = (unsigned long) LWORD(esp);
 
@@ -809,46 +763,51 @@ g_printf("+%d",(int)pic_ilevel);
   * change esp first to protect the stack we're about to use
   */
       LWORD(esp) = (LWORD(esp) - 4) & 0xffff;  
+#ifdef X86_EMULATOR
+      tmp_ssp = ssp+sp;
+      tmp = E_MUNPROT_STACK(tmp_ssp);
+#endif
       pushw(ssp, sp, REG(cs));
       pushw(ssp, sp, LWORD(eip));
+#ifdef X86_EMULATOR
+      if (tmp) E_MPROT_STACK(tmp_ssp);
+#endif
+      if(debug_level('r')>7) r_printf("PIC: setting iret trap at %04x:%04lx\n",
+        REG(cs), REG(eip));
       REG(cs) = PIC_SEG;
       LWORD(eip) = PIC_OFF;
-      
- /* schedule the requested interrupt, then enter the vm86() loop */
-      run_int(intr);
      }
-      pic_icount++;
-      pic_wcount++;
+
+     if (pic_iinfo[ilevel].callback)
+        pic_iinfo[ilevel].callback();
+     else {
+       if (in_dpmi) run_pm_int(intr);
+       else {
+ /* schedule the requested interrupt, then enter the vm86() loop */
+         run_int(intr);
+       }
 
  /* enter PIC loop - we can continue only when pic_isr has been cleared */
-      while(!fatalerr && test_bit(pic_ilevel,&pic_isr))
+      while(!fatalerr && test_bit(ilevel,&pic_isr))
       {
-	if (d.request>2)
-		r_printf("------ PIC: intr loop ---%06lx--%06lx----\n",
+	if (debug_level('r')>2)
+		r_printf("------ PIC: intr loop ---%06x--%06x----\n",
 			pic_vm86_count,pic_dpmi_count);
 	if (in_dpmi ) {
           ++pic_dpmi_count;
-	  pic_print(2, "Initiating DPMI irq lvl ", pic_ilevel, " in do_irq");
+	  pic_print(2, "Initiating DPMI irq lvl ", ilevel, " in do_irq");
 	  run_dpmi();
-	  }
+	}
 	else {
 	  ++pic_vm86_count;
-	  pic_print(2, "Initiating VM86 irq lvl ", pic_ilevel, " in do_irq");
+	  pic_print(2, "Initiating VM86 irq lvl ", ilevel, " in do_irq");
           run_vm86();
-          }
-        pic_isr &= PIC_IRQALL;    /*  levels 0 and 16-31 are Auto-EOI  */
+        }
+	if (!test_bit(ilevel,&pic_isr))
+	  break;
+
         serial_run();           /*  delete when moved to timer stuff */
-        if(pic_irr) run_irqs();
-#if 0
-#ifdef USING_NET
-        /* check for available packets on the packet driver interface */
-        /* (timeout=0, so it immediately returns when none are available) */
-        pkt_check_receive(0);
-#endif
-#endif
-#ifdef USE_INT_QUEUE
-        int_queue_run();        /*  delete when moved to timer stuff */
-#endif
+        pic_run();
       }
       pic_sti();
       return(0);
@@ -857,6 +816,48 @@ g_printf("+%d",(int)pic_ilevel);
     return(1);
 }
 
+/* DANG_BEGIN_FUNCTION pic_resched
+ *
+ * pic_resched decrements a count of interrupts on the stack
+ * (set by do_irq()). If the count is then less or equal to some pre-defined
+ * value (normally 1, pic_icount_od), pic_resched moves all queued interrupts 
+ * to the interrupt request register.
+ *
+ * Normally it is called from pic_iret(), but it can also be called 
+ * directly if dosemu was fooled by the program and failed to catch iret.
+ *
+ * DANG_END_FUNCTION
+ */
+void
+pic_resched(void) 
+{
+unsigned long pic_newirr, pic_last_ilevel;
+    if(pic_icount) {
+       pic_icount--;
+       pic_last_ilevel = pic_pop();
+       if (pic_icount != pic_sp)
+         error("pic_icount=%i != pic_sp=%i\n", pic_icount, pic_sp);
+       if (!test_bit(pic_last_ilevel, &pic_irqs_active)) {
+         error("PIC: bit %li is not set, irqs_active=%lx\n",
+	   pic_last_ilevel, pic_irqs_active);
+	 pic_irqs_active = 0;
+       }
+       clear_bit(pic_last_ilevel, &pic_irqs_active);
+    } else if (pic_irqs_active) {
+      error("pic_icount=%i and pic_irqs_active=%lx are out of sync!\n",
+        pic_icount, pic_irqs_active);
+      pic_irqs_active = 0;
+    }
+    if(pic_icount<=pic_icount_od) {
+	pic_newirr=pic_pirr&~pic_irr&~pic_isr;
+        pic_irr|=pic_newirr;
+        pic_pirr&=~pic_newirr;
+        pic_wirr&=~pic_newirr;        /* clear watchdog timer */
+	pic_activate();
+    }
+    if (!pic_icount)
+      pic_wcount = 0;
+}
 
 /* DANG_BEGIN_FUNCTION pic_request
  *
@@ -873,8 +874,7 @@ g_printf("+%d",(int)pic_ilevel);
  *
  * DANG_END_FUNCTION
  */
-int pic_request(inum)
-int inum;
+int pic_request(int inum)
 {
 static char buf[81];
   int ret=PIC_REQ_NOP;
@@ -911,7 +911,7 @@ static char buf[81];
     pic_ltime[inum] = pic_itime[inum];
     ret=PIC_REQ_OK;
   }
-  if (d.request&2) {
+  if (debug_level('r') >2) {
     /* avoid going through sprintf for non-debugging */
     sprintf(buf,", k%d",(int)pic_dpmi_count);
     pic_print(2,"Zeroing vm86, DPMI from ",pic_vm86_count,buf);
@@ -920,6 +920,15 @@ static char buf[81];
   return ret;
 }
 
+void pic_untrigger(int inum)
+{
+    if ((pic_irr | pic_pirr) & (1<<inum)) {
+      pic_print(2,"Requested irq lvl ", inum, " untriggered");
+    }
+    pic_pirr &= ~(1<<inum);
+    pic_irr &= ~(1<<inum);
+    pic_wirr &= ~(1<<inum);
+}
 
 /* DANG_BEGIN_FUNCTION pic_iret
  *
@@ -930,64 +939,68 @@ static char buf[81];
  * not been restored to its initial state by an iret.  pic_iret is called
  * whenever interrupts have been enabled by a popf, sti, or iret.  It 
  * determines if an iret was the cause by comparing stack contents with
- * cs and ip.  If so, it decrements a count of interrupts on the stack
- * (set by do_irq()).  If the count is then zero, pic_iret moves all queued
- * interrupts to the interrupt request register.  It is possible for pic_iret
- * to be fooled by dos code; for this reason active interrupts are checked,
- * any queued interrupts that are also active will remain queued.  Also,
- * some programs fake an iret, so that it is possible for pic_iret to fail.
+ * cs and ip. If so, it calls pic_resched() and does the actual iret by
+ * pop'ing ip and cs from stack.
+ * It is possible for pic_iret to be fooled by dos code; for this reason 
+ * active interrupts are checked, any queued interrupts that are also 
+ * active will remain queued.
+ * Also, some programs fake an iret, so that it is possible for pic_iret 
+ * to fail.
  * See pic_watch for the watchdog timer that catches and fixes this event.
  *
  * DANG_END_FUNCTION
  */
 void
-pic_iret()
+pic_iret(void)
 {
 unsigned char * ssp;
 unsigned long sp;
-unsigned long pic_newirr;
 
-  if(in_dpmi) {
-    if(pic_icount) {
-       pic_icount--;
-       pic_newirr=pic_pirr&~pic_irr&~pic_isr;
-       pic_irr|=pic_newirr;
-       pic_pirr&=~pic_newirr;
-       pic_wirr&=~pic_newirr;        /* clear watchdog timer */
-       if(pic_icount<=pic_icount_od)
-	  pic_activate();
-       if(!pic_irr) dpmi_eflags &= ~VIP;
-    } 
-     pic_print(2,"IRET in dpmi, loops=",pic_dpmi_count," ");
-     pic_dpmi_count=0;
-  return;
+  if(in_dpmi && !in_dpmi_dos_int) {
+/* Even if cs:ip now points to PIC_SEG:PIC_OFF hlt, it means nothing while
+ * in protected mode, so we can't modify it here.
+ */
+    pic_resched();
+    pic_print(2,"IRET in dpmi, loops=",pic_dpmi_count," ");
+    pic_dpmi_count=0;
+    if ((cb_cs || cb_ip) && !pic_icount) {
+      r_printf("PIC: entering callback at %x:%x\n", cb_cs, cb_ip);
+      fake_pm_int();
+     /* we will have an extra pic_iret() after this, but no problems
+      * since pic_icount==0
+      */
+      fake_call(REG(cs), LWORD(eip));
+      LWORD(eip) = cb_ip;
+      REG(cs) = cb_cs;
+      cb_cs = cb_ip = 0;
+    }
   }
-
+  else {
 /* if we've really come from an irq, cs:ip will point to PIC_SEG:PIC_OFF */
 /* if we haven't come from an irq and cs:ip points as above, we're going to
  * die anyway, since our next instruction is a hlt.
  */ 
- if(REG(cs) == PIC_SEG)
-   if(LWORD(eip) == PIC_OFF) {
-    if(pic_icount) { 
-       pic_icount--;
-       pic_newirr=pic_pirr&~pic_irr&~pic_isr;
-       pic_irr|=pic_newirr;
-       pic_pirr&=~pic_newirr;
-       pic_wirr&=~pic_newirr;        /* clear watchdog timer */
-       if(pic_icount<=pic_icount_od)
-	  pic_activate();
-       if(!pic_irr) REG(eflags)&=~(VIP);
+    if(REG(cs) == PIC_SEG && LWORD(eip) == PIC_OFF) {
+      pic_resched();
+      pic_print(2,"IRET in vm86, loops=", in_dpmi?
+      pic_dpmi_count : pic_vm86_count," ");
+      pic_vm86_count=0;
+      pic_dpmi_count=0;
+      if ((cb_cs || cb_ip) && !pic_icount) {
+	r_printf("PIC: entering callback at %x:%x\n", cb_cs, cb_ip);
+	LWORD(eip) = cb_ip;
+	REG(cs) = cb_cs;
+	cb_cs = cb_ip = 0;
+      }
+      else {
+	ssp = (unsigned char *)(LWORD(ss)<<4);
+	sp = (unsigned long) LWORD(esp);
+	LWORD(eip) = popw(ssp, sp);
+	REG(cs) = popw(ssp, sp);
+	LWORD(esp) = (LWORD(esp) + 4) & 0xffff;
+      }
     }
-     pic_print(2,"IRET in vm86, loops=",pic_vm86_count," ");
-     pic_vm86_count=0;
-      ssp = (unsigned char *)(LWORD(ss)<<4);
-      sp = (unsigned long) LWORD(esp);
-      LWORD(eip) = popw(ssp, sp);
-      REG(cs) = popw(ssp, sp);
-      LWORD(esp) = (LWORD(esp) + 4) & 0xffff;
-    }
- return;
+  }
 }
 
  
@@ -1001,8 +1014,7 @@ unsigned long pic_newirr;
  *
  * DANG_END_FUNCTION
  */
-inline void pic_watch(s_time)
-hitimer_u *s_time;	/* time in us, 64-bit unsigned */
+inline void pic_watch(hitimer_u *s_time)
 {
 hitimer_t t_time;
 unsigned long pic_newirr;
@@ -1015,10 +1027,9 @@ unsigned long pic_newirr;
 /*  pic_activate(); */
 
 /*  calculate new sys_time
- *  non-monoton: values are kept modulo (tick_rate*15min)
- *  monoton:     values are kept modulo 2^32 (exactly 1 hour)
+ *  values are kept modulo 2^32 (exactly 1 hour)
  */
-  t_time = UStoTICK(s_time->td);
+  t_time = s_time->td;
 
   /* check for any freshly initiated timers, and sync them to s_time */
   pic_print(2,"pic_itime[1]= ",pic_itime[1]," ");
@@ -1026,6 +1037,15 @@ unsigned long pic_newirr;
   pic_print(2,"pic_sys_time set to ",pic_sys_time," ");
   pic_dos_time = pic_itime[32];
   if(pic_icount<=pic_icount_od) pic_activate();
+  if(config.pic_watchdog > 0) {
+    if(pic_icount && !pic_isr) {
+      if(++pic_wcount >= config.pic_watchdog) {
+        r_printf("PIC: force reschedule\n");
+        pic_resched();
+        pic_wcount = 0;
+      }
+    }
+  }
 }      
 
 
@@ -1057,7 +1077,7 @@ int pic_pending(int ilevel)
  * pic_dos_time is advanced to the earliest time scheduled.
  * DANG_END_FUNCTION
  */
-void pic_activate()
+static void pic_activate(void)
 {
 hitimer_t earliest;
 int timer, count;
@@ -1077,6 +1097,7 @@ int timer, count;
    }
    if(count) pic_print(2,"Activated ",count, " interrupts.");
    pic_print(2,"Activate ++ dos time to ",earliest, " ");
+   pic_print(2,"pic_sys_time is ",pic_sys_time," ");
    /*if(!pic_icount)*/ pic_dos_time = pic_itime[32] = earliest;
 }
 
@@ -1127,7 +1148,7 @@ void pic_sched(int ilevel, int interval)
 	pic_itime[ilevel] = pic_itime[ilevel] + interval;
      }
   }
-  if (d.request > 2) {
+  if (debug_level('r') > 2) {
     /* avoid going through sprintf for non-debugging */
     sprintf(mesg,", delay= %d.",interval);
     pic_print(2,"Scheduling lvl= ",ilevel,mesg);
@@ -1135,6 +1156,14 @@ void pic_sched(int ilevel, int interval)
   }
 }
 
+void pic_set_callback(Bit16u cs, Bit16u ip)
+{
+  r_printf("PIC: setting callback to %x:%x (pic_icount=%u)\n",
+    cs, ip, pic_icount);
+  cb_ip = ip;
+  cb_cs = cs;
+}
+
 void pic_init(void)
 {
   /* do any one-time initialization of the PIC */
@@ -1164,55 +1193,4 @@ void pic_init(void)
 
 void pic_reset(void)
 {
-#if 0
-  int i;
-
-  pic_ilevel     = 32;
-  pic_isr        = 0;
-  pic_irq2_ivec  = 0;
-  pic1_mask      = 0x07f8;
-  pic0_imr       = 0xf800;
-  pic1_imr       = 0x07f8;
-  pic_imr        = 0xfff8;
-  pice_imr       = 0xffffffff;
-  pic_sp         = 0;
-  pic_vm86_count = 0;
-  pic_dpmi_count = 0;
-  pic_sys_time   = NEVER;
-  for (i = 0; i < 33; i++) {
-    pic_ltime[i] = NEVER;
-    pic_itime[i] = NEVER;
-  }
-
-  /* PIC reset on reboot and at bootup */
-  pic_seti(PIC_IRQ0, timer_int_engine, 0);  /* do_irq0 in pic.c */
-  pic_unmaski(PIC_IRQ0);
-  pic_request(PIC_IRQ0);                    /* start timer */
-  if (mice->intdrv || mice->type == MOUSE_PS2 || mice->type == MOUSE_IMPS2) {
-    pic_seti(PIC_IMOUSE, DOSEMUMouseEvents, 0);
-    pic_unmaski(PIC_IMOUSE);
-  }
-#ifdef USING_NET
-#ifdef CONFIG_IPX
-  pic_seti(PIC_IPX, IPXRelinquishControl, 0);
-  pic_unmaski(PIC_IPX);
-#endif
-#ifdef CONFIG_IPXPICPKT
-  pic_seti(PIC_NET, pkt_check_receive_quick, 0x61);
-#else
-  pic_seti(PIC_NET, pkt_check_receive_quick, 0);
-#endif
-  pic_unmaski(PIC_NET);
-#endif
-#endif
 }
-
-#undef set_pic0_imr
-#undef set_pic1_imr
-#undef get_pic0_imr
-#undef get_pic1_imr
-#undef get_pic0_isr
-#undef get_pic1_isr
-#undef get_pic0_irr
-#undef get_pic1_irr
-#undef NEVER
