@@ -1,11 +1,12 @@
 /* 
- * (C) Copyright 1992, ..., 2000 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2004 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 
 #include "emu.h"
 #include "memory.h"
@@ -41,7 +42,7 @@ int memcheck_addtype(unsigned char map_char, char *name)
     if (strcmp(mem_names[map_char], name) != 0) {
       c_printf("CONF: memcheck, conflicting map type '%c' defined for '%s' \
 & '%s'\n", map_char, mem_names[map_char], name);
-      leavedos(1);
+      config.exitearly = 1;
     }
     else
       c_printf("CONF: memcheck, map type '%c' re-defined for '%s'\n",
@@ -77,7 +78,7 @@ void memcheck_reserve(unsigned char map_char, int addr_start, int size)
 	fprintf(stderr, "    '%s' & '%s'\n", mem_names[map_char],
 		mem_names[mem_map[cntr]]);
 	memcheck_dump();
-	leavedos(1);
+	config.exitearly = 1;
       }
     }
     else
@@ -85,7 +86,7 @@ void memcheck_reserve(unsigned char map_char, int addr_start, int size)
   }
 }
 
-void memcheck_type_init()
+void memcheck_type_init(void)
 {
   static int once = 0;
   if (once) return;
@@ -95,7 +96,7 @@ void memcheck_type_init()
   memcheck_addtype('h', "Direct-mapped hardware page frame");
 }
 
-void memcheck_init()
+void memcheck_init(void)
 {
   memcheck_type_init();
   memcheck_reserve('d', 0x00000, config.mem_size*1024); /* dos memory  */
