@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 2001 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2004 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -18,11 +18,10 @@
 #include "emu-ldt.h"
 #include "dpmi.h"
 #include "int.h"
-
-extern int  dis_8086(unsigned int, const unsigned char *,
-                     unsigned char *, int, unsigned int *, unsigned int *,
-                     unsigned int, int);
-
+#ifdef X86_EMULATOR
+#include "cpu-emu.h"
+#endif
+#include "dis8086.h"
 
 char *emu_disasm(int sga, unsigned int ip)
 {
@@ -68,7 +67,7 @@ show_regs(char *file, int line)
   unsigned char *sp;
   unsigned char *cp;
 
-  if (!d.general)
+  if (debug_level('g') == 0)
     return;
 
   cp = SEG_ADR((unsigned char *), cs, ip);
@@ -127,22 +126,26 @@ show_regs(char *file, int line)
   /* display the 10 bytes before and after CS:EIP.  the -> points
    * to the byte at address CS:EIP
    */
-  g_printf("STACK: ");
-  sp -= 10;
-  for (i = 0; i < 10; i++)
-    g_printf("%02x ", *sp++);
-  g_printf("-> ");
-  for (i = 0; i < 10; i++)
-    g_printf("%02x ", *sp++);
-  g_printf("\n");
-  g_printf("OPS  : ");
-  cp -= 10;
-  for (i = 0; i < 10; i++)
-    g_printf("%02x ", *cp++);
-  g_printf("-> ");
-  for (i = 0; i < 10; i++)
-    g_printf("%02x ", *cp++);
-  g_printf("\n\t%s\n", emu_disasm(1,0));
+  if (((unsigned) sp < 0xa0000) && ((unsigned)sp >10)) {
+	  g_printf("STACK: ");
+	  sp -= 10;
+	  for (i = 0; i < 10; i++)
+		  g_printf("%02x ", *sp++);
+	  g_printf("-> ");
+	  for (i = 0; i < 10; i++)
+		  g_printf("%02x ", *sp++);
+	  g_printf("\n");
+  }
+  if (((unsigned) cp < 0xa0000) && ((unsigned)cp>10)) {
+	  g_printf("OPS  : ");
+	  cp -= 10;
+	  for (i = 0; i < 10; i++)
+		  g_printf("%02x ", *cp++);
+	  g_printf("-> ");
+	  for (i = 0; i < 10; i++)
+		  g_printf("%02x ", *cp++);
+	  g_printf("\n\t%s\n", emu_disasm(1,0));
+  }
 }
 
 void
@@ -152,29 +155,33 @@ show_ints(int min, int max)
 
   max = (max - min) / 3;
   for (i = 0, b = min; i <= max; i++, b += 3) {
-    g_printf("%02x| %04x:%04x->%05x    ", b, ISEG(b), IOFF(b),
+    g_printf("%02x| %04x:%04x->%06x   ", b, ISEG(b), IOFF(b),
 		IVEC(b));
-    g_printf("%02x| %04x:%04x->%05x    ", b + 1, ISEG(b + 1), IOFF(b + 1),
+    g_printf("%02x| %04x:%04x->%06x   ", b + 1, ISEG(b + 1), IOFF(b + 1),
 		IVEC(b + 1));
-    g_printf("%02x| %04x:%04x->%05x\n", b + 2, ISEG(b + 2), IOFF(b + 2),
+    g_printf("%02x| %04x:%04x->%06x\n", b + 2, ISEG(b + 2), IOFF(b + 2),
 		IVEC(b + 2));
   }
 }
 
+#if MAX_SELECTORS != 8192
+#error MAX_SELECTORS needs to be 8192
+#endif
 
-#define GetSegmentBaseAddress(s)	(((s) >= (MAX_SELECTORS << 3))? 0 :\
-					Segments[(s) >> 3].base_addr)
+#define GetSegmentBaseAddress(s)	(Segments[(s) >> 3].base_addr)
+#define IsSegment32(s)			(Segments[(s) >> 3].is_32)
 
-void DPMI_show_state(struct sigcontext_struct *scp)
+char *DPMI_show_state(struct sigcontext_struct *scp)
 {
+    static char buf[4096];
     unsigned char *csp2, *ssp2;
-    D_printf("eip: 0x%08lx  esp: 0x%08lx  eflags: 0x%08lx\n"
-	     "trapno: 0x%02lx  errorcode: 0x%08lx  cr2: 0x%08lx\n"
-	     "cs: 0x%04x  ds: 0x%04x  es: 0x%04x  ss: 0x%04x  fs: 0x%04x  gs: 0x%04x\n",
+    sprintf(buf, "eip: 0x%08lx  esp: 0x%08lx  eflags: 0x%08lx\n"
+	     "\ttrapno: 0x%02lx  errorcode: 0x%08lx  cr2: 0x%08lx\n"
+	     "\tcs: 0x%04x  ds: 0x%04x  es: 0x%04x  ss: 0x%04x  fs: 0x%04x  gs: 0x%04x\n",
 	     _eip, _esp, _eflags, _trapno, _err, _cr2, _cs, _ds, _es, _ss, _fs, _gs);
-    D_printf("EAX: %08lx  EBX: %08lx  ECX: %08lx  EDX: %08lx\n",
+    sprintf(buf, "%sEAX: %08lx  EBX: %08lx  ECX: %08lx  EDX: %08lx\n", buf,
 	     _eax, _ebx, _ecx, _edx);
-    D_printf("ESI: %08lx  EDI: %08lx  EBP: %08lx\n",
+    sprintf(buf, "%sESI: %08lx  EDI: %08lx  EBP: %08lx\n", buf,
 	     _esi, _edi, _ebp);
     /* display the 10 bytes before and after CS:EIP.  the -> points
      * to the byte at address CS:EIP
@@ -191,15 +198,19 @@ void DPMI_show_state(struct sigcontext_struct *scp)
      * 'not present' error and then we try accessing the code/stack
      * area, we fall into another fault which likely terminates dosemu.
      */
+#ifdef X86_EMULATOR
+    if (!config.cpuemu || (_trapno!=0x0b && _trapno!=0x0c)) {
+#else
     if (1) {
+#endif
       int i;
-      D_printf("OPS  : ");
+      sprintf(buf, "%sOPS  : ", buf);
       for (i = 0; i < 10; i++)
-        D_printf("%02x ", *csp2++);
-      D_printf("-> ");
+        sprintf(buf, "%s%02x ", buf, *csp2++);
+      sprintf(buf, "%s-> ", buf);
       for (i = 0; i < 10; i++)
-        D_printf("%02x ", *csp2++);
-      D_printf("\n");
+        sprintf(buf, "%s%02x ", buf, *csp2++);
+      sprintf(buf, "%s\n", buf);
       if (!((_ss) & 0x0004)) {
         /* GDT */
         ssp2 = (unsigned char *) _esp - 10;
@@ -211,14 +222,16 @@ void DPMI_show_state(struct sigcontext_struct *scp)
         else
 	  ssp2 = (unsigned char *) (GetSegmentBaseAddress(_ss) + _LWORD(esp) ) - 10;
       }
-      D_printf("STACK: ");
+      sprintf(buf, "%sSTACK: ", buf);
       for (i = 0; i < 10; i++)
-        D_printf("%02x ", *ssp2++);
-      D_printf("-> ");
+        sprintf(buf, "%s%02x ", buf, *ssp2++);
+      sprintf(buf, "%s-> ", buf);
       for (i = 0; i < 10; i++)
-        D_printf("%02x ", *ssp2++);
-      D_printf("\n");
+        sprintf(buf, "%s%02x ", buf, *ssp2++);
+      sprintf(buf, "%s\n", buf);
     }
+
+    return buf;
 }
 
 /* @@@ MOVE_END @@@ 32768 */
