@@ -107,7 +107,7 @@ static int sent_mouse_esc = FALSE;
 
 static mouse_t *mice = &config.mouse;
 /* the 'volatile' is there to cover some bug in gcc -O -g3 */
-static volatile struct mouse_struct mouse;
+volatile struct mouse_struct mouse;
 
 static inline int mouse_roundx(int x) 
 { 
@@ -331,7 +331,7 @@ int
 mouse_int(void)
 {
   /* delayed mouse init for xterms; should be done cleaner in 1.3.x */
-  if (mice->type == MOUSE_XTERM) {
+  if (mice->type == MOUSE_XTERM && !sent_mouse_esc) {
     /* save old highlight mouse tracking */
     printf("\033[?1001s");
     /* enable mouse tracking */
@@ -1031,8 +1031,6 @@ static void mouse_reset(int flag)
   memcpy((void *)mouse.graphcursormask,default_graphcursormask,32);
   mouse.hotx = mouse.hoty = -1;
 
-  mouse.enabled = TRUE;
-
   mouse_do_cur();
 }
 
@@ -1868,7 +1866,14 @@ dosemu_mouse_init(void)
 #ifdef USE_GPM
       !Mouse_gpm.init() &&
 #endif
-      !Mouse_xterm.init()) {
+#ifdef HAVE_KEYBOARD_V1
+      /* old keyboard plugin has the terminal code included without
+         xterm mouse support */
+      1
+#else
+      !Mouse_xterm.init()
+#endif
+     ) {
     if (mice->intdrv) {
       struct stat buf;
       m_printf("Opening internal mouse: %s\n", mice->dev);
@@ -1904,6 +1909,9 @@ dosemu_mouse_init(void)
     }
   }
 
+  /* Set to disabled, will be enabled in post_boot() */
+  mouse.enabled = FALSE;
+
   if (mice->type != MOUSE_X) {
     mouse.init_speed_x = 8;
     mouse.init_speed_y = 16;
@@ -1912,7 +1920,9 @@ dosemu_mouse_init(void)
     mouse.init_speed_x = 8;
     mouse.init_speed_y = 8;
   }
-  
+  mouse.speed_x = mouse.init_speed_x;
+  mouse.speed_y = mouse.init_speed_y;
+
   if (mice->intdrv) {
     pic_seti(PIC_IMOUSE, DOSEMUMouseEvents, 0, do_mouse_irq);
     pic_unmaski(PIC_IMOUSE);
@@ -1985,6 +1995,8 @@ int parent_open_mouse (void)
 #endif
 	  ) return 1;
 
+      if (!mice->dev || !strlen(mice->dev))
+        return 0;
       stat(mice->dev, &buf);
       mode = O_RDWR | O_NONBLOCK;
       if (S_ISFIFO(buf.st_mode) || mice->type == MOUSE_BUSMOUSE || mice->type == MOUSE_PS2) {
@@ -2018,6 +2030,7 @@ dosemu_mouse_close(void)
   int result;
 
   if (mice->type == MOUSE_X) return;   
+#ifndef HAVE_KEYBOARD_V1
   if (mice->type == MOUSE_XTERM) {
     if (sent_mouse_esc) {
       Mouse_xterm.close();
@@ -2025,6 +2038,7 @@ dosemu_mouse_close(void)
     }
     return;
   }
+#endif
 #ifdef USE_GPM
   if (mice->type == MOUSE_GPM) {
     Mouse_gpm.close();
