@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 2001 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2004 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -57,9 +57,9 @@
  * Just type 'make' in the dosemu dir and it will recompile the changes only.
  * DANG_END_REMARK
  */
-#define SER_DEBUG_MAIN       0		/* 0 or 1 */
-#define SER_DEBUG_HEAVY      0		/* 0 or 1 */
-#define SER_DEBUG_INTERRUPT  0		/* 0 or 1 */
+#define SER_DEBUG_MAIN       1		/* 0 or 1 */
+#define SER_DEBUG_HEAVY      1		/* 0 or 1 */
+#define SER_DEBUG_INTERRUPT  1		/* 0 or 1 */
 #define SER_DEBUG_FOSSIL_RW  0       	/* 0 or 1 */
 #define SER_DEBUG_FOSSIL_STATUS 0	/* 0 or 1 */
 
@@ -306,5 +306,9 @@ void fossil_int14(int);
 void ser_termios(int num);
 void modstat_engine(int num);
 inline int msr_compute_delta_bits(int oldmsr, int newmsr);
+int ser_open(int num);
+void ser_set_params(int num);
+void receive_engine(int num);
+void transmit_engine(int num);
 
 #endif /* SER_DEFS_H */
