@@ -1,6 +1,6 @@
 /* 
  * All modifications in this file to the original code are
- * (C) Copyright 1992, ..., 1999 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2004 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -33,7 +33,6 @@
  *
  * DANG_END_MODULE
  */
-#include <features.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -45,17 +44,8 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <errno.h>
-#ifdef __linux__
-#if GLIBC_VERSION_CODE < 1000
-#include <linux/fs.h>
-#else
 #include <limits.h>
-#endif
-#endif
 #include <pwd.h>
-#ifdef __NetBSD__
-#include <sys/syslimits.h>		/* PATH_MAX */
-#endif
 
 #include "config.h"
 #include "emu.h"
@@ -65,8 +55,10 @@
 #include "pic.h"
 #include "serial.h"
 #include "ser_defs.h"
-#include "priv.h"
 #include "utilities.h"	/* due to getpwnam */
+#include "iodev.h"
+
+int no_local_video = 0;
 
 /* See README.serial file for more information on the com[] structure 
  * The declarations for this is in ../include/serial.h
@@ -117,23 +109,13 @@ static int tty_already_locked(char *nam)
  */
 static int tty_lock(char *path, int mode)
 {
-  PRIV_SAVE_AREA
   char saved_path[PATH_MAX];
   char dev_nam[20];
   struct passwd *pw;
   pid_t ime;
   int cwrote;
 
-  /* Check that lockfiles can be created! */
-  if((mode == 1 || mode == 2) && get_orig_euid() != (uid_t)0) {
-    s_printf("DOSEMU: Need to be suid root to create Lock Files!\n"
-	     "        Serial port on %s not configured!\n", path);
-    error("\nDOSEMU: Need to be suid root to create Lock Files!\n"
-	  "        Serial port on %s not configured!\n", path);
-    return(-1);
-  }
-
-  bzero(dev_nam, sizeof(dev_nam));
+  memset(dev_nam, 0, sizeof(dev_nam));
   sprintf(saved_path, "%s/%s%s", config.tty_lockdir, config.tty_lockfile, 
          (strrchr(path, '/')+1));
   strcpy(dev_nam, path);
@@ -143,16 +125,14 @@ static int tty_lock(char *path, int mode)
     {
       FILE *fd;
       if (tty_already_locked(saved_path) == 1) {
-        s_printf("DOSEMU: attempt to use already locked tty %s\n", saved_path);
-        error("\nDOSEMU: attempt to use already locked tty %s\n", saved_path);
+        s_printf("attempt to use already locked tty %s\n", saved_path);
+        error("attempt to use already locked tty %s\n", saved_path);
         return (-1);
       }
-      enter_priv_on();
       fd = fopen(saved_path, "w");
-      leave_priv_setting();
       if (fd == (FILE *)0) {
-        s_printf("DOSEMU: lock: (%s): %s\n", saved_path, strerror(errno));
-        error("\nDOSEMU: tty: lock: (%s): %s\n", saved_path, strerror(errno));
+        s_printf("lock: (%s): %s\n", saved_path, strerror(errno));
+        error("tty: lock: (%s): %s\n", saved_path, strerror(errno));
         return(-1);
       }
 
@@ -167,7 +147,7 @@ static int tty_lock(char *path, int mode)
 
     /* Make sure UUCP owns the lockfile.  Required by some packages. */
     if ((pw = getpwnam(OWNER_LOCKS)) == NULL) {
-      error("\nDOSEMU: tty: lock: UUCP user %s unknown!\n", OWNER_LOCKS);
+      error("tty: lock: UUCP user %s unknown!\n", OWNER_LOCKS);
       return(0);        /* keep the lock anyway */
     }
     
@@ -177,13 +157,11 @@ static int tty_lock(char *path, int mode)
   else if (mode == 2) { /* re-acquire a lock after a fork() */
     FILE *fd;
 
-     enter_priv_on();
      fd = fopen(saved_path,"w");
-     leave_priv_setting();
      if (fd == (FILE *)0) {
-     s_printf("DOSEMU: tty_lock(%s) reaquire: %s\n", 
+     s_printf("tty_lock(%s) reaquire: %s\n", 
               saved_path, strerror(errno));
-      error("\nDOSEMU: tty_lock: reacquire (%s): %s\n",
+      error("tty_lock: reacquire (%s): %s\n",
               saved_path, strerror(errno));
       return(-1);
     }
@@ -196,29 +174,24 @@ static int tty_lock(char *path, int mode)
 
     (void) fclose(fd);
     (void) chmod(saved_path, 0444);
-    (void) chown(saved_path, get_orig_uid(), get_orig_gid());
     return(0);
   } 
   else {    /* unlock */
     FILE *fd;
     int retval;
 
-    enter_priv_on();
     fd = fopen(saved_path,"w");
-    leave_priv_setting();
     if (fd == (FILE *)0) {
       s_printf("DOSEMU: tty_lock: can't reopen to delete: %s\n",
              strerror(errno));
       return (-1);
     }
       
-    enter_priv_on();
     retval = unlink(saved_path);
-    leave_priv_setting();
     if (retval < 0) {
-      s_printf("DOSEMU: tty: unlock: (%s): %s\n", saved_path,
+      s_printf("tty: unlock: (%s): %s\n", saved_path,
              strerror(errno));
-      error("\nDOSEMU: tty: unlock: (%s): %s\n", saved_path,
+      error("tty: unlock: (%s): %s\n", saved_path,
              strerror(errno));
       return(-1);
     }
@@ -226,32 +199,49 @@ static int tty_lock(char *path, int mode)
   return(0);
 }
 
+static void async_serial_run(void)
+{
+  /* Currently the fd is not passed to function, so we just call serial_run() */
+  s_printf("SERIAL: Async notification received\n");
+  serial_run();
+}
 
 /* This function opens ONE serial port for DOSEMU.  Normally called only
  * by do_ser_init below.   [num = port, return = file descriptor]
  */
-static int ser_open(int num)
+int ser_open(int num)
 {
   s_printf("SER%d: Running ser_open, fd=%d\n",num, com[num].fd);
   
   if (com[num].fd != -1) return (com[num].fd);
   
-  if ( tty_lock(com[num].dev, 1) >= 0) {		/* Lock port */
-    /* We know that we have access to the serial port */
-    com[num].dev_locked = TRUE;
-    
-    /* If the port is used for a mouse, then remove lockfile, because
-     * the use of the mouse serial port can be switched between processes,
-     * such as on Linux virtual consoles.
-     */
-    if (com[num].mouse)
-      if (tty_lock(com[num].dev, 0) >= 0)   		/* Unlock port */
-        com[num].dev_locked = FALSE;
+  if ( com[num].virtual )
+  {
+    if (tty_lock(com[num].dev, 0) >= 0)
+    {
+      dbug_printf("Opening Virtual Port\n");
+      com[num].dev_locked = FALSE;
+      com[num].fd = 0;
+      no_local_video = 1;
+    }
   }
+  else
+    if ( tty_lock(com[num].dev, 1) >= 0) {		/* Lock port */
+      /* We know that we have access to the serial port */
+      com[num].dev_locked = TRUE;
+    
+      /* If the port is used for a mouse, then remove lockfile, because
+       * the use of the mouse serial port can be switched between processes,
+       * such as on Linux virtual consoles.
+       */
+      if (com[num].mouse)
+        if (tty_lock(com[num].dev, 0) >= 0)   		/* Unlock port */
+          com[num].dev_locked = FALSE;
+    }
   else {
     /* The port is in use by another process!  Don't touch the port! */
     com[num].dev_locked = FALSE;
-    com[num].fd = -1;
+    com[num].fd = -2;
     return(-1);
   }
   
@@ -261,8 +251,109 @@ static int ser_open(int num)
   }
   
   com[num].fd = RPT_SYSCALL(open(com[num].dev, O_RDWR | O_NONBLOCK));
+  if (com[num].fd < 0) {
+    error("SERIAL: Unable to open device %s: %s\n",
+      com[num].dev, strerror(errno));
+    goto fail_unlock;
+  }
+  if (!isatty(com[num].fd)) {
+    error("SERIAL: Serial port device %s is not a tty, closing\n",
+      com[num].dev);
+    goto fail_close;
+  }
   RPT_SYSCALL(tcgetattr(com[num].fd, &com[num].oldset));
-  return (com[num].fd);
+  add_to_io_select(com[num].fd, 1, async_serial_run);
+  return com[num].fd;
+
+fail_close:
+  close(com[num].fd);
+  /* fall through */
+fail_unlock:
+  if (tty_lock(com[num].dev, 0) >= 0)   		/* Unlock port */
+    com[num].dev_locked = FALSE;
+
+  com[num].fd = -2; // disable permanently
+  return -1;
+}
+
+void ser_set_params(int num)
+{
+  int data = 0;
+  /* The following adjust raw line settings needed for DOSEMU serial     */
+  /* These defines are based on the Minicom 1.70 communications terminal */
+#if 1
+  com[num].newset.c_cflag |= (CLOCAL | CREAD);
+  com[num].newset.c_cflag &= ~(HUPCL | CRTSCTS);
+  com[num].newset.c_iflag |= (IGNBRK | IGNPAR);
+  com[num].newset.c_iflag &= ~(BRKINT | PARMRK | INPCK | ISTRIP |
+                               INLCR | IGNCR | INLCR | ICRNL | IXON | 
+                               IXOFF | IUCLC | IXANY | IMAXBEL);
+  com[num].newset.c_oflag &= ~(OPOST | OLCUC | ONLCR | OCRNL | ONOCR |
+                               ONLRET | OFILL | OFDEL);
+  com[num].newset.c_lflag &= ~(XCASE | ISIG | ICANON | IEXTEN | ECHO | 
+                               ECHONL | ECHOE | ECHOK | ECHOPRT | ECHOCTL | 
+                               ECHOKE | NOFLSH | TOSTOP);
+#else
+  /* These values should only be used as a last resort, or for testing */
+  com[num].newset.c_iflag = IGNBRK | IGNPAR;
+  com[num].newset.c_lflag = 0;
+  com[num].newset.c_oflag = 0;
+  com[num].newset.c_cflag |= CLOCAL | CREAD;
+  com[num].newset.c_cflag &= ~(HUPCL | CRTSCTS);
+#endif
+
+#ifdef __linux__
+  com[num].newset.c_line = 0;
+#endif
+  com[num].newset.c_cc[VMIN] = 1;
+  com[num].newset.c_cc[VTIME] = 0;
+  if (com[num].system_rtscts) com[num].newset.c_cflag |= CRTSCTS;
+  tcsetattr(com[num].fd, TCSANOW, &com[num].newset);
+
+  com[num].dll = 0x30;			/* Baudrate divisor LSB: 2400bps */
+  com[num].dlm = 0;			/* Baudrate divisor MSB: 2400bps */
+  com[num].tx_char_time = DIV_2400 * 10;/* 115200ths of second per char */
+  com[num].TX = 0;			/* Transmit Holding Register */
+  com[num].RX = 0;			/* Received Byte Register */
+  com[num].IER = 0;			/* Interrupt Enable Register */
+  com[num].IIR = UART_IIR_NO_INT;	/* Interrupt I.D. Register */
+  com[num].LCR = UART_LCR_WLEN8;	/* Line Control Register: 5N1 */
+  com[num].DLAB = 0;			/* DLAB for baudrate change */
+  com[num].FCReg = 0; 			/* FIFO Control Register */
+  com[num].rx_fifo_trigger = 1;		/* Receive FIFO trigger level */
+  com[num].MCR = 0;			/* Modem Control Register */
+  com[num].LSR = UART_LSR_TEMT | UART_LSR_THRE;   /* Txmit Hold Reg Empty */
+  com[num].LSRqueued = 0;		/* Queued LSR bits */
+  com[num].MSR = 0;			/* Modem Status Register */
+  com[num].MSRqueued = 0;		/* Queued MSR bits */
+  com[num].SCR = 0; 			/* Scratch Register */
+  com[num].int_enab = 0;		/* FLAG: Interrupts disabled */
+  com[num].int_pend = 0;		/* FLAG: No interrupts pending */
+  com[num].int_condition = 0;		/* FLAG: No int conditions set */
+  com[num].int_request = 0;		/* FLAG: No int requested */
+  com[num].fifo_enable = 0;		/* FLAG: FIFO enabled */
+  com[num].ms_timer = 0;		/* Modem Status check timer */
+  com[num].rx_timer = 0;		/* Receive read() polling timer */
+  com[num].tx_timer = 0;		/* Transmi countdown to next char */
+  com[num].tx_trigger = 0;		/* FLAG: Dont start more xmit ints */
+  com[num].rx_timeout = TIMEOUT_RX;	/* FLAG: Receive timeout */
+  com[num].tx_overflow = 0;		/* FLAG: Outgoing buffer overflow */
+  com[num].rx_fifo_size = 16;		/* Size of receive FIFO to emulate */
+  uart_clear_fifo(num,UART_FCR_CLEAR_CMD);	/* Initialize FIFOs */
+
+  if(s2_printf) s_printf("SER%d: do_ser_init: running ser_termios\n",num);
+  ser_termios(num);			/* Set line settings now */
+  modstat_engine(num);
+
+  /* Pull down DTR and RTS.  This is the most natural for most comm */
+  /* devices including mice so that DTR rises during mouse init.    */
+  if (!com[num].pseudo) {
+    data = TIOCM_DTR | TIOCM_RTS;
+    if (ioctl(com[num].fd, TIOCMBIC, &data) && errno == EINVAL) {
+      s_printf("SER%d: TIOCMBIC unsupported, setting pseudo flag\n", num);
+      com[num].pseudo = 1;
+    }
+  }
 }
 
 
@@ -271,15 +362,16 @@ static int ser_open(int num)
  */
 static int ser_close(int num)
 {
-  static int i;
+  int i;
   s_printf("SER%d: Running ser_close\n",num);
+  remove_from_io_select(com[num].fd, 1);
   uart_clear_fifo(num,UART_FCR_CLEAR_CMD);
   
   /* save current dosemu settings of the file and restore the old settings
    * before closing the file down. 
    */
   RPT_SYSCALL(tcgetattr(com[num].fd, &com[num].newset));
-  RPT_SYSCALL(tcsetattr(com[num].fd, TCSANOW, &com[num].oldset));
+  RPT_SYSCALL(tcsetattr(com[num].fd, TCSADRAIN, &com[num].oldset));
   i = RPT_SYSCALL(close(com[num].fd));
   com[num].fd = -1;
   
@@ -291,19 +383,8 @@ static int ser_close(int num)
   return (i);
 }
 
-#ifdef __NetBSD__
-#define IUCLC 0
-#define OLCUC 0
-#define OCRNL ONLCR
-#define ONOCR 0
-#define ONLRET 0
-#define OFILL 0
-#define OFDEL 0
-#define XCASE 0
-#endif
 
-Bit8u
-com_readb(ioport_t port) {
+static Bit8u com_readb(ioport_t port) {
   int tmp;
   for (tmp = 0; tmp < config.num_ser; tmp++) {
     if (((u_short)(port & ~7)) == com[tmp].base_port) {
@@ -313,13 +394,11 @@ com_readb(ioport_t port) {
   return 0;
 }
 
-void
-com_writeb(ioport_t port, Bit8u value) {
+static void com_writeb(ioport_t port, Bit8u value) {
   int tmp;
   for (tmp = 0; tmp < config.num_ser; tmp++) {
     if (((u_short)(port & ~7)) == com[tmp].base_port) {
       do_serial_out(tmp, (int)port, (int)value);
-      return;
     }
   }
 }
@@ -332,16 +411,15 @@ com_writeb(ioport_t port, Bit8u value) {
 static void do_ser_init(int num)
 {
   emu_iodev_t io_device;
-  int data = 0;
   int i;
   
   /* The following section sets up default com port, interrupt, base
   ** port address, and device path if they are undefined. The defaults are:
   **
-  **   COM1:   irq = 4    base_port = 0x3F8    device = /dev/cua0
-  **   COM2:   irq = 3    base_port = 0x2F8    device = /dev/cua1
-  **   COM3:   irq = 4    base_port = 0x3E8    device = /dev/cua2
-  **   COM4:   irq = 3    base_port = 0x2E8    device = /dev/cua3
+  **   COM1:   irq = 4    base_port = 0x3F8    device = /dev/ttyS0
+  **   COM2:   irq = 3    base_port = 0x2F8    device = /dev/ttyS1
+  **   COM3:   irq = 4    base_port = 0x3E8    device = /dev/ttyS2
+  **   COM4:   irq = 3    base_port = 0x2E8    device = /dev/ttyS3
   **
   ** If COMx is unspecified, the next unused COMx port number is assigned.
   */
@@ -372,12 +450,13 @@ static void do_ser_init(int num)
 
   if (com[num].dev[0] == 0) {			/* Is the device file undef? */
     switch (com[num].real_comport) {		/* Define it using std devs */
-    case 4:  strcpy(com[num].dev, "/dev/cua3"); break;
-    case 3:  strcpy(com[num].dev, "/dev/cua2"); break;
-    case 2:  strcpy(com[num].dev, "/dev/cua1"); break;
-    default: strcpy(com[num].dev, "/dev/cua0"); break;
+    case 4:  strcpy(com[num].dev, "/dev/ttyS3"); break;
+    case 3:  strcpy(com[num].dev, "/dev/ttyS2"); break;
+    case 2:  strcpy(com[num].dev, "/dev/ttyS1"); break;
+    default: strcpy(com[num].dev, "/dev/ttyS0"); break;
     }
   }
+  iodev_add_device(com[num].dev);
 
   /* FOSSIL emulation is inactive at startup. */
   com[num].fossil_active = FALSE;
@@ -394,7 +473,7 @@ static void do_ser_init(int num)
   if(com[num].interrupt < 16) {
     com[num].interrupt = pic_irq_list[com[num].interrupt];
     s_printf("SER%d: enabling interrupt %d\n", num, com[num].interrupt);
-    pic_seti(com[num].interrupt,pic_serial_run,0);
+    pic_seti(com[num].interrupt, pic_serial_run, 0, NULL);
     pic_unmaski(com[num].interrupt);
   }
   irq_source_num[com[num].interrupt] = num;	/* map interrupt to port */
@@ -460,78 +539,7 @@ static void do_ser_init(int num)
   serial_timer_update();
 
   /* Set file descriptor as unused, then attempt to open serial port */
-  com[num].fd = -1;
-  ser_open(num);
-  
-  /* The following adjust raw line settings needed for DOSEMU serial     */
-  /* These defines are based on the Minicom 1.70 communications terminal */
-#if 1
-  com[num].newset.c_cflag |= (CLOCAL | CREAD);
-  com[num].newset.c_cflag &= ~(HUPCL | CRTSCTS);
-  com[num].newset.c_iflag |= (IGNBRK | IGNPAR);
-  com[num].newset.c_iflag &= ~(BRKINT | PARMRK | INPCK | ISTRIP |
-                               INLCR | IGNCR | INLCR | ICRNL | IXON | 
-                               IXOFF | IUCLC | IXANY | IMAXBEL);
-  com[num].newset.c_oflag &= ~(OPOST | OLCUC | ONLCR | OCRNL | ONOCR |
-                               ONLRET | OFILL | OFDEL);
-  com[num].newset.c_lflag &= ~(XCASE | ISIG | ICANON | IEXTEN | ECHO | 
-                               ECHONL | ECHOE | ECHOK | ECHOPRT | ECHOCTL | 
-                               ECHOKE | NOFLSH | TOSTOP);
-#else
-  /* These values should only be used as a last resort, or for testing */
-  com[num].newset.c_iflag = IGNBRK | IGNPAR;
-  com[num].newset.c_lflag = 0;
-  com[num].newset.c_oflag = 0;
-  com[num].newset.c_cflag |= CLOCAL | CREAD;
-  com[num].newset.c_cflag &= ~(HUPCL | CRTSCTS);
-#endif
-
-#ifdef __linux__
-  com[num].newset.c_line = 0;
-#endif
-  com[num].newset.c_cc[VMIN] = 1;
-  com[num].newset.c_cc[VTIME] = 0;
-  if (com[num].system_rtscts) com[num].newset.c_cflag |= CRTSCTS;
-  tcsetattr(com[num].fd, TCSANOW, &com[num].newset);
-
-  com[num].dll = 0x30;			/* Baudrate divisor LSB: 2400bps */
-  com[num].dlm = 0;			/* Baudrate divisor MSB: 2400bps */
-  com[num].tx_char_time = DIV_2400 * 10;/* 115200ths of second per char */
-  com[num].TX = 0;			/* Transmit Holding Register */
-  com[num].RX = 0;			/* Received Byte Register */
-  com[num].IER = 0;			/* Interrupt Enable Register */
-  com[num].IIR = UART_IIR_NO_INT;	/* Interrupt I.D. Register */
-  com[num].LCR = UART_LCR_WLEN8;	/* Line Control Register: 5N1 */
-  com[num].DLAB = 0;			/* DLAB for baudrate change */
-  com[num].FCReg = 0; 			/* FIFO Control Register */
-  com[num].rx_fifo_trigger = 1;		/* Receive FIFO trigger level */
-  com[num].MCR = 0;			/* Modem Control Register */
-  com[num].LSR = UART_LSR_TEMT | UART_LSR_THRE;   /* Txmit Hold Reg Empty */
-  com[num].LSRqueued = 0;		/* Queued LSR bits */
-  com[num].MSR = 0;			/* Modem Status Register */
-  com[num].MSRqueued = 0;		/* Queued MSR bits */
-  com[num].SCR = 0; 			/* Scratch Register */
-  com[num].int_enab = 0;		/* FLAG: Interrupts disabled */
-  com[num].int_pend = 0;		/* FLAG: No interrupts pending */
-  com[num].int_condition = 0;		/* FLAG: No int conditions set */
-  com[num].fifo_enable = 0;		/* FLAG: FIFO enabled */
-  com[num].ms_timer = 0;		/* Modem Status check timer */
-  com[num].rx_timer = 0;		/* Receive read() polling timer */
-  com[num].tx_timer = 0;		/* Transmi countdown to next char */
-  com[num].tx_trigger = 0;		/* FLAG: Dont start more xmit ints */
-  com[num].rx_timeout = TIMEOUT_RX;	/* FLAG: Receive timeout */
-  com[num].tx_overflow = 0;		/* FLAG: Outgoing buffer overflow */
-  com[num].rx_fifo_size = 16;		/* Size of receive FIFO to emulate */
-  uart_clear_fifo(num,UART_FCR_CLEAR_CMD);	/* Initialize FIFOs */
-
-  if(s2_printf) s_printf("SER%d: do_ser_init: running ser_termios\n",num);
-  ser_termios(num);			/* Set line settings now */
-  modstat_engine(num);
-
-  /* Pull down DTR and RTS.  This is the most natural for most comm */
-  /* devices including mice so that DTR rises during mouse init.    */
-  data = TIOCM_DTR | TIOCM_RTS;
-  ioctl(com[num].fd, TIOCMBIC, &data);
+  com[num].fd = -1;  
 }
 
 
@@ -550,7 +558,7 @@ static void do_ser_init(int num)
 void serial_init(void)
 {
   int i;
-  warn("SERIAL $Header: /usr/src/dosemu0.60/serial/RCS/ser_init.c,v 1.1 1995/05/06 16:26:19 root Exp root $\n");
+  warn("SERIAL $Header: /cvsroot/dosemu/dosemu/src/base/serial/ser_init.c,v 1.1.2.5 2004/02/10 00:36:26 bartoldeman Exp $\n");
   s_printf("SER: Running serial_init, %d serial ports\n", config.num_ser);
 
   /* Clean the BIOS data area at 0040:0000 for serial ports */
@@ -576,8 +584,13 @@ void serial_init(void)
      * to the fact the mouse is in use by Xwindows (internal driver is used)
      * Direct access to the mouse by dosemu is useful mainly at the console.
      */
-    if (com[i].mouse && (config.usesX || !config.console)) 
+    if (com[i].mouse && !config.console)
       s_printf("SER%d: Not touching mouse outside of the console!\n",i);
+#ifdef USE_GPM    
+    else if (com[i].mouse && config.mouse.type == MOUSE_GPM)
+      s_printf("SER%d: GPM competing with direct access is racy: "
+               "only using GPM\n",i);
+#endif
     else
       do_ser_init(i);
   }
@@ -589,14 +602,20 @@ void serial_init(void)
  */
 void serial_close(void)
 {
-  static int i;
+  int i;
   s_printf("SER: Running serial_close\n");
   for (i = 0; i < config.num_ser; i++) {
-  
-    if (com[i].mouse && (config.usesX || !config.console)) 
+    if (com[i].fd < 0)
+      continue;
+    if (com[i].mouse && !config.console)
       s_printf("SER%d: Not touching mouse outside of the console!\n",i);
+#ifdef USE_GPM    
+    else if (com[i].mouse && config.mouse.type == MOUSE_GPM)
+      s_printf("SER%d: GPM competing with direct access is racy: "
+               "only using GPM\n",i);
+#endif
     else {
-      RPT_SYSCALL(tcsetattr(com[i].fd, TCSANOW, &com[i].oldset));
+      RPT_SYSCALL(tcsetattr(com[i].fd, TCSADRAIN, &com[i].oldset));
       ser_close(i);
     }
   }
@@ -609,8 +628,8 @@ void serial_close(void)
  */
 void child_close_mouse(void)
 {
-  static u_char i, rtrn;
-  if (!config.usesX || config.console) {
+  u_char i, rtrn;
+  if (config.console) {
     s_printf("MOUSE: CLOSE function starting. num_ser=%d\n", config.num_ser);
     for (i = 0; i < config.num_ser; i++) {
       s_printf("MOUSE: CLOSE port=%d, dev=%s, fd=%d, valid=%d\n", 
@@ -635,12 +654,12 @@ void child_close_mouse(void)
  */
 void child_open_mouse(void)
 {
-  static u_char i;
-  if (!config.usesX || config.console) {
+  u_char i;
+  if (config.console) {
     s_printf("MOUSE: OPEN function starting.\n");
     for (i = 0; i < config.num_ser; i++) {
       s_printf("MOUSE: OPEN port=%d, type=%d, dev=%s, valid=%d\n",
-                i, mice->type, com[i].dev, com[i].mouse);
+                i, config.mouse.type, com[i].dev, com[i].mouse);
       if (com[i].mouse == TRUE) {
         s_printf("MOUSE: OPEN port=%d: Running ser-open.\n", i);
         com[i].fd = -1;
@@ -650,3 +669,4 @@ void child_open_mouse(void)
     }
   }
 }
+
