@@ -601,8 +601,6 @@ static void put_tx(int num, int val)
   /* Update the transmit timer */
   com[num].tx_timer += com[num].tx_char_time;
 
-  com[num].TX = val;			/* Mainly used in overflow cases */
-  com[num].tx_trigger = 1;		/* Time to trigger next tx int */
   com[num].int_condition &= ~TX_INTR;	/* TX interrupt condition satisifed */
   com[num].LSR &= ~UART_LSR_TEMT;	/* TEMT not empty */
 
@@ -610,6 +608,9 @@ static void put_tx(int num, int val)
   if ((com[num].IIR & UART_IIR_ID) == UART_IIR_THRI)
     serial_int_engine(num, 0);		/* Update interrupt status */
 
+  com[num].TX = val;			/* Mainly used in overflow cases */
+  com[num].tx_trigger = 1;		/* Time to trigger next tx int */
+
   /* Loop-back writes.  Parity is currently not calculated.  No
    * UART diagnostics programs including COMTEST.EXE, that I tried,
    * complained about parity problems during loopback tests anyway!
@@ -1086,10 +1087,6 @@ do_serial_out(int num, ioport_t address, int val)
       if(s2_printf) s_printf("SER%d: Divisor MSB = 0x%x\n", num, val);
     }
     else {			/* Else, write to Interrupt Enable Register */
-      if ( !(com[num].IER & 2) && (val & 2) ) {
-        /* Flag to allow THRI if enable THRE went from state 0 -> 1 */
-        com[num].tx_trigger = 1;
-      }
       com[num].IER = (val & 0xF);	/* Write to IER */
       if(s1_printf) s_printf("SER%d: Write IER = 0x%x\n", num, val);
       serial_int_engine(num, 0);		/* Update interrupt status */
