@@ -1,5 +1,5 @@
 /*
- * (C) Copyright 1992, ..., 2001 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2004 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -67,7 +67,8 @@ void usage (void)
   printf ("Usage: UNIX [FLAG COMMAND]\n");
   printf ("Run Linux commands from DOSEMU\n\n");
   printf ("UNIX -e [ENVVAR]\n");
-  printf ("  Execute the command given in the Linux environment variable \"ENVVAR\".\n");
+  printf ("  Execute the DOS command given in the Linux environment variable \"ENVVAR\"\n");
+  printf ("  and then exit DOSEMU.\n");
   printf ("  If not given, use the argument to the -E flag of DOSEMU\n\n");
   printf ("UNIX -s ENVVAR\n");
   printf ("  Set the DOS environment to the Linux environment variable \"ENVVAR\".\n\n");
@@ -127,13 +128,7 @@ void do_execute_dos (int argc, char **argv)
   intr(0xe6, &preg);
 
   if (! preg.r_ax) {
-    /* SUCCESFUL */
-
-    if (!data[0])
-    	/* empty string, assume we had to exec -E and this wasn't given
-    	 * ( may have 'unix -e' in your atoexec.bat )
-    	 */
-    	exit(1);
+    /* SUCCESSFUL */
 
     printf ("About to Execute : %s\n", data);
 
@@ -149,13 +144,20 @@ void do_execute_dos (int argc, char **argv)
 		fprintf (stderr, "SYSTEM failed ....(%d)", errno);
 		exit (-1);
 	}
+	/* bye bye! */
+        if (data[strlen(data)+1] == '\0') {
+	  preg.r_ax = 0xffff;
+	  intr(0xe6, &preg);
+        }
 
 	exit (0);
-  } else {
-    /* UNSUCESSFUL */
-    puts ("FAILED to get DATA from DOSEMU.\n");
-
-    exit (-1);
+  }
+  else {
+    /* UNSUCCESSFUL */
+    /* empty string, assume we had to exec -E and this wasn't given
+     * ( may have 'unix -e' in your autoexec.bat )
+     */
+    exit (1);
   }
 
 }
@@ -164,7 +166,6 @@ void do_execute_dos (int argc, char **argv)
 void do_set_dosenv (int argc, char **argv)
 {
   struct REGPACK preg;
-  extern int msetenv(char *,char *);
   char data[256];
 
   if (argc == 0) usage();
