@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 2001 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2004 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -27,34 +27,34 @@
 #include "mapping.h"
 
 unsigned long dpmi_free_memory;           /* how many bytes memory client */
-dpmi_pm_block *pm_block_root[DPMI_MAX_CLIENTS];
 unsigned long pm_block_handle_used;       /* tracking handle */
 
-/* ultilities routines */
+/* utility routines */
 
 /* I don\'t think these function will ever become bottleneck, so just */
 /* keep it simple, --dong */
-/* alloc_pm_block allocate a dpmi_pm_block struct and add it to the list */
-static dpmi_pm_block * alloc_pm_block()
+/* alloc_pm_block: allocate a dpmi_pm_block struct and add it to the list */
+static dpmi_pm_block * alloc_pm_block(void)
 {
     dpmi_pm_block *p = malloc(sizeof(dpmi_pm_block));
     if(!p)
 	return NULL;
-    p -> next =	pm_block_root[current_client];	/* add it to list */
-    pm_block_root[current_client] = p;
+    p -> next =	DPMI_CLIENT.pm_block_root;	/* add it to list */
+    DPMI_CLIENT.pm_block_root = p;
     return p;
 }
+
 /* free_pm_block free a dpmi_pm_block struct and delete it from list */
 static int free_pm_block(dpmi_pm_block *p)
 {
     dpmi_pm_block *tmp;
     if (!p) return -1;
-    if (p == pm_block_root[current_client]) {
-	pm_block_root[current_client] = p -> next;
+    if (p == DPMI_CLIENT.pm_block_root) {
+	DPMI_CLIENT.pm_block_root = p -> next;
 	free(p);
 	return 0;
     }
-    for(tmp = pm_block_root[current_client]; tmp; tmp = tmp -> next)
+    for(tmp = DPMI_CLIENT.pm_block_root; tmp; tmp = tmp -> next)
 	if (tmp -> next == p)
 	    break;
     if (!tmp) return -1;
@@ -62,11 +62,12 @@ static int free_pm_block(dpmi_pm_block *p)
     free(p);
     return 0;
 }
+
 /* lookup_pm_block returns a dpmi_pm_block struct from its handle */
 dpmi_pm_block *lookup_pm_block(unsigned long h)
 {
     dpmi_pm_block *tmp;
-    for(tmp = pm_block_root[current_client]; tmp; tmp = tmp -> next)
+    for(tmp = DPMI_CLIENT.pm_block_root; tmp; tmp = tmp -> next)
 	if (tmp -> handle == h)
 	    return tmp;
     return 0;
@@ -75,7 +76,7 @@ dpmi_pm_block *lookup_pm_block(unsigned long h)
 unsigned long base2handle( void *base )
 {
     dpmi_pm_block *tmp;
-    for(tmp = pm_block_root[current_client]; tmp; tmp = tmp -> next)
+    for(tmp = DPMI_CLIENT.pm_block_root; tmp; tmp = tmp -> next)
 	if (tmp -> base == base)
 	    return tmp -> handle;
     return 0;
@@ -93,7 +94,10 @@ DPMImalloc(unsigned long size)
 	return NULL;
     if ((block = alloc_pm_block()) == NULL)
 	return NULL;
-
+//
+//    { char buf[128]; FILE *fs=fopen("/proc/self/maps","r");
+//      while (fgets(buf,120,fs)) dbug_printf("%s",buf); fclose(fs); }
+//
     block->base = alloc_mapping(MAPPING_DPMI, size, 0);
     if (!block->base) {
 	free_pm_block(block);
@@ -209,8 +213,8 @@ DPMIrealloc(unsigned long handle, unsigned long newsize)
 void 
 DPMIfreeAll(void)
 {
-    if (pm_block_root[current_client]) {
-	dpmi_pm_block *p = pm_block_root[current_client];
+    if (DPMI_CLIENT.pm_block_root) {
+	dpmi_pm_block *p = DPMI_CLIENT.pm_block_root;
 	while(p) {
 	    dpmi_pm_block *tmp;
 	    if (p->base) {
@@ -231,19 +235,19 @@ DPMIMapConventionalMemory(dpmi_pm_block *block, unsigned long offset,
     /* NOTE:
      * This DPMI function makes appear memory from below 1Meg to
      * address space allocated via DPMImalloc(). We use it only for
-     * DPMI function 0x0509 (Map convientional memory, DPMI version 1.0)
-     * We way, however, we implement it, doesn't cover mapping hardware RAM
+     * DPMI function 0x0509 (Map conventional memory, DPMI version 1.0)
+     * The way, however, we implement it, doesn't cover mapping hardware RAM
      * (e.g. Video buffers, adapter RAM, etc).
-     * This may lead to some imcompatibilities.        --Hans, 2000/02/04
+     * This may lead to some incompatibilities.        --Hans, 2000/02/04
      */
-    void *mapped_base;
+    char *mapped_base;
 
     mapped_base = block->base + offset;
     /* it seems we can\'t map low_addr to mapped_base, we must map */
     /* mapped_base to low_addr, so first copy the content */
     dpmi_eflags &= ~IF;
     pic_cli();
-    memmove((void *)mapped_base, (void *)low_addr, cnt*DPMI_page_size);
+    memmove(mapped_base, (void *)low_addr, cnt*DPMI_page_size);
 
     if ((int)mmap_mapping(MAPPING_DPMI | MAPPING_ALIAS,
 	    (void *)low_addr, cnt*DPMI_page_size,
