@@ -165,11 +165,80 @@ int msdos_pre_extender(struct sigcontext_struct *scp, int intr)
 	if (_LWORD(eax) == 0x1684) {
 	    D_printf("DPMI: Get VxD entry point BX = 0x%04x\n",
 		     _LWORD(ebx));
-	    /* no entry point */
-	    _es = _edi = 0;
+	    switch (_LWORD(ebx)) {
+		case 0x01:
+		    D_printf("DPMI: VMM VxD entry point requested\n");
+		    _es = DPMI_CLIENT.DPMI_SEL;
+		    _edi = DPMI_OFF + HLT_OFF(DPMI_VXD_VMM);
+		    break;
+		case 0x05:
+		    D_printf("DPMI: VTD VxD entry point requested\n");
+		    _es = DPMI_CLIENT.DPMI_SEL;
+		    _edi = DPMI_OFF + HLT_OFF(DPMI_VXD_VTD);
+		    break;
+		case 0x09:
+		    D_printf("DPMI: Reboot VxD entry point requested\n");
+		    _es = DPMI_CLIENT.DPMI_SEL;
+		    _edi = DPMI_OFF + HLT_OFF(DPMI_VXD_Reboot);
+		    break;
+		case 0x0a:
+		    D_printf("DPMI: VDD VxD entry point requested\n");
+		    _es = DPMI_CLIENT.DPMI_SEL;
+		    _edi = DPMI_OFF + HLT_OFF(DPMI_VXD_VDD);
+		    break;
+		case 0x0c:
+		    D_printf("DPMI: VMD VxD entry point requested\n");
+		    _es = DPMI_CLIENT.DPMI_SEL;
+		    _edi = DPMI_OFF + HLT_OFF(DPMI_VXD_VMD);
+		    break;
+		case 0x0e:
+		    D_printf("DPMI: VCD VxD entry point requested\n");
+		    _es = DPMI_CLIENT.DPMI_SEL;
+		    _edi = DPMI_OFF + HLT_OFF(DPMI_VXD_VCD);
+		    break;
+		case 0x17:
+		    D_printf("DPMI: SHELL VxD entry point requested\n");
+		    _es = DPMI_CLIENT.DPMI_SEL;
+		    _edi = DPMI_OFF + HLT_OFF(DPMI_VXD_SHELL);
+		    break;
+		case 0x21:
+		    D_printf("DPMI: PageFile VxD entry point requested\n");
+		    _es = DPMI_CLIENT.DPMI_SEL;
+		    _edi = DPMI_OFF + HLT_OFF(DPMI_VXD_PageFile);
+		    break;
+		case 0x26:
+		    D_printf("DPMI: APM VxD entry point requested\n");
+		    _es = DPMI_CLIENT.DPMI_SEL;
+		    _edi = DPMI_OFF + HLT_OFF(DPMI_VXD_APM);
+		    break;
+		case 0x27:
+		    D_printf("DPMI: VXDLDR VxD entry point requested\n");
+		    _es = DPMI_CLIENT.DPMI_SEL;
+		    _edi = DPMI_OFF + HLT_OFF(DPMI_VXD_VXDLDR);
+		    break;
+		case 0x33:
+		    D_printf("DPMI: CONFIGMG VxD entry point requested\n");
+		    _es = DPMI_CLIENT.DPMI_SEL;
+		    _edi = DPMI_OFF + HLT_OFF(DPMI_VXD_CONFIGMG);
+		    break;
+		case 0x37:
+		    D_printf("DPMI: ENABLE VxD entry point requested\n");
+		    _es = DPMI_CLIENT.DPMI_SEL;
+		    _edi = DPMI_OFF + HLT_OFF(DPMI_VXD_ENABLE);
+		    break;
+		case 0x442:
+		    D_printf("DPMI: VTDAPI VxD entry point requested\n");
+		    _es = DPMI_CLIENT.DPMI_SEL;
+		    _edi = DPMI_OFF + HLT_OFF(DPMI_VXD_VTDAPI);
+		    break;
+		default:
+		    D_printf("DPMI: ERROR: Unsupported VxD\n");
+		    /* no entry point */
+		    _es = _edi = 0;
+  	    }
 	    return 1;
-	}
-	return 0;
+  	}
+  	return 0;
     case 0x41:			/* win debug */
 	return 1;
     default:
@@ -286,7 +355,7 @@ int msdos_pre_extender(struct sigcontext_struct *scp, int intr)
 		    _eflags &= ~CF;
 		    DPMIfree(h);
 		    FreeDescriptor(_es);
-		    _es = 0;
+		    FreeSegRegs(scp, _es);
 		}
 		return 1;
 	    }
@@ -378,6 +447,7 @@ int msdos_pre_extender(struct sigcontext_struct *scp, int intr)
 	case 0x14: case 0x15:	/* dosx.exe, according to Ralf Brown */
 	case 0x21 ... 0x24:
 	case 0x27: case 0x28:
+	    error("MS-DOS: Unsupported function 0x%x\n", _HI(ax));
 	    _HI(ax) = 0xff;
 	    return 1;
 	case 0x11: case 0x12:	/* find first/next using FCB */
@@ -638,14 +708,14 @@ int msdos_pre_extender(struct sigcontext_struct *scp, int intr)
 		    snprintf((char *)(REG(ds)<<4), MAX_DOS_PATH, "%s",
 			     (char *)GetSegmentBaseAddress(_ds) +
 			     (DPMI_CLIENT.is_32 ? _edx : (_LWORD(edx))));
-		    seg += 200;
+		    seg += 0x20;
                 } else {
                     REG(ds) = (long) GetSegmentBaseAddress(_ds) >> 4;
 		}
 		if ( !is_dos_selector(_es)) {
 		    REG(es) = seg;
-		    snprintf((char *)((REG(es)<<4) + _LWORD(edi)),
-                             MAX_DOS_PATH, "%s",
+		    REG(edi) = 0;
+		    snprintf((char *)(REG(es)<<4), MAX_DOS_PATH, "%s",
 			     (char *)GetSegmentBaseAddress(_es) +
 			     (DPMI_CLIENT.is_32 ? _edi : (_LWORD(edi))));
                 } else {
@@ -703,10 +773,6 @@ int msdos_pre_extender(struct sigcontext_struct *scp, int intr)
 		+ 100;
 	    S_REG(es) = _es;
 	    REG(edi) = 0;
-	    memmove ((void *)(REG(es)<<4),
-		    (char *)GetSegmentBaseAddress(_es) +
-		    (DPMI_CLIENT.is_32 ? _edi : (_LWORD(edi))),
-		    0x100);
 	    in_dos_21++;
 	    return 0;
 	case 0x6c:		/*  Extended Open/Create */
@@ -1026,15 +1092,6 @@ void msdos_post_extender(int intr)
 	    DPMI_CLIENT.stack_frame.eax =
 		ConvertSegmentToDescriptor(LWORD(eax));
 	    break;
-#endif	    
-#if 0
-	case 0x4e:		/* find first */
-	case 0x4f:		/* find next */
-	    if (LWORD(eflags) & CF)
-		break;
-	    if (DPMI_CLIENT.USER_DTA_SEL)
-		memmove(DTA_over_1MB, DTA_under_1MB, 43);
-	    break;
 #endif	    
 	case 0x51:		/* get PSP */
 	case 0x62:
@@ -1116,6 +1173,15 @@ void msdos_post_extender(int intr)
 			0x100);
 	    }
 	    break;
+	case 0x60:		/* Canonicalize file name */
+	    DPMI_CLIENT.stack_frame.esi = S_REG(esi);
+	    DPMI_CLIENT.stack_frame.edi = S_REG(edi);
+	    memmove ((void *)GetSegmentBaseAddress(S_REG(es))
+			+ (DPMI_CLIENT.is_32 ? S_REG(edi) :  (S_LWORD(edi))),
+			(void *)(REG(es)<<4),
+			0x80);
+	    break;
+
 	default:
 	    break;
 	}
