@@ -1,8 +1,20 @@
-/* drivers/cdrom.c
+/* 
+ * All modifications in this file to the original code are
+ * (C) Copyright 1992, ..., 2004 the "DOSEMU-Development-Team".
+ *
+ * for details see file COPYING in the DOSEMU distribution
+ */
+
+/* 
+ * DANG_BEGIN_MODULE
+ * 
+ * drivers/cdrom.c
  *
  * Author: Karsten Rucker (rucker@astro.uni-bonn.de)
  *
- * See dosemu/doc/README.CDROM for further information
+ * REMARK
+ * See dosemu/doc/README.txt (8. Using CDROMS) for further information
+ * /REMARK
  *
  * History:
  *   May 25, 95, Karsten Rucker (rucker@astro.uni-bonn.de)
@@ -12,6 +24,8 @@
  *   Dec 3, 95, lee
  *   - Debugging output changed from *printf()/error() to C_printf()
  *   - Minor editing, no real changes
+ *
+ * DANG_END_MODULE
  */
 
 #include <stdio.h>
@@ -21,20 +35,24 @@
 #include <sys/types.h>
 #include <sys/ioctl.h>
 #include <unistd.h>
+#include <ctype.h>
 #include <errno.h>
-#ifdef __linux__
-#include <linux/cdrom.h>
+#ifndef ENOMEDIUM
+  /* kernels < 2.0.32 don't have this defined */
+  #define ENOMEDIUM       123     /* No medium found */
 #endif
-#ifdef __NetBSD__
-#include "netbsd_cdio.h"
-#include <string.h>
+#ifdef __linux__
+#include "Linux/cdrom.h"
 #endif
 
 #include "emu.h"
 
-extern int errno;
+#undef CDROM_DEBUG
+
+static int CdromFd[4] = {-1,-1,-1,-1};
+static int IndexCd = -1;
+#define cdrom_fd CdromFd[IndexCd]
 
-int cdrom_fd = -1;
 int cdu33a = 0;
 
 /* eject_allowed = 0
@@ -47,6 +65,38 @@ int cdu33a = 0;
 
 int eject_allowed = 1;
 
+/*
+ * From mscdex21.doc:
+ *
+ * The device driver will return a 32-bit value. Bit 0 is the least significant
+ * bit. The bits are interpreted as follows (* = dosemu defaults):
+ *
+ *  Bit 0     0    Door closed
+ *            1    Door open
+ *  Bit 1     0    Door locked
+ *            1    Door unlocked
+ *  Bit 2     0    Supports only cooked reading (*)
+ *            1    Supports cooked and raw reading
+ *  Bit 3     0    Read only (*)
+ *            1    Read/write
+ *  Bit 4     0    Data read only
+ *            1    Data read and plays audio/video tracks (*)
+ *  Bit 5     0    No interleaving (*)
+ *            1    Supports interleaving
+ *  Bit 6     0    Reserved
+ *  Bit 7     0    No prefetching (*)
+ *            1    Supports prefetching requests
+ *  Bit 8     0    No audio channel manipulation
+ *            1    Supports audio channel manipulation (*)
+ *  Bit 9     0    Supports HSG addressing mode
+ *            1    Supports HSG and Red Book addressing modes (*)
+ *  "Bit 10-31 0    Reserved (all 0)"
+ *  Bit 10    0    Ignore XA directory entries (?)
+ *            1    Read XA directory entries (*)(?)
+ *  Bit 11    0    Disk present (?)
+ *            1    No disk (?)
+ *
+ */
 unsigned int device_status;
 struct audio_status { unsigned int status;
 		      unsigned char media_changed;
@@ -104,45 +154,68 @@ struct audio_status { unsigned int status;
 #define MSCD_CTRL_VOLUME3          8
 
 #ifdef __linux__
-#define _PATH_CDROM "/dev/cdrom"
-#endif
-#ifdef __NetBSD__
-#define _PATH_CDROM "/dev/rcd0a"
+char *Path_cdrom[]={"/dev/cdrom","/dev/cdrom2","/dev/cdrom3","/dev/cdrom4"};
 #endif
+#define path_cdrom Path_cdrom[IndexCd]
 
 #ifdef CDROM_DEBUG
 int logging_ioctl(int fd, unsigned int cmd, void *arg)
 {
     int rval;
     int err;
-    C_printf("ioctl 0x%x\n", cmd);
+    if (debug_level('C')>5)
+      C_printf("CDROM: ioctl %#x\n", cmd);
     rval = ioctl(fd, cmd, arg);
     err = errno;
 
     if (rval == -1) {
-	C_printf(" --> %s\n", strerror(errno));
+	C_printf("CDROM: err %s\n", strerror(errno));
 	errno = err;
     }
     return rval;
 }
 
+void dump_cd_sect (char *tb)
+{
+    unsigned char buf[128];
+    int i,j;
+    unsigned char *p,*q,c;
+
+    q=tb;
+    for (i=0; i<32 && debug_level('C')>5; i++) {		/* 32x64 */
+      p=buf;
+      for (j=0; j<64; j++) {
+        c=*q++;
+        *p++ = (isprint(c)? c:'.');
+      }
+      C_printf("%03x[%s]\n",i*64,buf);
+    }
+    q=tb;
+    for (i=0; i<64 && debug_level('C')>8; i++) {		/* 64x32 */
+      p=buf;
+      for (j=0; j<4; j++) {
+        p+=sprintf(p,":%02x%02x%02x%02x%02x%02x%02x%02x",q[0],q[1],q[2],
+        	q[3],q[4],q[5],q[6],q[7]);
+        q+=8;
+      }
+      C_printf("%03x%s\n",i*32,buf);
+    }
+}
+
+
 #define ioctl logging_ioctl
 #endif
 
-void cdrom_reset()
+static void cdrom_reset(void)
 {
   /* after a disk change a new read access will
-     return in error. In order to unlock this condition
-     the drive must be reopend.
-     Does some one knows a better way?                   */
-    C_printf("cdrom reset\n");
+     return an error. In order to unlock this condition
+     the drive must be reopened.
+     Does someone know a better way?                   */
+   C_printf("CDROM: cdrom reset\n");
    close (cdrom_fd);
-   priv_off();
-   cdrom_fd = open (_PATH_CDROM, O_RDONLY);
-#ifdef __NetBSD__
-   if (cdrom_fd >= 0) ioctl(cdrom_fd, CDIOCALLOW, 0);
-#endif
-   priv_default();
+   cdrom_fd = open (path_cdrom, O_RDONLY);
+   if (cdrom_fd >= 0) ioctl (cdrom_fd, CDROMRESET, NULL);
 }
 
 #define MSCD_AUDCHAN_VOLUME0       2
@@ -160,54 +233,50 @@ void cdrom_helper(void)
    struct cdrom_tocentry cdrom_tocentry;
    struct cdrom_volctrl cdrom_volctrl;
    int n, error;
-#ifdef CDROM_DEBUG
-   int ioctlin,;
-#endif
 
-#ifdef __NetBSD__
-   struct cd_toc_entry toc_entry;
-   struct cd_sub_channel_info scinfo;
-
-   memset(&cdrom_subchnl, 0, sizeof(cdrom_subchnl));
-   memset(&scinfo, 0, sizeof(scinfo));
-
-   memset(&cdrom_tocentry, 0, sizeof(cdrom_tocentry));
-   memset(&toc_entry, 0, sizeof(toc_entry));
-
-   cdrom_subchnl.data = &scinfo;
-   cdrom_subchnl.data_len = sizeof(scinfo);
-   cdrom_tocentry.data =  &toc_entry;
-   cdrom_tocentry.data_len =  sizeof(toc_entry);
-#endif
    cdrom_subchnl.cdsc_format = CDROM_MSF;
 
+   IndexCd=(int)((HI(ax) & 0xC0)>>6);
+   HI(ax) = HI(ax) & 0x3F;
+
    if ((cdu33a) && (cdrom_fd < 0)) {
-        priv_off();
-        cdrom_fd = open (_PATH_CDROM, O_RDONLY);
-#ifdef __NetBSD__
-        if (cdrom_fd >= 0) ioctl(cdrom_fd, CDIOCALLOW, 0);
-#endif
-        priv_default();
+        cdrom_fd = open (path_cdrom, O_RDONLY);
 
         if (cdrom_fd < 0) {
-          LO(bx) = 1; /* media changed (for media changed request) */
-          LWORD(ebx) = audio_status.status | 0x800; /* no disc (for device status request) */
-
-          if ((HI(ax) == 0x09) || (HI(ax) == 0x0A))
-            LO(ax) = 0;
-           else LO(ax) = 1; /* for other requests return with error */
+          switch (HI(ax)) {
+            case 0x09:    /* media changed request */
+              LO(bx) = 1; /* media changed */
+              LO(ax) = 0;
+              return;
+            case 0x0A:    /* device status request */
+              LWORD(ebx) = audio_status.status | 0x800; /* no disc */
+              LO(ax) = 0;
+              return;
+          }
+          LO(ax) = 1; /* for other requests return with error */
           return ;
         }
    }
 
-#ifdef __NetBSD__
-   cdrom_subchnl.address_format = CD_MSF_FORMAT;
-   cdrom_subchnl.data_format = CD_CURRENT_POSITION;
-   cdrom_subchnl.track = 0;
-#endif
 
    switch (HI(ax)) {
-     case 0x01: audio_status.status = 0x00000310;
+     case 0x01:	/* NOTE: you can't see XA data disks if bit 10 of status
+		 * is cleared, MSCDEX will test it and skip XA entries!
+		 * Actually the entries skipped must have this pattern:
+		 *   xxxx1xxx xxxxxxxx 0x58 0x41
+		 * and the mscdex 2.25 code is:
+		 *	test	word ptr [bx+1Eh],400h
+		 *	jz	[check for XA]
+		 *	[return 0 = valid entry]
+		 * [check for XA]
+		 * ...
+		 *	cmp	word ptr es:[bx+6],4158h  'XA'
+		 *	jne	[return 0]
+		 *	mov	ax,es:[bx+4]
+		 *	and	ax,8
+		 *	[return ax]
+		 */
+		audio_status.status = 0x00000710; /* see function 0x0A below */
                 audio_status.paused_bit = 0;
                 audio_status.media_changed = 0;
                 audio_status.volume0 = 0xFF;
@@ -219,22 +288,19 @@ void cdrom_helper(void)
                 audio_status.outchan2 = 2;
                 audio_status.outchan3 = 3;
 
-                priv_off();
-                cdrom_fd = open (_PATH_CDROM, O_RDONLY);
+                cdrom_fd = open (path_cdrom, O_RDONLY);
 		error = errno;
-#ifdef __NetBSD__
-                if (cdrom_fd >= 0) ioctl(cdrom_fd, CDIOCALLOW, 0);
-#endif
-                priv_default();
 
                 if (cdrom_fd < 0) {
-		  C_printf("cdrom open (" _PATH_CDROM ") failed: %s\n",
-			    strerror(error));
-                  if (error == EIO) {
+		  C_printf("CDROM: cdrom open (%s) failed: %s\n",
+			    path_cdrom, strerror(error));
+                  LO(ax) = 0;
+                  if ((error == EIO) || (error==ENOMEDIUM)) {
                     /* drive which cannot be opened if no
                        disc is inserted!                   */
-                    LO(ax) = 0;
                     cdu33a = 1;
+                    if (! eject_allowed)
+                       LO(ax) = 1; /* no disk in drive */
                    }
                   else LO(ax) = 1; /* no cdrom drive installed */
                   if (! eject_allowed)
@@ -274,46 +340,45 @@ void cdrom_helper(void)
                  else { Sector = *CALC_PTR(req_buf,MSCD_READ_STARTSECTOR,u_long);
                       }
 
-		C_printf("reading sector %x (fmt %d)\n", Sector,
+		C_printf("CDROM: reading sector %#x (fmt %d)\n", Sector,
 			  *CALC_PTR(req_buf,MSCD_READ_ADRESSING,u_char));
-                if ((off_t) -1 == lseek (cdrom_fd, Sector*2048, SEEK_SET)) {
+                if ((off_t) -1 == lseek (cdrom_fd, Sector*CD_FRAMESIZE, SEEK_SET)) {
 		    HI(ax) = (errno == EINVAL ? 0x08 : 0x0F);
-		    C_printf("lseek failed: %s\n", strerror(errno));
+		    C_printf("CDROM: lseek failed: %s\n", strerror(errno));
 		    LO(ax) = 1;
 		} else {
-		    if ( (n = read (cdrom_fd, transfer_buf, *CALC_PTR(req_buf,MSCD_READ_NUMSECTORS,u_short)*2048)) < 0) {
+		    if ( (n = read (cdrom_fd, transfer_buf, *CALC_PTR(req_buf,MSCD_READ_NUMSECTORS,u_short)*CD_FRAMESIZE)) < 0) {
 			/* cd must be in drive, reset drive and try again */
 			cdrom_reset();
-			if ((off_t) -1 == lseek (cdrom_fd, Sector*2048, SEEK_SET)) {
+			if ((off_t) -1 == lseek (cdrom_fd, Sector*CD_FRAMESIZE, SEEK_SET)) {
 			    HI(ax) = (errno == EINVAL ? 0x08 : 0x0F);
-			    C_printf("lseek failed: %s\n", strerror(errno));
+			    C_printf("CDROM: lseek failed: %s\n", strerror(errno));
 			    LO(ax) = 1;
 			} else
-			    if ( (n = read (cdrom_fd, transfer_buf, *CALC_PTR(req_buf,MSCD_READ_NUMSECTORS,u_short)*2048)) < 0) {
+			    if ( (n = read (cdrom_fd, transfer_buf, *CALC_PTR(req_buf,MSCD_READ_NUMSECTORS,u_short)*CD_FRAMESIZE)) < 0) {
 				HI(ax) = (errno == EFAULT ? 0x0A : 0x0F);
-				C_printf("sector read (to %p, len %x) failed: %s\n",
-					  transfer_buf, *CALC_PTR(req_buf,MSCD_READ_NUMSECTORS,u_short)*2048, strerror(errno));
+				C_printf("CDROM: sector read (to %p, len %#x) failed: %s\n",
+					  transfer_buf, *CALC_PTR(req_buf,MSCD_READ_NUMSECTORS,u_short)*CD_FRAMESIZE, strerror(errno));
 				LO(ax) = 1;
 			    } else LO(ax) = 0;
 		    }
-		    if (n != *CALC_PTR(req_buf,MSCD_READ_NUMSECTORS,u_short)*2048) {
-			C_printf("sector read len %x got %x\n",
-				  *CALC_PTR(req_buf,MSCD_READ_NUMSECTORS,u_short)*2048, n);
+		    if (n != *CALC_PTR(req_buf,MSCD_READ_NUMSECTORS,u_short)*CD_FRAMESIZE) {
+			C_printf("CDROM: sector read len %#x got %#x\n",
+				  *CALC_PTR(req_buf,MSCD_READ_NUMSECTORS,u_short)*CD_FRAMESIZE, n);
 			LO(ax) = 1;
 			HI(ax) = 0x0F;
 		    } else {
+#ifdef CDROM_DEBUG
+			dump_cd_sect(transfer_buf);
+#endif
 			LO(ax) = 0;
-			if (dbg_fd) {
-			    fwrite(transfer_buf, 1, n, dbg_fd);
-			    fflush(dbg_fd);
-			}
 		    }
 		}
                 break;
      case 0x03: /* seek */
                 req_buf = SEG_ADR((char *), es, di);
-                if ((off_t)-1 == lseek (cdrom_fd, *CALC_PTR(req_buf,MSCD_SEEK_STARTSECTOR,u_long)*2048, SEEK_SET)) {
-		    C_printf("lseek failed: %s\n", strerror(errno));
+                if ((off_t)-1 == lseek (cdrom_fd, *CALC_PTR(req_buf,MSCD_SEEK_STARTSECTOR,u_long)*CD_FRAMESIZE, SEEK_SET)) {
+		    C_printf("CDROM: lseek failed: %s\n", strerror(errno));
 		    LO(ax) = 1;
 		}
                 break;
@@ -411,13 +476,13 @@ void cdrom_helper(void)
                 break;
      case 0x08: /* return sectorsize */
                 LO(ax) = 0;
-                LWORD(ebx) = 2048;
+                LWORD(ebx) = CD_FRAMESIZE;
                 break;
      case 0x09: /* media changed */
                 /* this function will be called from MSCDEX before
                    each new disk access !                         */
                 HI(ax) = 0; LO(ax) = 0; LO(bx) = 0;
-		C_printf("media changed?  %x\n", audio_status.media_changed);
+		C_printf("CDROM: media changed?  %#x\n", audio_status.media_changed);
 		errno = 0;
 		if (eject_allowed) {
                   if ((audio_status.media_changed) ||
@@ -426,7 +491,7 @@ void cdrom_helper(void)
 			cdrom_reset();
                     audio_status.media_changed = 0;
                     LO(bx) = 1; /* media has been changed */
-		    C_printf("media changed?  yes\n");
+		    C_printf("CDROM: media changed?  yes\n");
                     ioctl (cdrom_fd, CDROMSUBCHNL, &cdrom_subchnl);
                     if (! ioctl (cdrom_fd, CDROMSUBCHNL, &cdrom_subchnl))
                       cdrom_reset(); /* disc in drive */
@@ -439,14 +504,15 @@ void cdrom_helper(void)
      case 0x0A: /* device status */
                 HI(ax) = 0; LO(ax) = 0;
                 if (eject_allowed) {
-                  if (ioctl (cdrom_fd, CDROMSUBCHNL, &cdrom_subchnl))
+                  if (ioctl (cdrom_fd, CDROMSUBCHNL, &cdrom_subchnl)) {
                     if (ioctl (cdrom_fd, CDROMSUBCHNL, &cdrom_subchnl))
                       { /* no disk in drive */
                         LWORD(ebx) = audio_status.status | 0x800;
-			C_printf("subch failed: %s\n", strerror(errno));
+			C_printf("CDROM: subch failed: %s\n", strerror(errno));
                         break;
                       }
-                     else cdrom_reset();
+                    else cdrom_reset();
+		  }
                 }
                 /* disk in drive */
                 LWORD(ebx) = audio_status.status;
@@ -465,8 +531,11 @@ void cdrom_helper(void)
      case 0x0D: /* eject */
                 LO(ax) = 0;
                 if ((eject_allowed) && (audio_status.status & 0x02)) /* drive unlocked ? */
+                {
+                  audio_status.media_changed = 1;
                   if (ioctl (cdrom_fd, CDROMEJECT, NULL))
                     LO(ax) = 1;
+                }
                 break;
      case 0x0E: /* close tray */
                 LO(ax) = 0;
@@ -520,12 +589,6 @@ void cdrom_helper(void)
                   LO(ax) = 1;
                   break;
                 }
-#ifdef __NetBSD__
-                *CALC_PTR(req_buf,MSCD_DISKINFO_LEADOUT+3,u_char) = 0;
-                *CALC_PTR(req_buf,MSCD_DISKINFO_LEADOUT+2,u_char) = cdrom_tocentry.data->addr[1];
-                *CALC_PTR(req_buf,MSCD_DISKINFO_LEADOUT+1,u_char) = cdrom_tocentry.data->addr[2];
-                *CALC_PTR(req_buf,MSCD_DISKINFO_LEADOUT+0,u_char) = cdrom_tocentry.data->addr[3];
-#endif
 #ifdef __linux__
                 *CALC_PTR(req_buf,MSCD_DISKINFO_LEADOUT+3,u_char) = 0;
                 *CALC_PTR(req_buf,MSCD_DISKINFO_LEADOUT+2,u_char) = cdrom_tocentry.cdte_addr.msf.minute;
@@ -537,7 +600,7 @@ void cdrom_helper(void)
                 req_buf = SEG_ADR((char *), ds, si);
                 cdrom_tocentry.cdte_track = *CALC_PTR(req_buf,MSCD_TRACKINFO_TRACKNUM,u_char);
                 cdrom_tocentry.cdte_format = CDROM_MSF;
-		C_printf("track info, track %d\n", cdrom_tocentry.cdte_track);
+		C_printf("CDROM: track info, track %d\n", cdrom_tocentry.cdte_track);
                 if (ioctl (cdrom_fd, CDROMREADTOCENTRY, &cdrom_tocentry)) {
 		    /* XXX MSCDEX reads beyond the end of existing tracks.  Sigh. */
 		    if (errno != EINVAL)
@@ -551,12 +614,6 @@ void cdrom_helper(void)
                     break;
                   }
                 }
-#ifdef __NetBSD__
-                *CALC_PTR(req_buf,MSCD_TRACKINFO_TRACKPOS+3,u_char) = 0;
-                *CALC_PTR(req_buf,MSCD_TRACKINFO_TRACKPOS+2,u_char) = cdrom_tocentry.data->addr[1];
-                *CALC_PTR(req_buf,MSCD_TRACKINFO_TRACKPOS+1,u_char) = cdrom_tocentry.data->addr[2];
-                *CALC_PTR(req_buf,MSCD_TRACKINFO_TRACKPOS+0,u_char) = cdrom_tocentry.data->addr[3];
-#endif
 #ifdef __linux__
                 *CALC_PTR(req_buf,MSCD_TRACKINFO_TRACKPOS+3,u_char) = 0;
                 *CALC_PTR(req_buf,MSCD_TRACKINFO_TRACKPOS+2,u_char) = cdrom_tocentry.cdte_addr.msf.minute;
@@ -578,12 +635,6 @@ void cdrom_helper(void)
                   }
                 }
                 req_buf = SEG_ADR((char *), ds, si);
-#ifdef __NetBSD__
-                *CALC_PTR(req_buf,MSCD_GETVOLUMESIZE_SIZE,int) =
-		     cdrom_tocentry.data->addr[1]*60*75
-		     +cdrom_tocentry.data->addr[2]*60
-		     +cdrom_tocentry.data->addr[3];
-#endif
 #ifdef __linux__
                 *CALC_PTR(req_buf,MSCD_GETVOLUMESIZE_SIZE,int) = cdrom_tocentry.cdte_addr.msf.minute*60*75
                                                                     +cdrom_tocentry.cdte_addr.msf.second*60
@@ -608,11 +659,6 @@ void cdrom_helper(void)
                 *CALC_PTR(req_buf,MSCD_QCHAN_CTRL,u_char) = (cdrom_subchnl.cdsc_adr << 4) + (cdrom_subchnl.cdsc_ctrl);
                 *CALC_PTR(req_buf,MSCD_QCHAN_TNO,u_char)  = cdrom_subchnl.cdsc_trk;
                 *CALC_PTR(req_buf,MSCD_QCHAN_IND,u_char)  = cdrom_subchnl.cdsc_ind;
-#ifdef __NetBSD__
-                *CALC_PTR(req_buf,MSCD_QCHAN_MIN,u_char)  = cdrom_subchnl.data->what.position.reladdr[1];
-                *CALC_PTR(req_buf,MSCD_QCHAN_SEC,u_char)  = cdrom_subchnl.data->what.position.reladdr[2];
-                *CALC_PTR(req_buf,MSCD_QCHAN_FRM,u_char)  = cdrom_subchnl.data->what.position.reladdr[3];
-#endif
 #ifdef __linux__
                 *CALC_PTR(req_buf,MSCD_QCHAN_MIN,u_char)  = cdrom_subchnl.cdsc_reladdr.msf.minute;
                 *CALC_PTR(req_buf,MSCD_QCHAN_SEC,u_char)  = cdrom_subchnl.cdsc_reladdr.msf.second;
@@ -664,20 +710,18 @@ void cdrom_helper(void)
                 *CALC_PTR(req_buf,MSCD_AUDCHAN_VOLUME2-1,u_char) = audio_status.outchan2;
                 *CALC_PTR(req_buf,MSCD_AUDCHAN_VOLUME3-1,u_char) = audio_status.outchan3;
                 break;
-     default: C_printf ("CDROM: unknown request !\n");
+     default: C_printf ("CDROM: unknown request %#x!\n",HI(ax));
    }
 
 #ifdef CDROM_DEBUG
-                if (ioctlin) {
-                  C_printf ("            return  : ");
-                  req_buf = SEG_ADR((char *), ds, si);
-                  for (n = 0; n <= 9; ++n)
-                     C_printf ("  %3x", req_buf[n]);
-                  C_printf ("\n");
-                 }
-                else ("\n");
-
-                C_printf ("Leave cdrom request with return status %d.\n", LWORD(eax));
+   if (debug_level('C')>5) {
+     C_printf ("CDROM: req_buf ");
+     req_buf = SEG_ADR((char *), es, di);
+     for (n = 0; n < 22; ++n)
+       C_printf ("%02x", req_buf[n]);
+       C_printf ("\n");
+     }
 #endif
-   return ;
+   C_printf ("Leave cdrom request with return status %#x\n", LWORD(eax));
+   return;
 }
