@@ -193,7 +193,7 @@ static struct handle_record {
   int numpages;
   mach_port_t object;
   size_t objsize;
-  char name[8];
+  char name[9];
   int saved_mappings_handle[EMM_MAX_PHYS];
   int saved_mappings_logical[EMM_MAX_PHYS];
 }
@@ -319,7 +319,7 @@ ems_init(void)
   handle_info[OS_HANDLE].objsize = 0;
   handle_info[OS_HANDLE].active = 1;
   for (j = 0; j < EMM_MAX_PHYS; j++) {
-    handle_info[OS_HANDLE].saved_mappings_handle[j] = NULL_PAGE;
+    handle_info[OS_HANDLE].saved_mappings_logical[j] = NULL_PAGE;
   }
 
   handle_total++;
@@ -390,7 +390,7 @@ allocate_handle(int pages_needed)
       handle_total++;
       emm_allocated += pages_needed;
       for (j = 0; j < EMM_MAX_PHYS; j++) {
-	handle_info[i].saved_mappings_handle[j] = NULL_PAGE;
+	handle_info[i].saved_mappings_logical[j] = NULL_PAGE;
       }
       handle_info[i].active = 1;
       return (i);
@@ -1062,7 +1062,7 @@ move_memory_region(state_t * state)
   struct mem_move_struct *mem_move=NULL;
   caddr_t dest, source, mem;
 
-  (u_char *) mem = (u_char *) Addr(state, ds, esi);
+  mem = (caddr_t)((u_char *) Addr(state, ds, esi));
   load_move_mem(mem, mem_move);
   show_move_struct(mem_move);
   if (mem_move->source_type == 0)
@@ -1134,7 +1134,7 @@ exchange_memory_region(state_t * state)
   struct mem_move_struct *mem_move=NULL;
   caddr_t dest, source, mem, tmp;
 
-  (u_char *) mem = (u_char *) Addr(state, ds, esi);
+  mem = (caddr_t)((u_char *) Addr(state, ds, esi));
   load_move_mem(mem, mem_move);
   show_move_struct(mem_move);
   if (mem_move->source_type == 0)
