@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 2001 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2004 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -15,7 +15,7 @@
  * around in specific calls. This spec also includes the allocation of UMB's,
  * so they are also included as part of this file. The amount of xms memory
  * returned to DOS programs via the XMS requests, or int15 fnc88 is set in
- * "/etc/dosemu.conf" via the XMS paramter.
+ * "/etc/dosemu.conf" via the XMS parameter.
  *
  * /REMARK
  * DANG_END_MODULE
@@ -36,13 +36,15 @@
 #include "dosio.h"
 #include "machcompat.h"
 
+#undef  DEBUG_XMS
+
 static int umb_find_unused(void);
 /* 128*1024 is the amount of memory currently reserved in dos.c above
  * the 1 MEG mark.  ugly.  fix this.
  */
 
 /*
-static char RCSxms[] = "$Header: /usr/src/dosemu0.60/dosemu/RCS/xms.c,v 2.11 1995/05/06 16:25:30 root Exp root $";
+static char RCSxms[] = "$Header: /cvsroot/dosemu/dosemu/src/dosext/misc/xms.c,v 1.1.1.1.4.1 2004/01/16 21:48:25 bartoldeman Exp $";
 */
 
 #define	 XMS_GET_VERSION		0x00
@@ -80,7 +82,7 @@ static int xms_grab_int15again = 0;
 
 struct EMM get_emm(unsigned int, unsigned int);
 void show_emm(struct EMM);
-static void xms_query_freemem(int), xms_allocate_EMB(int), xms_free_EMB(void),
+static unsigned char xms_query_freemem(int), xms_allocate_EMB(int), xms_free_EMB(void),
  xms_move_EMB(void), xms_lock_EMB(int), xms_EMB_info(int), xms_realloc_EMB(int);
 
 static int FindFreeHandle(int);
@@ -135,9 +137,7 @@ static struct umb_record {
 /* #define dbg_fd stderr */
 
 static boolean_t
-umb_memory_empty(addr, size)
-     vm_address_t addr;
-     int size;
+umb_memory_empty(vm_address_t addr, int size)
 {
   int i;
   int *memory = (int *) addr;
@@ -147,7 +147,7 @@ umb_memory_empty(addr, size)
       return FALSE;
     }
   }
-#if 0
+#ifdef DEBUG_XMS
   Debug0((dbg_fd, "Found free UMB region: %p\n", (void *) addr));
 #endif
   return (TRUE);
@@ -224,7 +224,7 @@ umb_setup(void)
     for (addr = UMB_BASE; addr < (vm_address_t) (UMB_BASE + UMB_SIZE);
 	 addr += UMB_PAGE) {
       if (IN_EMM_SPACE(addr) || IN_EMU_SPACE(addr) || IN_HARDWARE_PAGES(addr)) {
-#if 0
+#ifdef DEBUG_XMS
 	Debug0((dbg_fd, "skipped UMB region: %p EMM-%d EMU=%d hardw=%d\n",(void *) addr, 
 		IN_EMM_SPACE(addr),IN_EMU_SPACE(addr),IN_HARDWARE_PAGES(addr)));
 #endif
@@ -382,7 +382,8 @@ umb_query(void)
 	largest = umbs[i].size;
     }
   }
-  Debug0((dbg_fd, "umb_query: largest UMB was %d bytes\n", largest));
+  Debug0((dbg_fd, "umb_query: largest UMB was %d(%#x) bytes\n",
+	largest, largest));
   return (largest);
 }
 
@@ -423,6 +424,8 @@ xms_control(void)
    * a problem's first appearance, maybe we wouldn't have these kinds
    * of compatibility hacks.
    */
+  unsigned char bl = 0;
+        
   if (HI(ax) != 0) {
     if (!xms_grab_int15again)
       xms_grab_int15 = 1;
@@ -447,25 +450,21 @@ xms_control(void)
     }
     else {
       x_printf("XMS: HMA already allocated\n");
-      LWORD(eax) = 0;
-      LO(bx) = 0x91;
+      bl = 0x91;
     }
-    LWORD(ebx) = 0xff00;
     break;
 
   case 2:			/* Release High Memory Area */
     x_printf("XMS release HMA\n");
     if (freeHMA) {
       x_printf("XMS: HMA already free\n");
-      LWORD(eax) = 0;
-      LO(bx) = 0x93;
+      bl = 0x93;
     }
     else {
       x_printf("XMS: freeing HMA\n");
       LWORD(eax) = 1;
       freeHMA = 1;
     }
-    LWORD(ebx) = 0xff00;
     break;
 
   case 3:			/* Global Enable A20 */
@@ -485,6 +484,7 @@ xms_control(void)
     break;
 
   case 5:			/* Local Enable A20 */
+    x_printf("XMS LOCAL enable A20\n");
     a20++;
     if (a20 == 1)
       set_a20(1);
@@ -505,39 +505,39 @@ xms_control(void)
           */
   case 7:			/* Query A20 */
     LWORD(eax) = a20 ? 1 : 0;
-    LWORD(ebx) = 0;		/* no error */
+    LO(bx) = 0;			/* no error */
     break;
 
   case 8:			/* Query Free Extended Memory */
-    xms_query_freemem(OLDXMS);
+    bl = xms_query_freemem(OLDXMS);
     break;
 
   case 9:			/* Allocate Extended Memory Block */
-    xms_allocate_EMB(OLDXMS);
+    bl = xms_allocate_EMB(OLDXMS);
     break;
 
   case 0xa:			/* Free Extended Memory Block */
-    xms_free_EMB();
+    bl = xms_free_EMB();
     break;
 
   case 0xb:			/* Move Extended Memory Block */
-    xms_move_EMB();
+    bl = xms_move_EMB();
     break;
 
   case 0xc:			/* Lock Extended Memory Block */
-    xms_lock_EMB(1);
+    bl = xms_lock_EMB(1);
     break;
 
   case 0xd:			/* Unlock Extended Memory Block */
-    xms_lock_EMB(0);
+    bl = xms_lock_EMB(0);
     break;
 
   case 0xe:			/* Get EMB Handle Information */
-    xms_EMB_info(OLDXMS);
-    return;
+    bl = xms_EMB_info(OLDXMS);
+    break;
 
-  case 0xf:			/* Realocate Extended Memory Block */
-    xms_realloc_EMB(OLDXMS);
+  case 0xf:			/* Reallocate Extended Memory Block */
+    bl = xms_realloc_EMB(OLDXMS);
     break;
 
 #define state (&_regs)
@@ -546,8 +546,7 @@ xms_control(void)
       int size = WORD(state->edx) << 4;
       vm_address_t addr = umb_allocate(size);
 
-      Debug0((dbg_fd, "Allocate UMB memory: 0x%04x\n",
-	      (unsigned) WORD(state->edx)));
+      Debug0((dbg_fd, "Allocate UMB memory: %#x\n", size));
       if (addr == (vm_address_t) 0) {
         int avail=umb_query();
 
@@ -563,7 +562,7 @@ xms_control(void)
 	SETWORD(&(state->ebx), (int) addr >> 4);
 	SETWORD(&(state->edx), size >> 4);
       }
-      Debug0((dbg_fd, "umb_allocated: 0x%04x, 0x%04x\n",
+      Debug0((dbg_fd, "umb_allocated: %#x0:%#x0\n",
 	      (unsigned) WORD(state->ebx),
 	      (unsigned) WORD(state->edx)));
       /* retval = UNCHANGED; */
@@ -583,34 +582,35 @@ xms_control(void)
   case 0x12:
     x_printf("XMS realloc UMB segment 0x%04x size 0x%04x\n",
 	     LWORD(edx), LWORD(ebx));
-    LWORD(eax) = 0;		/* failure */
-    LO(bx) = 0x80;		/* function unimplemented */
+    bl = 0x80;		        /* function unimplemented */
     break;
 
     /* the functions below are the newer, 32-bit XMS 3.0 interface */
 
   case 0x88:			/* Query Any Free Extended Memory */
-    xms_query_freemem(NEWXMS);
+    bl = xms_query_freemem(NEWXMS);
     break;
 
   case 0x89:			/* Allocate Extended Memory */
-    xms_allocate_EMB(NEWXMS);
+    bl = xms_allocate_EMB(NEWXMS);
     break;
 
   case 0x8e:			/* Get EMB Handle Information */
-    xms_EMB_info(NEWXMS);
+    bl = xms_EMB_info(NEWXMS);
     break;
 
   case 0x8f:			/* Reallocate EMB */
-    xms_realloc_EMB(NEWXMS);
+    bl = xms_realloc_EMB(NEWXMS);
     break;
 
   default:
     x_printf("XMS: Unimplemented XMS function AX=0x%04x", LWORD(eax));
     show_regs(__FILE__, __LINE__);		/* if you delete this, put the \n on the line above */
-    LWORD(eax) = 0;		/* failure */
-    LO(bx) = 0x80;		/* function not implemented */
-
+    bl = 0x80;		/* function not implemented */
+  }
+  if (bl != 0) {
+    LWORD(eax) = 0;             /* failure */
+    LO(bx) = bl;
   }
 }
 
@@ -641,7 +641,7 @@ ValidHandle(unsigned short h)
     return 0;
 }
 
-static void
+static unsigned char
 xms_query_freemem(int api)
 {
   unsigned long totalBytes = 0, subtotal;
@@ -685,11 +685,11 @@ xms_query_freemem(int api)
     x_printf("XMS query free memory(new): %ldK %ldK\n",
 	     REG(eax), REG(edx));
   }
-
-  LWORD(ebx) = 0xff00;			/* no error */
+  _BL = 0;
+  return 0;
 }
 
-static void
+static unsigned char
 xms_allocate_EMB(int api)
 {
   unsigned long totalBytes, subtotal;
@@ -700,7 +700,7 @@ xms_allocate_EMB(int api)
     kbsize = LWORD(edx);
   else
     kbsize = REG(edx);
-  x_printf("XMS alloc EMB(%s) size 0x%08lx KB\n", (api==OLDXMS)?"old":"new",kbsize);
+  x_printf("XMS alloc EMB(%s) size %ld KB\n", (api==OLDXMS)?"old":"new",kbsize);
 
   totalBytes = 0;
   for (h = FIRST_HANDLE; h <= NUM_HANDLES; h++) {
@@ -713,15 +713,12 @@ xms_allocate_EMB(int api)
   if(kbsize > subtotal)
   {
     x_printf("XMS: out of memory (only %ldK available)\n",subtotal);
-    LWORD(eax) = 0;
-    LWORD(ebx) = 0xFFa0; /* Out of memory */
-    return;
+    return 0xa0; /* Out of memory */
   }
   
   if (!(h = FindFreeHandle(FIRST_HANDLE))) {
     x_printf("XMS: out of handles\n");
-    LWORD(eax) = 0;
-    LWORD(ebx) = 0xffa1;
+    return 0xa1;
   }
   else {
     handles[h].num = h;
@@ -752,19 +749,18 @@ xms_allocate_EMB(int api)
       REG(edx) = h;
 
     LWORD(eax) = 1;		/* success */
-    LWORD(ebx) = 0xa900;
+    return 0;
   }
 }
 
-static void
+static unsigned char
 xms_free_EMB(void)
 {
   unsigned short int h = LWORD(edx);
 
   if (!ValidHandle(h)) {
     x_printf("XMS: free EMB bad handle %d\n", h);
-    LWORD(eax) = 0;
-    LO(bx) = 0xa2;
+    return 0xa2;
   }
   else {
 
@@ -778,11 +774,11 @@ xms_free_EMB(void)
 
     x_printf("XMS: free'd EMB %d\n", h);
     LWORD(eax) = 1;
-    LWORD(ebx) = 0xff00;
+    return 0;
   }
 }
 
-static void
+static unsigned char
 xms_move_EMB(void)
 {
   char *src, *dest;
@@ -790,23 +786,21 @@ xms_move_EMB(void)
 
   x_printf("XMS move extended memory block\n");
   show_emm(e);
-  LWORD(eax) = 1;		/* start with success */
-  LWORD(ebx) = 0xff00;		/* start with success */
 
   if (e.SourceHandle == 0) {
     src = (char *) (((e.SourceOffset >> 16) << 4) + \
 		    (e.SourceOffset & 0xffff));
   }
   else {
-    if (handles[e.SourceHandle].valid == 0)
+    if (handles[e.SourceHandle].valid == 0) {
       x_printf("XMS: invalid source handle\n");
-    src = handles[e.SourceHandle].addr + e.SourceOffset;
-    if (((unsigned long int)handles[e.SourceHandle].addr + handles[e.SourceHandle].size) <
-        ((unsigned long int)src + e.Length) ) {
-      e.Length = (unsigned long int)handles[e.SourceHandle].addr + handles[e.SourceHandle].size - (unsigned long int)src;
-      LWORD(eax) = 0;
-      LO(bx) = 0xa7;		/* invalid Length */
+      return 0xa3;
     }
+    src = handles[e.SourceHandle].addr + e.SourceOffset;
+    if (src > handles[e.SourceHandle].addr + handles[e.SourceHandle].size)
+      return 0xa4;              /* invalid source offset */
+    if (src + e.Length > handles[e.SourceHandle].addr + handles[e.SourceHandle].size)
+      return 0xa7;              /* invalid Length */
   }
 
   if (e.DestHandle == 0) {
@@ -814,14 +808,15 @@ xms_move_EMB(void)
 		     (e.DestOffset & 0xffff));
   }
   else {
-    if (handles[e.DestHandle].valid == 0)
+    if (handles[e.DestHandle].valid == 0) {
       x_printf("XMS: invalid dest handle\n");
+      return 0xa5;
+    }
     dest = handles[e.DestHandle].addr + e.DestOffset;
-    if (((unsigned long int)handles[e.DestHandle].addr + handles[e.DestHandle].size) <
-        ((unsigned long int)dest + e.Length) ) {
-      e.Length = (unsigned long int)handles[e.DestHandle].addr + handles[e.DestHandle].size - (unsigned long int)dest;
-      LWORD(eax) = 0;
-      LO(bx) = 0xa7;		/* invalid Length */
+    if (dest > handles[e.DestHandle].addr + handles[e.DestHandle].size)
+      return 0xa6;              /* invalid dest offset */
+    if (dest + e.Length > handles[e.DestHandle].addr + handles[e.DestHandle].size) {
+      return 0xa7;		/* invalid Length */
     }
   }
 
@@ -829,13 +824,14 @@ xms_move_EMB(void)
 	   (void *) src, (void *) dest, e.Length);
 
   memmove(dest, src, e.Length);
+  LWORD(eax) = 1;
 
   x_printf("XMS: block move done\n");
 
-  return;
+  return 0;
 }
 
-static void
+static unsigned char
 xms_lock_EMB(int flag)
 {
   int h = LWORD(edx);
@@ -855,52 +851,48 @@ xms_lock_EMB(int flag)
         handles[h].lockcount--;
       else {
         x_printf("XMS: Unlock handle %d already at 0\n", h);
-        LWORD(eax) = 0;
-        LO(bx) = 0xaa;		/* Block is not locked */
-	return;
+        return 0xaa;		/* Block is not locked */
       }
 
     LWORD(edx) = (int) handles[h].addr >> 16;
     LWORD(ebx) = (int) handles[h].addr & 0xffff;
 
     LWORD(eax) = 1;
+    return 0;
   }
   else {
     x_printf("XMS: invalid handle %d, can't (un)lock\n", h);
-    LWORD(eax) = 0;
-    LO(bx) = 0xa2;		/* invalid handle */
+    return 0xa2;		/* invalid handle */
   }
 }
 
-static void
+static unsigned char
 xms_EMB_info(int api)
 {
   int h = LWORD(edx);
 
   if (ValidHandle(h)) {
     if (api == OLDXMS) {
-      HI(bx) = handles[h].lockcount;
       LO(bx) = NUM_HANDLES - handle_count;
       LWORD(edx) = handles[h].size / 1024;
-      LWORD(eax) = 1;
       x_printf("XMS Get EMB info(old) %d\n", h);
     }
     else {			/* api == NEWXMS */
-      HI(bx) = handles[h].lockcount;
       LWORD(ecx) = NUM_HANDLES - handle_count;
       REG(edx) = handles[h].size / 1024;
-      LWORD(eax) = 1;
       x_printf("XMS Get EMB info(new) %d\n", h);
     }
+    HI(bx) = handles[h].lockcount;
+    LWORD(eax) = 1;
+    return 0;
   }
   else {
-    LWORD(eax) = 0;
-    LO(bx) = 0xa2;		/* invalid handle */
+    return 0xa2;		/* invalid handle */
   }
 }
 
 /* untested! if you test it, please tell me! */
-static void
+static unsigned char
 xms_realloc_EMB(int api)
 {
   int h;
@@ -911,18 +903,12 @@ xms_realloc_EMB(int api)
 
   if (!ValidHandle(h)) {
     x_printf("XMS: invalid handle %d, cannot realloc\n", h);
-
-    LO(bx) = 0xa2;
-    LWORD(eax) = 0;
-    return;
+    return 0xa2;
   }
   else if (handles[h].lockcount) {
     x_printf("XMS: handle %d locked (%d), cannot realloc\n",
 	     h, handles[h].lockcount);
-
-    LO(bx) = 0xab;
-    LWORD(eax) = 0;
-    return;
+    return 0xab;
   }
 
   oldsize = handles[h].size;
@@ -952,6 +938,7 @@ xms_realloc_EMB(int api)
   free(oldaddr);
 
   LWORD(eax) = 1;		/* success */
+  return 0;
 }
 
 void
@@ -1014,9 +1001,7 @@ show_emm(struct EMM e)
 {
   x_printf("XMS show_emm:\n");
 
-  x_printf("L: 0x%08lx\n"
-	   "SH: 0x%04x  SO: 0x%08lx\n"
-	   "DH: 0x%04x  DO: 0x%08lx\n",
+  x_printf("L: 0x%08lx SH: 0x%04x  SO: 0x%08lx DH: 0x%04x  DO: 0x%08lx\n",
 	   e.Length, e.SourceHandle, e.SourceOffset,
 	   e.DestHandle, e.DestOffset);
 }
