@@ -1,3 +1,10 @@
+/* 
+ * All modifications in this file to the original code are
+ * (C) Copyright 1992, ..., 2004 the "DOSEMU-Development-Team".
+ *
+ * for details see file COPYING in the DOSEMU distribution
+ */
+
 /* ipx.c for the DOS emulator
  * 		Tim Bird, tbird@novell.com
  *
@@ -15,41 +22,44 @@
 #include <sys/time.h>
 #include <sys/types.h>
 #include <sys/vm86.h>
-#include <linux/sockios.h>
-#include <linux/ipx.h>
+#include <netipx/ipx.h>
 #include <netinet/in.h>
+#include <errno.h>
 
-#include "timers.h"
 #include "memory.h"
 #include "emu.h"
+#include "timers.h"
 #include "cpu.h"
+#include "int.h"
+#include "dpmi.h"
+#include "bios.h"
 #include "inifile.h"
+#include "doshelpers.h"
 
 #define MAX_PACKET_DATA		1500
 
 #if 0
 /* NOTE: the below is already defined with #include "emu.h"
- *       Must NOT redefine it, else USE_VM86PLUS won't work !!!
+ *       Must NOT redefine it, else vm86plus won't work !!!
  */
 extern struct vm86_struct vm86s;
 #endif
 
 /* declare some function prototypes */
-extern u_char IPXCancelEvent(far_t ECBPtr);
+static u_char IPXCancelEvent(far_t ECBPtr);
 far_t ESRPopRegistersReturn;
 far_t ESRPopRegistersIRet;
 far_t ESRFarCall;
 
 static ipx_socket_t *ipx_socket_list = NULL;
-static IPXRunning = 0;
+static int IPXRunning = 0;
 
 /* DANG_FIXTHIS - get a real value for my address !! */
 static unsigned char MyAddress[10] =
 {0x01, 0x01, 0x00, 0xe0,
  0x00, 0x00, 0x1b, 0x33, 0x2b, 0x13};
 
-int 
-GetMyAddress( void )
+static int GetMyAddress( void )
 {
   int sock;
   struct sockaddr_ipx ipxs;
@@ -60,7 +70,7 @@ GetMyAddress( void )
   sock=socket(AF_IPX,SOCK_DGRAM,PF_IPX);
   if(sock==-1)
   {
-    n_printf("IPX: could not open socket in GetMyAddress\n");
+    n_printf("IPX: could not open socket in GetMyAddress: %s\n", strerror(errno));
     return(-1);
   }
 
@@ -71,14 +81,14 @@ GetMyAddress( void )
   
   if(bind(sock,(struct sockaddr *)&ipxs,sizeof(ipxs))==-1)
   {
-    n_printf("IPX: could bind to network 0 in GetMyAddress\n");
+    n_printf("IPX: could not bind to network 0 in GetMyAddress: %s\n", strerror(errno));
     close( sock );
     return(-1);
   }
   
   len = sizeof(ipxs2);
   if (getsockname(sock,(struct sockaddr *)&ipxs2,&len) < 0) {
-    n_printf("IPX: could not get socket name in GetMyAddress\n");
+    n_printf("IPX: could not get socket name in GetMyAddress: %s\n", strerror(errno));
     close( sock );
     return(-1);
   }
@@ -113,7 +123,7 @@ InitIPXFarCallHelper(void)
     n_printf("IPX: trying to read network and node address from DOS.INI\n");
     
     /* take some time here to read my address from the DOS.INI file */
-    GetValueFromIniFile( "/etc/dosemu/dos.ini", "ipx", "network", value );
+    GetValueFromIniFile(IPX_DOS_INI_PATH, "ipx", "network", value );
     if( strlen( value )==0 ) {
       n_printf("IPX: cannot read IPX network address from dos.ini file\n");
       n_printf("IPX: please create an [IPX] section in your dos.ini file\n");
@@ -135,7 +145,7 @@ InitIPXFarCallHelper(void)
       }
     }
     
-    GetValueFromIniFile( "/etc/dosemu/dos.ini", "ipx", "node", value );
+    GetValueFromIniFile(IPX_DOS_INI_PATH, "ipx", "node", value );
     if( strlen( value )==0 ) {
       n_printf("IPX: cannot read IPX node address from dos.ini file\n");
     } else {
@@ -158,7 +168,7 @@ InitIPXFarCallHelper(void)
       MyAddress[0], MyAddress[1], MyAddress[2], MyAddress[3] );
     n_printf("IPX: using node address of %02X%02X%02X%02X%02X%02X\n",
       MyAddress[4], MyAddress[5], MyAddress[6], MyAddress[7],
-      MyAddress[8], MyAddress[6] );
+      MyAddress[8], MyAddress[9] );
   }
    
 #if 0
@@ -166,8 +176,8 @@ InitIPXFarCallHelper(void)
   *ptr++ = 0x50;		/* push ax - FarCallHandler removes this before returning */
   *ptr++ = 0xb0;		/* mov al, 7a */
   *ptr++ = 0x7a;
-  *ptr++ = 0xcd;		/* int 0xe6 */
-  *ptr++ = 0xe6;
+  *ptr++ = 0xcd;		/* int DOS_HELPER_INT (0xe6) */
+  *ptr++ = DOS_HELPER_INT;
   *ptr++ = 0xcb;		/* far ret */
 
   ESRPopRegistersReturn.segment = ((int) ptr) >> 4;
@@ -197,7 +207,6 @@ InitIPXFarCallHelper(void)
   *ptr++ = 0xcf;		/* iret */
 #else
   {
-    extern void bios_f000(), bios_IPX_PopRegistersReturn(), bios_IPX_PopRegistersIRet(), bios_IPX_FarCall();
     long i = (long)bios_IPX_PopRegistersReturn - (long)bios_f000;
     i += BIOSSEG << 4;
     ESRPopRegistersReturn.segment = i >> 4;
@@ -286,8 +295,7 @@ ipx_find_socket(int port)
   return (NULL);
 }
 
-u_short
-SwapInt(u_short value)
+static u_short SwapInt(u_short value)
 {
   u_short temp;
 
@@ -295,8 +303,7 @@ SwapInt(u_short value)
   return (temp + (value >> 8));
 }
 
-void
-dumpBytes(u_char * memptr, int count)
+static void dumpBytes(u_char * memptr, int count)
 {
   int i, linecounter;
 
@@ -322,12 +329,11 @@ dumpBytes(u_char * memptr, int count)
   n_printf("\n");
 }
 
-void
-printECB(ECB_t * ECB)
+static void printECB(ECB_t * ECB)
 {
   int i;
    
-  if( d.network ) {
+  if(debug_level('n')) {
     n_printf("--DOS ECB (dump)--\n");
     dumpBytes((u_char *) ECB, 60);
     n_printf("--DOS ECB--\n");
@@ -353,10 +359,9 @@ printECB(ECB_t * ECB)
   }
 }
 
-void
-printIPXHeader(IPXPacket_t * IPXHeader)
+static void printIPXHeader(IPXPacket_t * IPXHeader)
 {
-  if (d.network) {
+  if (debug_level('n')) {
     n_printf("--IPX Header (dump)--\n");
     dumpBytes((u_char *) IPXHeader, 30);
     n_printf("--IPX Header --\n");
@@ -393,8 +398,7 @@ printIPXHeader(IPXPacket_t * IPXHeader)
   }
 }
 
-u_char
-IPXOpenSocket(u_short port, u_short * newPort)
+static u_char IPXOpenSocket(u_short port, u_short * newPort)
 {
   int sock;			/* sock here means Linux socket handle */
   int opt;
@@ -420,45 +424,36 @@ IPXOpenSocket(u_short port, u_short * newPort)
   /* do a socket call, then bind to this port */
   sock = socket(AF_IPX, SOCK_DGRAM, PF_IPX);
   if (sock == -1) {
-    n_printf("IPX: could not open IPX socket.\n");
+    n_printf("IPX: could not open IPX socket: %s.\n", strerror(errno));
     /* I can't think of anything else to return */
     return (RCODE_SOCKET_TABLE_FULL);
   }
 
-  opt = 1;
-  /* turn on socket debugging */
-  if (d.network) {
-    if (setsockopt(sock, SOL_SOCKET, SO_DEBUG, &opt, sizeof(opt)) == -1) {
-      n_printf("IPX: could not set socket option for debugging.\n");
-      /* I can't think of anything else to return */
-      return (RCODE_SOCKET_TABLE_FULL);
-    }
-  }
   opt = 1;
   /* Permit broadcast output */
   if (setsockopt(sock, SOL_SOCKET, SO_BROADCAST,
 		 &opt, sizeof(opt)) == -1) {
     /* I can't think of anything else to return */
-    n_printf("IPX: could not set socket option for broadcast.\n");
+    n_printf("IPX: could not set socket option for broadcast: %s.\n", strerror(errno));
     return (RCODE_SOCKET_TABLE_FULL);
   }
   /* allow setting the type field in the IPX header */
   opt = 1;
-  if (setsockopt(sock, SOL_SOCKET, IPX_TYPE, &opt, sizeof(opt)) == -1) {
+  if (setsockopt(sock, SOL_IPX, IPX_TYPE, &opt, sizeof(opt)) == -1) {
     /* I can't think of anything else to return */
-    n_printf("IPX: could not set socket option for type.\n");
+    n_printf("IPX: could not set socket option for type: %s.\n", strerror(errno));
     return (RCODE_SOCKET_TABLE_FULL);
   }
   ipxs.sipx_family = AF_IPX;
   ipxs.sipx_network = *((unsigned int *)&MyAddress[0]);
 /*  ipxs.sipx_network = htonl(MyNetwork); */
-  bzero(ipxs.sipx_node, 6);	/* Please fill in my node name */
+  memset(ipxs.sipx_node, 0, 6);	/* Please fill in my node name */
   ipxs.sipx_port = htons(port);
 
   /* now bind to this port */
   if (bind(sock, (struct sockaddr *) &ipxs, sizeof(ipxs)) == -1) {
     /* I can't think of anything else to return */
-    n_printf("IPX: could not bind socket to address\n");
+    n_printf("IPX: could not bind socket to address: %s\n", strerror(errno));
     close( sock );
     return (RCODE_SOCKET_TABLE_FULL);
   }
@@ -467,7 +462,7 @@ IPXOpenSocket(u_short port, u_short * newPort)
     len = sizeof(ipxs2);
     if (getsockname(sock,(struct sockaddr *)&ipxs2,&len) < 0) {
       /* I can't think of anything else to return */
-      n_printf("IPX: could not get socket name in IPXOpenSocket\n");
+      n_printf("IPX: could not get socket name in IPXOpenSocket: %s\n", strerror(errno));
       close( sock );
       return(RCODE_SOCKET_TABLE_FULL);
     } else {
@@ -483,8 +478,7 @@ IPXOpenSocket(u_short port, u_short * newPort)
   return (RCODE_SUCCESS);
 }
 
-u_char
-IPXCloseSocket(u_short port)
+static u_char IPXCloseSocket(u_short port)
 {
   ipx_socket_t *mysock;
   far_t ECBPtr;
@@ -501,13 +495,15 @@ IPXCloseSocket(u_short port)
   n_printf("IPX: canceling all listen events on socket %x\n", port);
   ECBPtr = mysock->listenList;
   while (ECBPtr.segment | ECBPtr.offset) {
-    IPXCancelEvent(ECBPtr);
+    if (IPXCancelEvent(ECBPtr) != RCODE_SUCCESS)
+      return RCODE_CANNOT_CANCEL_EVENT;
     ECBPtr = mysock->listenList;
   }
   n_printf("IPX: canceling all AES events on socket %x\n", port);
   ECBPtr = mysock->AESList;
   while (ECBPtr.segment | ECBPtr.offset) {
-    IPXCancelEvent(ECBPtr);
+    if (IPXCancelEvent(ECBPtr) != RCODE_SUCCESS)
+      return RCODE_CANNOT_CANCEL_EVENT;
     ECBPtr = mysock->AESList;
   }
   /* now close the file descriptor for the socket, and free it */
@@ -518,8 +514,7 @@ IPXCloseSocket(u_short port)
   return (RCODE_SUCCESS);
 }
 
-int
-GatherFragmentData(char *buffer, ECB_t * ECB)
+static int GatherFragmentData(char *buffer, ECB_t * ECB)
 {
   int i;
   int nextFragLen, totalDataCount;
@@ -548,14 +543,13 @@ GatherFragmentData(char *buffer, ECB_t * ECB)
   }
   n_printf("IPX: gathered %d fragments for %d bytes of data\n",
 	   ECB->FragmentCount, totalDataCount);
-  if (d.network) {
+  if (debug_level('n')) {
     dumpBytes(buffer, totalDataCount);
   }
   return (totalDataCount);
 }
 
-void
-PrepareForESR(int iretFlag, ECB_t * ECB, far_t ECBPtr, u_char AXVal)
+static void PrepareForESR(int iretFlag, ECB_t * ECB, far_t ECBPtr, u_char AXVal)
 {
   unsigned char *ssp;
   unsigned long sp;
@@ -599,8 +593,7 @@ PrepareForESR(int iretFlag, ECB_t * ECB, far_t ECBPtr, u_char AXVal)
 	   _regs.es, _regs.esi, LWORD(eax), LWORD(eflags) & VIF);
 }
 
-void
-IPXSendPacket(far_t ECBPtr)
+static u_char IPXSendPacket(far_t ECBPtr)
 {
   ECB_t *ECB;
   IPXPacket_t *IPXHeader;
@@ -615,7 +608,7 @@ IPXSendPacket(far_t ECBPtr)
   if (dataLen == -1) {
     ECB->InUseFlag = IU_ECB_FREE;
     ECB->CompletionCode = CC_FRAGMENT_ERROR;
-    return;
+    return RCODE_ECB_NOT_IN_USE;	/* FIXME - what is to return here?? */;
   }
   IPXHeader = (IPXPacket_t *) ((ECB->FragTable[0].Address.segment << 4)
 			       + ECB->FragTable[0].Address.offset);
@@ -644,13 +637,14 @@ IPXSendPacket(far_t ECBPtr)
     /* DANG_FIXTHIS - should do a bind, send, close here */
     /* DOS IPX allows sending on unopened sockets */
     n_printf("IPX: send to unopened socket %04x\n", htons( ECB->ECBSocket ));
-    return;
+    return RCODE_SOCKET_NOT_OPEN;
   }
   if (sendto(mysock->fd, (void *) &data, dataLen, 0,
 	     (struct sockaddr *) &ipxs, sizeof(ipxs)) == -1) {
-    n_printf("IPX: error sending packet\n");
+    n_printf("IPX: error sending packet: %s\n", strerror(errno));
     ECB->InUseFlag = IU_ECB_FREE;
     ECB->CompletionCode = CC_HARDWARE_ERROR;
+    return RCODE_CANNOT_FIND_ROUTE;	/* FIXME - what is to return here?? */
   }
   else {
     ECB->InUseFlag = IU_ECB_FREE;
@@ -665,11 +659,10 @@ IPXSendPacket(far_t ECBPtr)
     PrepareForESR(0 /* not an iret, just a normal return */ ,
 		  ECB, ECBPtr, ESR_CALLOUT_IPX);
   }
-  return;
+  return RCODE_SUCCESS;
 }
 
-u_char
-IPXListenForPacket(far_t ECBPtr)
+static u_char IPXListenForPacket(far_t ECBPtr)
 {
   ECB_t *ECB;
   ipx_socket_t *mysock;
@@ -697,8 +690,7 @@ IPXListenForPacket(far_t ECBPtr)
   return (RCODE_SUCCESS);
 }
 
-u_char
-IPXScheduleEvent(far_t ECBPtr, u_char inUseCode, u_short delayTime)
+static u_char IPXScheduleEvent(far_t ECBPtr, u_char inUseCode, u_short delayTime)
 {
   ECB_t *ECB;
   ipx_socket_t *mysock;
@@ -728,8 +720,7 @@ IPXScheduleEvent(far_t ECBPtr, u_char inUseCode, u_short delayTime)
   return (RCODE_SUCCESS);
 }
 
-u_char
-IPXCancelEvent(far_t ECBPtr)
+static u_char IPXCancelEvent(far_t ECBPtr)
 {
   ECB_t *ECB;
   ipx_socket_t *mysock;
@@ -804,8 +795,7 @@ IPXCancelEvent(far_t ECBPtr)
   return (RCODE_CANNOT_CANCEL_EVENT);
 }
 
-void
-AESTimerTick(void)
+void AESTimerTick(void)
 {
   ipx_socket_t *mysock;
   far_t ECBPtr;
@@ -831,8 +821,7 @@ AESTimerTick(void)
   }
 }
 
-void
-ipx_fdset(fd_set * set)
+static void ipx_fdset(fd_set * set)
 {
   ipx_socket_t *s;
 
@@ -843,9 +832,8 @@ ipx_fdset(fd_set * set)
   }
 }
 
-int
-ScatterFragmentData(int size, char *buffer, ECB_t * ECB,
-		    struct sockaddr_ipx *sipx)
+static int ScatterFragmentData(int size, char *buffer, ECB_t * ECB,
+                               struct sockaddr_ipx *sipx)
 {
   int i;
   int nextFragLen, dataLeftCount;
@@ -853,7 +841,7 @@ ScatterFragmentData(int size, char *buffer, ECB_t * ECB,
   IPXPacket_t *IPXHeader;
 
   n_printf("received data bytes:\n");
-  if (d.network) {
+  if (debug_level('n')) {
     dumpBytes(buffer, size);
   }
   dataLeftCount = size;
@@ -892,6 +880,7 @@ ScatterFragmentData(int size, char *buffer, ECB_t * ECB,
       /* IPX stack returns data count including IPX header */
       /*			dataLeftCount -= 30; */
       memptr += 30;
+      printIPXHeader(IPXHeader);
     }
     if (nextFragLen > dataLeftCount) {
       nextFragLen = dataLeftCount;
@@ -908,8 +897,7 @@ ScatterFragmentData(int size, char *buffer, ECB_t * ECB,
   return (dataLeftCount);
 }
 
-int
-IPXReceivePacket(ipx_socket_t * s)
+static int IPXReceivePacket(ipx_socket_t * s)
 {
   struct sockaddr_ipx ipxs;
   char buffer[MAX_PACKET_DATA];
@@ -946,8 +934,7 @@ IPXReceivePacket(ipx_socket_t * s)
   return (ESRFired);
 }
 
-int
-IPXCheckForAESReady(void)
+static int IPXCheckForAESReady(void)
 {
   ipx_socket_t *s;
   far_t ECBPtr;
@@ -991,8 +978,7 @@ IPXCheckForAESReady(void)
   return (ESRFired);
 }
 
-int
-check_ipx_ready(fd_set * set)
+static int check_ipx_ready(fd_set * set)
 {
   ipx_socket_t *s;
   int ESRFired = 0;
@@ -1011,8 +997,7 @@ check_ipx_ready(fd_set * set)
   return (ESRFired);
 }
 
-void
-IPXRelinquishControl(void)
+static void IPXRelinquishControl(void)
 {
   /* DOS program has given us a time slice */
   /* let's use this as an opportunity to poll outstanding listens */
@@ -1042,7 +1027,7 @@ IPXRelinquishControl(void)
     break;
 
   case -1:			/* error (not EINTR) */
-    error("ERROR: bad ipc_select: %s\n", strerror(errno));
+    error("bad ipc_select: %s\n", strerror(errno));
     break;
 
   default:			/* has at least 1 descriptor ready */
@@ -1117,7 +1102,7 @@ IPXFarCallHandler(void)
     ECBPtr.offset = LWORD(esi);
     n_printf("IPX: send packet ECB at %x:%x\n",
 	     ECBPtr.segment, ECBPtr.offset);
-    IPXSendPacket(ECBPtr);
+    LO(ax) = IPXSendPacket(ECBPtr);
     break;
   case IPX_LISTEN_FOR_PACKET:
     ECBPtr.segment = _regs.es & 0xffff;
@@ -1206,18 +1191,30 @@ IPXFarCallHandler(void)
 void IPXEndCall(void) {
 	if(--IPXRunning < 0) IPXRunning = 0; /* Just incase */
     	n_printf("IPX: ESR Ended\n");
-	if(!IPXRunning) IPXCallRel(); /* Ask for more */
+	if(!IPXRunning) pic_request(PIC_IPX); /* Ask for more */
 	
 }
 
 void IPXCallRel(void) {
-  unsigned char *ssp;
-  unsigned long sp;
-  ssp = (unsigned char *)(REG(ss)<<4);
-  sp = (unsigned long) LWORD(esp);
-  pushw(ssp, sp, LWORD(cs));
-  pushw(ssp, sp, LWORD(eip));
-  LWORD(esp) -= 4;
+
+  if(in_dpmi && !in_dpmi_dos_int)
+    fake_pm_int();
+
+  /* push iret frame on _SS:_SP. At F000:2146 (bios.S) we get an
+   * iret and return to _CS:_IP */
+  fake_int(LWORD(cs), LWORD(eip));
+
+  /* push iret frame. At F000:20F7 (bios.S) we get an
+   * iret and return to F000:2140 for EOI */
+  fake_int(BIOSSEG, EOI_OFF);
+
+  /* push all 16-bit regs plus _DS,_ES. At F000:20F4 (bios.S) we find
+   * 'pop es; pop ds; popa' */
+  fake_pusha();
+
+  /* push return address F000:20F4 */
+  fake_call(BIOSSEG, POPA_IRET_OFF);	/* popa+iret */
+
   _regs.cs = ESRFarCall.segment;
   _regs.eip = ESRFarCall.offset;
 }
