@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 1999 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2004 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -30,37 +30,23 @@
  *
  */
 
-#include <features.h>
 #include "emu.h"
 #include "pktdrvr.h"
 #include "dosio.h"
 #include "memory.h"
 #include "int.h"
+#include "bios.h"
 #include <stddef.h>
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
 #include <sys/time.h>
-#if GLIBC_VERSION_CODE >= 2000
-  #if GLIBC_VERSION_CODE == 2000
-    #include <asm/byteorder.h>
-  #endif
-  #include <netinet/if_ether.h>
-#else
-  #include <asm/byteorder.h>
-  #include <linux/if_ether.h>
-#endif
+#include <netinet/in.h>
+#include <netinet/if_ether.h>
 #include "libpacket.h"
 #include "dosnet.h"
-#include "dosemu_select.h"
-
-/* flag to activate use of pic by packet driver */
-#if 0
-#define PICPKT 1
-#endif
-#ifdef PICPKT
 #include "pic.h"
-#endif
+#include "dpmi.h"
 
 #define min(a,b)	((a) < (b)? (a) : (b))
 
@@ -96,7 +82,6 @@ struct pkt_globs
     unsigned char hw_address[8];	/* our hardware address */
     short type;				/* our type */
     int flags;				/* configuration flags */
-    int helpvec;			/* vector number for helper */
     int nfds;				/* number of fd's for select() */
     fd_set sockset;			/* set of sockets for select() */
 
@@ -112,62 +97,84 @@ struct pkt_globs
     } handle[MAX_HANDLE];          
 } pg;
 
-static char devname[10] = "eth0";   /* linux device name */
-
-/* various declarations for interfacing with the packet driver code in
-   bios.S */
-
-extern void bios_f000();
-extern void PKTDRV_size();
-extern void PKTDRV_handle();
-extern void PKTDRV_receiver();
-extern void PKTDRV_buf();
-extern void PKTDRV_helper();
-extern void PKTDRV_signature();
-extern void PKTDRV_driver_name();
-extern void PKTDRV_param();
-extern void PKTDRV_stats();
+static char devname[10];		/* linux device name */
 
 /* creates a pointer into the BIOS from the asm exported labels */
 #define MK_PTR(ofs) ( (void *)((long)(ofs)-(long)bios_f000+(BIOSSEG << 4)) )
 
 /* calculates offset of a label from the start of the packet driver */
-#define MK_PKT_OFS(ofs) ((long)(ofs)-(long)PKTDRV_signature)
+#define MK_PKT_OFS(ofs) ((long)(ofs)-(long)PKTDRV_start)
 
-#define pkt_buf ((char *)MK_PTR(PKTDRV_buf))
+char pkt_buf[PKT_BUF_SIZE];
 
-short *p_helper_size;
-long *p_helper_receiver;
-short *p_helper_handle;
+short p_helper_size;
+long p_helper_receiver;
+short p_helper_handle;
 struct pkt_param *p_param;
 struct pkt_statistics *p_stats;
 
 /************************************************************************/
 
 /* initialize the packet driver interface (called at startup) */
-void
-pkt_init(int vec)
+void pkt_priv_init(void)
 {
     /* initialize the globals */
 
     pktdrvr_installed = 1; /* Will be cleared if some error occurs */
 
-    p_helper_size = MK_PTR(PKTDRV_size);
-    p_helper_receiver = MK_PTR(PKTDRV_receiver);
-    p_helper_handle = MK_PTR(PKTDRV_handle);
+    if (config.vnet == VNET_TYPE_DSN && Open_sockets() < 0) {
+	error("Cannot open dosnet device\n");
+        pktdrvr_installed = -1;
+    }
+}
+
+void
+pkt_init(int vec)
+{
+    if (pktdrvr_installed == -1)
+      goto fail;
     p_param = MK_PTR(PKTDRV_param);
     p_stats = MK_PTR(PKTDRV_stats);
 
     /* use dosnet device (dsn0) for virtual net */
-    if (config.vnet) {
+    pd_printf("PKT: VNET mode is %i\n", config.vnet);
+    switch (config.vnet) {
+      case VNET_TYPE_ETH:
+        strncpy(devname, config.netdev, sizeof(devname) - 1);
+        devname[sizeof(devname) - 1] = 0;
+	break;
+
+      case VNET_TYPE_DSN:
 	strcpy(devname, DOSNET_DEVICE);
-	if (Open_sockets() < 0)
-	    goto fail;
+	add_to_io_select(pkt_fd, 1, pkt_receive_async);
+	add_to_io_select(pkt_broadcast_fd, 1, pkt_receive_async);
+	break;
+
+      case VNET_TYPE_TAP:
+        if (strncmp(config.netdev, TAP_DEVICE, 3) == 0) {
+          pd_printf("PKT: trying to bind to device %s\n", config.netdev);
+          strcpy(devname, config.netdev);
+          pkt_fd = tun_alloc(devname);
+        }
+        if (pkt_fd < 0) {
+          strcpy(devname, TAP_DEVICE);
+          if ((pkt_fd = tun_alloc(devname)) < 0) {
+            error("Cannot allocate TAP device\n");
+            goto fail;
+          }
+        }
+	max_pkt_fd = pkt_fd + 1;
+	add_to_io_select(pkt_fd, 1, pkt_receive_async);
+	break;
+
+      default:
+        error("Unknown vnet mode %i\n", config.vnet);
+	goto fail;
     }
+    pd_printf("PKT: Using device %s\n", devname);
 
     /* hook the interrupt vector by pointing it into the magic table */
     SETIVEC(vec, PKTDRV_SEG, PKTDRV_OFF);
-    strcpy(MK_PTR(PKTDRV_signature), "PKT DRVR");
     
     /* fill other global data */
 
@@ -177,16 +184,14 @@ pkt_init(int vec)
     pg.type = 12;			/* dummy type (3c503) */
     pg.flags = config.pktflags;	/* global config flags */
 
-    p_param->major_rev = 9;		/* pkt driver spec */
-    p_param->minor_rev = 1;
+    p_param->major_rev = 1;		/* pkt driver spec */
+    p_param->minor_rev = 9;
     p_param->length = sizeof(struct pkt_param);
     p_param->addr_len = ETH_ALEN;
     p_param->mtu = GetDeviceMTU(devname);
     p_param->rcv_bufs = 8 - 1;		/* a guess */
     p_param->xmt_bufs = 2 - 1;
 
-    pg.helpvec = vec + 1;		/* use this for helper vec */
-    SETIVEC(pg.helpvec, BIOSSEG, (long)PKTDRV_helper-(long)bios_f000);
     return;
 
 fail:
@@ -204,12 +209,14 @@ pkt_int (void)
     if (!pktdrvr_installed)
 	return 0;
 
-#if 0
-    pd_printf("NPKT: AX=%04x BX=%04x CX=%04x DX=%04x FLAGS=%08x\n",
+#if 1
+  if (debug_level('P') > 8) {
+    pd_printf("NPKT: AX=%04x BX=%04x CX=%04x DX=%04x FLAGS=%08lx\n",
 		LWORD(eax),LWORD(ebx),LWORD(ecx),LWORD(edx),REG(eflags));
     pd_printf("      SI=%04x DI=%04x BP=%04x SP=%04x CS=%04x DS=%04x ES=%04x SS=%04x\n",
 		LWORD(esi),LWORD(edi),LWORD(ebp),LWORD(esp),
 		LWORD(cs),LWORD(ds),LWORD(es),LWORD(ss));
+  }
 #endif
 
     /* first clear CARRY flag that we may later set (on error) */
@@ -235,6 +242,14 @@ pkt_int (void)
     {
     case F_DRIVER_INFO:
         pd_printf("Driver info called ...\n");
+
+	if( LO(ax) != 255 ) {
+	   /* entry condition AL=255 not given
+	    *         Frank <molzahn@cc.umanitoba.ca>, 2000/02/18
+	    */
+	   HI(dx) = E_BAD_COMMAND;
+	   break;
+	}
 	REG(eax) = 2;				/* basic+extended functions */
 	REG(ebx) = 1;				/* version */
 
@@ -342,10 +357,10 @@ pkt_int (void)
 		    }
 		}
 	    }
-	    if (config.vnet)
+	    if (config.vnet) {
 		Insert_Type(free_handle, hdlp->packet_type_len, 
 			    hdlp->packet_type);
-	    else {
+	    } else {
 		hdlp->sock = OpenNetworkType(type); /* open the socket */
 
 		if (hdlp->sock < 0) {
@@ -355,7 +370,7 @@ pkt_int (void)
 		}
 
 		FD_SET(hdlp->sock, &pg.sockset); /* keep track of sockets */
-		add_to_io_select(hdlp->sock, 1);
+		add_to_io_select(hdlp->sock, 1, pkt_receive_async);
 		if (hdlp->sock >= pg.nfds)
 		    pg.nfds = hdlp->sock + 1;
 	    }
@@ -399,6 +414,8 @@ pkt_int (void)
 	p_stats->packets_out++;
 	p_stats->bytes_out += LWORD(ecx);
 
+	pd_printf("========Sending packet======\n");
+	printbuf("packet to send:", SEG_ADR((struct ethhdr *), ds, si)); 
 	for (handle = 0; handle < MAX_HANDLE; handle++) {
 	    hdlp = &pg.handle[handle];
 
@@ -433,13 +450,6 @@ pkt_int (void)
 				   SEG_ADR((char *), ds, si),
 				   LWORD(ecx)) >= 0) {
 		    pd_printf("Write to net was ok\n");
-		    /* check for a reply to speedup things 
-		       (background poll is quite slow) */
-#ifdef PICPKT
-		    pic_request(PIC_NET);
-#else
-		    pkt_check_receive(50000);
-#endif
 		    return 1;
 		} 
 		else {
@@ -602,6 +612,41 @@ Remove_Type(int handle)
     return 0;
 }
 
+void pkt_receiver_callback(void)
+{
+    struct vm86_regs saved_regs;
+
+    if (p_helper_size == 0)
+      return;
+
+    if(in_dpmi && !in_dpmi_dos_int)
+	fake_pm_int();
+
+    saved_regs = REGS;
+
+    _AX = 0;
+    _BX = p_helper_handle;
+    _CX = p_helper_size;
+    do_call_back(p_helper_receiver);
+    if (_ES == 0 && _DI == 0)
+      goto out;
+    MEMCPY_2DOS(SEGOFF2LINEAR(_ES, _DI), pkt_buf, p_helper_size);
+    _DS = _ES;
+    _SI = _DI;
+    _AX = 1;
+    _BX = p_helper_handle;
+    do_call_back(p_helper_receiver);
+
+out:
+    p_helper_size = 0;
+    REGS = saved_regs;
+}
+
+void pkt_receive_async(void)
+{
+  pic_request(PIC_NET);
+}
+
 void
 pkt_check_receive_quick(void)
 {
@@ -623,24 +668,6 @@ pkt_check_receive(int timeout)
 	return -1;
     }
 
-    /*
-    ** sometimes, dosemu enters this routine with the previous packet 
-    ** not sent by helper, and the packet driver hangs. 
-    ** This patch tryes to resend the packet, and make dosemu happy :)
-    **
-    ** Gloriano 
-    */
-    if (*p_helper_size != 0)			/* transfer area busy? */
-    {
-#ifdef PICPKT
-	do_irq();
-#else
-	run_int(pg.helpvec);
-#endif
-	pd_printf("Called the helpvector... (after failure)\n");
-	return 1;
-    }
-
     tv.tv_sec = 0;				/* set a (small) timeout */
     tv.tv_usec = timeout;
 
@@ -648,19 +675,20 @@ pkt_check_receive(int timeout)
     if (config.vnet) {
 	FD_ZERO(&readset);
 	FD_SET(pkt_fd, &readset);
-	FD_SET(pkt_broadcast_fd, &readset);
-	add_to_io_select(pkt_broadcast_fd, 1);
-	add_to_io_select(pkt_fd, 1);
-
+	if (config.vnet == VNET_TYPE_DSN) {
+	  FD_SET(pkt_broadcast_fd, &readset);
+	}
 	/* anything ready? */
 	if (select(max_pkt_fd,&readset,NULL,NULL,&tv) <= 0)
 	    return 0;
+
 	if(FD_ISSET(pkt_fd, &readset)) 
 	    fd = pkt_fd;
-	else if(FD_ISSET(pkt_broadcast_fd, &readset)) 
+	else if(config.vnet == VNET_TYPE_DSN && FD_ISSET(pkt_broadcast_fd, &readset)) 
 	    fd = pkt_broadcast_fd;
 	else return 0;
 
+	strcpy(device, devname);
 	size = ReadFromNetwork(fd, device, pkt_buf, PKT_BUF_SIZE);
 	if (size < 0) {
 	    p_stats->errors_in++;		/* select() somehow lied */
@@ -682,7 +710,7 @@ pkt_check_receive(int timeout)
 	    printbuf("received packet:", (struct ethhdr *)pkt_buf); 
 
             /* VINOD: If it is broadcast type, translate it back ... */
-	    if (memcmp(pkt_buf, DOSNET_BROADCAST_ADDRESS, 4) == 0) {
+	    if (config.vnet == VNET_TYPE_DSN && memcmp(pkt_buf, DOSNET_BROADCAST_ADDRESS, 4) == 0) {
 		pd_printf("It is a broadcast packet\n");
 		if(memcmp(pkt_buf + ETH_ALEN, pg.hw_address, ETH_ALEN) == 0) {
 		    /* Ignore our own ethernet broadcast. */
@@ -719,14 +747,12 @@ pkt_check_receive(int timeout)
 	    /* stuff things in global vars and queue a hardware */
 	    /* interrupt which will perform the upcall */
 
-	    *p_helper_size = size;
-	    *p_helper_receiver = hdlp->receiver;
-	    *p_helper_handle = handle;
-#ifdef PICPKT
-	    do_irq();
-#else
-	    run_int(pg.helpvec);
-#endif
+	    p_helper_size = size;
+	    p_helper_receiver = hdlp->receiver;
+	    p_helper_handle = handle;
+
+	    pkt_receiver_callback();
+
 	    pd_printf("Called the helpvector ... \n");
 	    return 1;
 	} else {
@@ -783,14 +809,12 @@ pkt_check_receive(int timeout)
 			
 		    /* stuff things in global vars and queue a hardware */
 		    /* interrupt which will perform the upcall */
-			*p_helper_size = size;
-			*p_helper_receiver = hdlp->receiver;
-			*p_helper_handle = handle;
-#ifdef PICPKT
-			do_irq();
-#else
-			run_int(pg.helpvec);
-#endif
+			p_helper_size = size;
+			p_helper_receiver = hdlp->receiver;
+			p_helper_handle = handle;
+
+			pkt_receiver_callback();
+
 			return 1;
 		    } else
 			p_stats->packets_lost++; /* not really lost... */
@@ -820,7 +844,7 @@ Find_Handle(u_char *buf)
 	p = buf + 2 * ETH_ALEN;		/* Ethernet-II */
     else 
 	p = buf + 2 * ETH_ALEN + 2;     /* All the rest frame types. */
-    pd_printf("Received packet type: %.2x\n", ntohs(eth->h_proto));
+    pd_printf("Received packet type: 0x%x\n", ntohs(eth->h_proto));
 
     for(i=0; i<max_pkt_type_array; i++) {
 	nchars = pkt_type_array[i].pkt_type_len;
@@ -833,10 +857,19 @@ Find_Handle(u_char *buf)
 
 static void
 printbuf(char *mesg, struct ethhdr *buf)
-{ int i;
+{
+int i;
+u_char *p;
   pd_printf( "%s :\n Dest.=", mesg);
   for (i=0;i<6;i++) pd_printf("%x:",buf->h_dest[i]);
-  pd_printf( " Source=") ;
+  pd_printf( " Source=");
   for (i=0;i<6;i++) pd_printf("%x:",buf->h_source[i]);
-  pd_printf( " Type= %x \n", buf->h_proto) ;
+  if (ntohs(buf->h_proto) >= 1536) {
+    p = (u_char *)buf + 2 * ETH_ALEN;		/* Ethernet-II */
+    pd_printf(" Ethernet-II;");
+  } else {
+    p = (u_char *)buf + 2 * ETH_ALEN + 2;     /* All the rest frame types. */
+    pd_printf(" 802.3;");
+  }
+  pd_printf( " Type= 0x%x \n", ntohs(*(u_short *)p));
 }
