@@ -203,15 +203,8 @@ boot(void)
 static inline void 
 vm86plus_init(void)
 {
-    static inline int vm86_old(struct vm86_struct* v86)
-    {
-        int __res;
-        __asm__ __volatile__("int $0x80\n"
-        :"=a" (__res):"a" ((int)113), "b" ((int)v86));
-        return __res;
-    }
     if (!vm86_plus(VM86_PLUS_INSTALL_CHECK,0)) return;
-    if (!vm86_old((void *)0xffffff01)) {
+    if (!syscall(SYS_vm86old, (void *)0xffffff01)) {
       fprintf(stderr, "your kernel contains an older (interim) vm86plus version\n\r"
       		      "please upgrade to an newer one\n\r");
     }
@@ -359,9 +352,9 @@ emulate(int argc, char **argv)
     cpu_setup();		/* setup the CPU */
     pcibios_init();
     pci_setup();
+    device_init();		/* initialize keyboard, disk, video, etc. */
     extra_port_init();		/* setup ports dependent on config */
     signal_init();              /* initialize sig's & sig handlers */
-    device_init();		/* initialize keyboard, disk, video, etc. */
     map_video_bios();           /* map the video bios */
     map_hardware_ram();         /* map the direct hardware ram */
     pkt_priv_init();
@@ -497,7 +490,7 @@ leavedos(int sig)
     /* try to notify dosdebug */
 #ifdef USE_MHPDBG
     if (fault_cnt > 0)
-      error("leavedos() called from within a signal context!\n");
+      dbug_printf("leavedos() called from within a signal context!\n");
     else
       mhp_exit_intercept(sig);
 #endif
