@@ -1,6 +1,6 @@
 /* 
  * All modifications in this file to the original code are
- * (C) Copyright 1992, ..., 2001 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2004 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -212,7 +212,7 @@ static void cirrusEnterLeave(char enter)
  * cirrusProbe -- 
  *      check up whether a cirrus based board is installed
  */
-static int cirrusProbe()
+static int cirrusProbe(void)
 {  
 /*     int cirrusClockNo; */
      unsigned char lockreg,IdentVal;
