@@ -1,6 +1,6 @@
 /* 
  * All modifications in this file to the original code are
- * (C) Copyright 1992, ..., 2001 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2004 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -160,4 +160,7 @@ union SRB16 {
 
 typedef union SRB16 SRB16;
 
+extern char *aspi_add_device(char *name, char *devtype, int);
+extern void aspi_helper(int);
+
 #endif
