@@ -60,22 +60,22 @@ extern int dpmi_eflags;  /* don't include 'dpmi.h' just for this! */
 #define _BP      (*(Bit16u *)&vm86s.regs.ebp)
 #define _SP      (*(Bit16u *)&vm86s.regs.esp)
 #define _IP      (*(Bit16u *)&vm86s.regs.eip)
-#define _EAX     ((Bit32u)(vm86s.regs.eax))
-#define _EBX     ((Bit32u)(vm86s.regs.ebx))
-#define _ECX     ((Bit32u)(vm86s.regs.ecx))
-#define _EDX     ((Bit32u)(vm86s.regs.edx))
-#define _ESI     ((Bit32u)(vm86s.regs.esi))
-#define _EDI     ((Bit32u)(vm86s.regs.edi))
-#define _EBP     ((Bit32u)(vm86s.regs.ebp))
-#define _ESP     ((Bit32u)(vm86s.regs.esp))
-#define _EIP     ((Bit32u)(vm86s.regs.eip))
-#define _CS      ((Bit16u)(vm86s.regs.cs))
-#define _DS      ((Bit16u)(vm86s.regs.ds))
-#define _SS      ((Bit16u)(vm86s.regs.ss))
-#define _ES      ((Bit16u)(vm86s.regs.es))
-#define _FS      ((Bit16u)(vm86s.regs.fs))
-#define _GS      ((Bit16u)(vm86s.regs.gs))
-#define _EFLAGS  ((Bit32u)(vm86s.regs.eflags))
+#define _EAX     (*(Bit32u *)&vm86s.regs.eax)
+#define _EBX     (*(Bit32u *)&vm86s.regs.ebx)
+#define _ECX     (*(Bit32u *)&vm86s.regs.ecx)
+#define _EDX     (*(Bit32u *)&vm86s.regs.edx)
+#define _ESI     (*(Bit32u *)&vm86s.regs.esi)
+#define _EDI     (*(Bit32u *)&vm86s.regs.edi)
+#define _EBP     (*(Bit32u *)&vm86s.regs.ebp)
+#define _ESP     (*(Bit32u *)&vm86s.regs.esp)
+#define _EIP     (*(Bit32u *)&vm86s.regs.eip)
+#define _CS      (*(Bit16u *)&vm86s.regs.cs)
+#define _DS      (*(Bit16u *)&vm86s.regs.ds)
+#define _SS      (*(Bit16u *)&vm86s.regs.ss)
+#define _ES      (*(Bit16u *)&vm86s.regs.es)
+#define _FS      (*(Bit16u *)&vm86s.regs.fs)
+#define _GS      (*(Bit16u *)&vm86s.regs.gs)
+#define _EFLAGS  (*(Bit32u *)&vm86s.regs.eflags)
 #define _FLAGS   (*(Bit16u *)&vm86s.regs.eflags)
 
 
