@@ -60,6 +60,7 @@ extern inline void irq_select(void);
 EXTERN struct vm86plus_struct vm86s INIT ( {
    {0},0,0,0,{{0}},{{0}}, {0}
 } );
+EXTERN struct sigcontext_struct _emu_stack_frame;
 
 EXTERN volatile sig_atomic_t signal_pending INIT(0);
 EXTERN volatile int fault_cnt INIT(0);
@@ -167,16 +168,11 @@ EXTERN struct ioctlq curi INIT({0, 0, 0, 0});
 
 #define SILENT_DOS_SYSCALL(sc) sc
 
-#if 0
+#if 1
 #define RPT_SYSCALL(sc) ({ int s_tmp, s_err; \
    do { \
 	  s_tmp = sc; \
 	  s_err = errno; \
-	  if (errno == EINTR) {\
-	    g_printf("Recursive run_irqs() RPT_SYSCALL()\n"); \
-	    handle_signals(); \
-	/*    run_irqs(); */ \
-	  } \
       } while ((s_tmp == -1) && (s_err == EINTR)); \
   s_tmp; })
 #else
@@ -314,7 +310,6 @@ typedef struct vesamode_type_struct {
        unsigned short detach;
        unsigned char *debugout;
        unsigned char *pre_stroke;  /* pointer to keyboard pre strokes */
-       unsigned char *pre_stroke_mem;
 
        /* Lock File business */
        boolean full_file_locks;
@@ -442,6 +437,9 @@ EXTERN void SIG_close(void);
 /* set if sigaltstack(2) is available */
 EXTERN int have_working_sigaltstack;
 
+EXTERN unsigned long int stack_init_top INIT(0xffffffff);
+EXTERN unsigned long int stack_init_bot INIT(0xffffffff);
+
 /* signals for Linux's process control of consoles */
 #define SIG_RELEASE     SIGWINCH
 #define SIG_ACQUIRE     SIGUSR2
@@ -508,14 +506,14 @@ do { \
 	sa.sa_handler = (SignalHandler)fun; \
 	sa.sa_flags = SA_RESTART; \
 	sigemptyset(&sa.sa_mask); \
-	sigaddset(&sa.sa_mask, SIGALRM); \
+	ADDSET_SIGNALS_THAT_QUEUE(&sa.sa_mask); \
 	sigaction(sig, &sa, NULL);
 
 #define NEWSETSIG(sig, fun) \
 	sa.sa_handler = (__sighandler_t) fun; \
-	sa.sa_flags = SA_RESTART; \
+	sa.sa_flags = SA_RESTART | SA_NODEFER; \
 	sigemptyset(&sa.sa_mask); \
-	sigaddset(&sa.sa_mask, SIGALRM); \
+	ADDSET_SIGNALS_THAT_QUEUE(&sa.sa_mask); \
 	if (have_working_sigaltstack) { \
 		sa.sa_flags |= SA_ONSTACK; \
 		sigaction(sig, &sa, NULL); \
