@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 2001 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2004 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -30,6 +30,14 @@ struct printer {
 
 };
 
+/* public functions */
+int printer_open(int prnum);
+int printer_close(int prnum);
+int printer_flush(int prnum);
+int printer_write(int prnum, int outchar);
+void printer_config(int prnum, struct printer *pptr);
+void printer_print_config(int prnum, void (*print)(char *, ...));
+
 /* status bits */
 #define LPT_TIMEOUT	0x1
 #define LPT_IOERR	0x8
