@@ -1,6 +1,6 @@
 /* 
  * All modifications in this file to the original code are
- * (C) Copyright 1992, ..., 1999 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2004 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -34,7 +34,7 @@
 #ifndef	_BASE_H
 #define	_BASE_H
 
-#if !defined(__linux__) && !defined(__NetBSD__)
+#if !defined(__linux__)
 
 #include <mach.h>
 #include <mach_error.h>
@@ -185,6 +185,12 @@ typedef int onoff_t;
  *
  * HISTORY:
  * $Log: machcompat.h,v $
+ * Revision 1.1.1.1.4.1  2004/01/16 21:48:27  bartoldeman
+ * Happy new year!
+ *
+ * Revision 1.1.1.1  2003/06/23 00:02:10  bartoldeman
+ * Initial import (dosemu-1.1.5.2).
+ *
  * Revision 2.4  1995/04/08  22:35:19  root
  * Release dosemu0.60.0
  *
@@ -326,7 +332,7 @@ typedef int onoff_t;
 #define VGA_ENABLED		1
 #define VGA_DISABLED		2
 
-#if !defined(__linux__) && !defined(__NetBSD__)
+#if !defined(__linux__)
 #include <mach/thread_status.h>
 #include <machine/psl.h>
 #include <i386/pio.h>
@@ -395,17 +401,12 @@ typedef struct i386_thread_state state_t;
 #include "cpu.h"
 #endif
 
-#ifdef __NetBSD__
-typedef struct sigcontext state_t;
-#endif
 #ifdef __linux__
 typedef struct vm86_regs state_t;
 #endif
 typedef int boolean_t;
 typedef caddr_t vm_address_t;
-#ifndef __NetBSD__
 typedef size_t vm_size_t;
-#endif
 
 
 /* this is used for memory objects (pointer to malloc()ed memory */
@@ -432,7 +433,7 @@ typedef size_t vm_size_t;
 
 EXTERN FILE *dbg_fd INIT(NULL);
 
-#if !defined(__linux__) && !defined(__NetBSD__)
+#if !defined(__linux__)
 
 #define MAX_IO_PORTS	128
 extern u_char io_ports[MAX_IO_PORTS];
