@@ -1,6 +1,6 @@
 /* 
  * All modifications in this file to the original code are
- * (C) Copyright 1992, ..., 2001 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2004 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -65,8 +65,6 @@
 #define PIC_IMOUSE 17      /*  internal mouse driver       */
 #define PIC_IPX    18      /*  IPX Signal */
 
-#define PIC_IRQALL 0xfffe  /*  bits for all IRQs set. This never changes  */
-
 /* pic_irq_list translates irq numbers to pic_ilevels.  This is not used
    by pic routines; it is simply made available for configuration ease */
 EXTERN unsigned long pic_irq_list[] INIT({PIC_IRQ0,  PIC_IRQ1,  PIC_IRQ9,  PIC_IRQ3,
@@ -85,29 +83,33 @@ EXTERN unsigned long pic_level_list[] INIT({
 
 EXTERN unsigned long pic_irr;          /* interrupt request register */
 EXTERN unsigned long pic_isr;          /* interrupt in-service register */
-EXTERN unsigned long pic_iflag;        /* interrupt enable flag: en-/dis- =0/0xfffe */
-EXTERN unsigned long pic_icount;       /* iret counter (to avoid filling stack) */
-EXTERN unsigned long pic_ilevel INIT(32);    /* current interrupt level */
+EXTERN unsigned int pic_iflag;        /* interrupt enable flag: en-/dis- =0/0xfffe */
+EXTERN unsigned int pic_icount;       /* iret counter (to avoid filling stack) */
+EXTERN unsigned long pic_irqall INIT(0xfffe);       /* bits for all IRQs set. */
+EXTERN unsigned int pic_ilevel INIT(32);    /* current interrupt level */
 
 EXTERN unsigned long pic0_imr INIT(0xf800);  /* interrupt mask register, pic0 */
 EXTERN unsigned long pic1_imr INIT(0x07f8);         /* interrupt mask register, pic1 */
 EXTERN unsigned long pic_imr INIT(0xfff8);          /* interrupt mask register */
 EXTERN unsigned long pice_imr INIT(-1);         /* interrupt mask register, dos emulator */  
-EXTERN unsigned long pic_stack[32];     /* list of active irqd */
-EXTERN unsigned long pic_sp INIT(0);	       /* pointer to pic_stack */ 
-EXTERN unsigned long pic_rflag;        /* flag to control pic_watch */
-EXTERN unsigned long pic_vm86_count INIT(0);   /* count of times 'round the vm86 loop*/
-EXTERN unsigned long pic_dpmi_count INIT(0);   /* count of times 'round the dpmi loop*/
+EXTERN unsigned int pic_stack[32];     /* list of active irqd */
+EXTERN unsigned int pic_sp INIT(0);	       /* pointer to pic_stack */ 
+EXTERN unsigned int pic_rflag;        /* flag to control pic_watch */
+EXTERN unsigned int pic_vm86_count INIT(0);   /* count of times 'round the vm86 loop*/
+EXTERN unsigned int pic_dpmi_count INIT(0);   /* count of times 'round the dpmi loop*/
 
 EXTERN hitimer_t pic_dos_time;     /* dos time of last interrupt,1193047/sec.*/
 EXTERN hitimer_t pic_sys_time INIT(NEVER);     /* system time set by pic_watch */
+extern hitimer_t pic_itime[33];
+
 /* IRQ definitions.  Each entry specifies the emulator routine to call, and
    the dos interrupt vector to use.  pic_iinfo.func is set by pic_seti(),
    as are the interrupt vectors for all but [1] through [15], which  may be
    changed by the ICW2 command from dos. (Some dos extenders do this.) */
    
 struct lvldef {
-       void (*func)();
+       void (*func)(void);
+       void (*callback)(void);
        int    ivec;
        };
 
@@ -120,28 +122,35 @@ Bit8u read_pic0(ioport_t port);             /* read from PIC 0 */
 Bit8u read_pic1(ioport_t port);             /* read from PIC 1 */
 void pic_unmaski(int level);                 /* clear dosemu's irq mask bit */
 void pic_maski(int level);                   /*  set  dosemu's irq mask bit */
-void pic_seti(unsigned int level, void (*func), unsigned int ivec); 
+void pic_seti(unsigned int, void (*), unsigned int, void (*)); 
                                        /* set function and interrupt vector */
-void run_irqs();                                      /* run requested irqs */
+void run_irqs(void);                                  /* run requested irqs */
 #define pic_run() if(pic_irr)run_irqs()   /* the right way to call run_irqs */
-int do_irq();                                /* run dos portion of irq code */
+int do_irq(void);                            /* run dos portion of irq code */
 
 #define PIC_REQ_NOP	0
 #define PIC_REQ_OK	1
 #define PIC_REQ_LOST	(-2)
 #define PIC_REQ_PEND	(-1)
 int pic_request(int inum);                            /* interrupt trigger */
+void pic_untrigger(int inum);                          /* interrupt untrigger */
 
-void pic_iret();                             /* interrupt completion notify */
-void pic_watch();		       /* interrupt pending watchdog timer */
-void do_irq0();						 /* timer interrupt */
-int  pic_pending();		   /* inform caller if interrupt is pending */
+void pic_set_callback(Bit16u cs, Bit16u ip);
+
+void pic_iret(void);                         /* interrupt completion notify */
+void pic_resched(void);
+inline void pic_watch(hitimer_u *s_time);    /* interrupt pending watchdog timer */
+void do_irq0(void);				/* timer interrupt */
+int pic_pending(int ilevel);			/* inform caller if interrupt is pending */
 void pic_sched(int ilevel, int interval);          /* schedule an interrupt */
 /* The following are too simple to be anything but in-line */
 
 #define pic_set_mask pic_imr=(pic0_imr|pic1_imr|pice_imr|pic_iflag)
 #define pic_sti() (pic_iflag=0,pic_set_mask, (void)0)          /*    emulate STI      */
-#define pic_cli() (pic_iflag=PIC_IRQALL,pic_set_mask, (void)0) /*    emulate CLI      */
+#define pic_cli() (pic_iflag=pic_irqall,pic_set_mask, (void)0) /*    emulate CLI      */
+
+#define CAN_SLEEP() (!(pic_icount || pic_isr || (REG(eflags) & VIP) || \
+    (pic_sys_time > pic_dos_time)))
 
 /* Experimental TIMER-IRQ CHAIN code */
 extern void timer_int_engine(void);
