@@ -1,6 +1,6 @@
 /* 
  * All modifications in this file to the original code are
- * (C) Copyright 1992, ..., 2001 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2004 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -45,6 +45,7 @@
 #define RX_BUFFER_SIZE            128
 #define TX_BUFFER_SIZE            64
 
+extern int no_local_video; /* used by virtual port code */
 EXTERN u_char irq_source_num[255];	/* Index to map from IRQ no. to serial port */
 EXTERN u_char com_port_used[17];       /* Used for auto-assign comport config */
 
@@ -56,6 +57,7 @@ typedef struct serial_struct {
   ioport_t base_port;		/* Base port address handled by device */
   int interrupt;		/* IRQ line handled by device */
   boolean virtual;		/* virtual modem */
+  boolean pseudo;		/* pseudo-tty is used */
   boolean mouse;		/* Flag to turn on mouse sharing features */
   boolean dev_locked;           /* Flag to indicate that device is locked */
   boolean fossil_active;	/* Flag: FOSSIL emulation active */
@@ -120,10 +122,12 @@ typedef struct serial_struct {
 
 #define MAX_SER 4
 EXTERN serial_t com[MAX_SER];
-extern void int14(u_char);
+extern int int14(void);
 extern inline void serial_run(void);
 extern inline int do_serial_in(int, ioport_t);
 extern inline int do_serial_out(int, ioport_t, int);
-extern void serial_helper();
+extern void serial_helper(void);
+extern void child_close_mouse(void);
+extern void child_open_mouse(void);
 
 #endif /* SERIAL_H */
