@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 2001 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2004 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -36,7 +36,7 @@ struct EMM {
 
 struct Handle {
   unsigned short int num;
-  unsigned char *addr;
+  char *addr;
   unsigned long size;
   int valid;
   int lockcount;
@@ -48,5 +48,6 @@ struct UMB {
 };
 
 void xms_control(void), xms_int15(void), xms_init(void);
+extern int a20;
 
 #endif /* XMS_H */
