@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 2001 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2004 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -9,15 +9,8 @@
  *
  * It should contain a valid call to the init function of the plug-in such as
  *
- *    {
- *       extern void my_plugin_init(void);
  *       my_plugin_init();
- *    }
  * 
- * Don't forget the curly brackets around your statement.
  */
 
-{
-	extern void demo_plugin_init(void);
-	demo_plugin_init();
-}
+demo_plugin_init();
