@@ -1,5 +1,5 @@
 /* 
- * (C) Copyright 1992, ..., 2001 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2004 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -16,16 +16,7 @@
 
 #include <stdlib.h>
 #include <stdio.h>
-#include <features.h>
-#if defined (__GLIBC__) && __GLIBC__ >= 2
-  #define __need_timeval
-  #if __GLIBC__== 2 && __GLIBC_MINOR__ < 1
-    #include <timebits.h>
-    #include <sys/time.h>
-  #else
-    #include <bits/time.h>
-  #endif
-#endif
+#include <sys/time.h>
 #include <time.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -36,11 +27,9 @@
 #include <string.h>
 #include <signal.h>
 
-#include "Linux/vt.h" 
+#include <sys/vt.h>
 #include <sys/ioctl.h>
 
-#include "dosemu_select.h"
-
 static void usage(void)
 {
   printf(
@@ -92,7 +81,7 @@ static int switch_console(char new_console)
 }
 
 
-void handle_console_input(void)
+static void handle_console_input(void)
 {
   char buf[UHOOK_BUFFERSIZE];
   static char sbuf[UHOOK_BUFFERSIZE]="\n";
@@ -116,7 +105,7 @@ void handle_console_input(void)
 }
 
 
-void handle_uhook_input(void)
+static void handle_uhook_input(void)
 {
   char buf[UHOOK_BUFFERSIZE];
   int n;
