@@ -1,6 +1,6 @@
 /* 
  * All modifications in this file to the original code are
- * (C) Copyright 1992, ..., 1999 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2004 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -220,6 +220,9 @@ static void usage(void)
 int main(int argc, char *argv[])
 {
   int n, m;
+  char boot_sect[] = {
+  #include "bootsect.h"
+  };
   
   /* Parse command line. */
   if ((argc <= 1) && isatty(STDOUT_FILENO))
@@ -287,15 +290,8 @@ int main(int argc, char *argv[])
 
   /* Write partition boot sector. */
   clear_buffer();
-  buffer[0] = 0xeb;                     /* Jump to dosemu exit code. */
-  buffer[1] = 0x3c;                     /* (jmp 62; nop) */
-  buffer[2] = 0x90; 
-  buffer[62] = 0xb8;                    /* Exit dosemu. */
-  buffer[63] = 0xff;                    /* (mov ax,0xffff; int 0xe6) */
-  buffer[64] = 0xff;
-  buffer[65] = 0xcd;
-  buffer[66] = 0xe6;
-  memmove(buffer + 3, "DOSEMU  ", 8);
+  memcpy(buffer, boot_sect, sizeof(boot_sect));
+
   put_word(&buffer[11], BYTES_PER_SECTOR);
   buffer[13] = SECTORS_PER_CLUSTER;
   put_word(&buffer[14], RESERVED_SECTORS);
