@@ -1,6 +1,6 @@
 /* 
  * All modifications in this file to the original code are
- * (C) Copyright 1992, ..., 2001 the "DOSEMU-Development-Team".
+ * (C) Copyright 1992, ..., 2004 the "DOSEMU-Development-Team".
  *
  * for details see file COPYING in the DOSEMU distribution
  */
@@ -26,7 +26,6 @@
  * - There's no real boot sector code, just "dosemu exit" call.
  * 
  */
-#include <features.h>
 #include <ctype.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -35,9 +34,6 @@
 #include <sys/stat.h>
 #include <time.h>
 #include <unistd.h>
-#if GLIBC_VERSION_CODE >= 2000
-#include <errno.h>
-#endif
 
 /* These can be changed -- at least in theory. In practise, it doesn't
  * seem to work very well (I don't know why). 
@@ -265,9 +261,12 @@ static void usage(void)
 int main(int argc, char *argv[])
 {
   int n, m;
+  char boot_sect[] = {
+  #include "bootsect.h"
+  };
   int kbytes = -1;
   outfile = stdout;
-  
+
   /* Parse command line. */
   if ((argc <= 1) && isatty(STDOUT_FILENO))
   {
@@ -327,7 +326,7 @@ int main(int argc, char *argv[])
       break;
     case 'f':
       if ((outfile=fopen(optarg, "w")) == 0) {
-        fprintf(stderr, "Error: cannot open file %s: %s\n", optarg, sys_errlist[errno]);
+        fprintf(stderr, "Error: cannot open file %s: %s\n", optarg, strerror(errno));
         exit(1);
       }
       break;
@@ -422,18 +421,9 @@ int main(int argc, char *argv[])
       fclose(f);
       memset(buffer+11, 0, 51);
     }
-  }
-  if (!bootsect_file) {
+  } else {
     clear_buffer();
-    buffer[0] = 0xeb;                     /* Jump to dosemu exit code. */
-    buffer[1] = 0x3c;                     /* (jmp 62; nop) */
-    buffer[2] = 0x90; 
-    buffer[62] = 0xb8;                    /* Exit dosemu. */
-    buffer[63] = 0xff;                    /* (mov ax,0xffff; int 0xe6) */
-    buffer[64] = 0xff;
-    buffer[65] = 0xcd;
-    buffer[66] = 0xe6;
-    memmove(buffer + 3, "DOSEMU  ", 8);
+    memcpy(buffer, boot_sect, sizeof(boot_sect));
   }
   put_word(&buffer[11], BYTES_PER_SECTOR);
   buffer[13] = sectors_per_cluster;
